	TITLE	'MHBED -- MINISCRIBE MS-3012 BACKEND DRIVER FOR CPM 2.2'

;	04-18-83




;   MHBED is a Hard Disk only backend driver for any CPM based system
;   This program is designed to work with a MiniScribe MS-3012 Hard
;   Disk Drive. The MS-3012 is configured as two logical 5 megabyte
;   drives. Read the file "BED.DOC" for information regarding usage
;   of this program.






;	---PROGRAM CONFIGURATION---
;
;
;   THE FOLLOWING STATMENTS ARE USED TO CONFIGURE THE PROGRAM.
;
;	1. Z80 -- SET THIS "TRUE" IF YOU ARE USING A Z-80 MICROPROCESSOR.
;	   THIS WILL SPEED UP DATA TRANSFERS TO/FROM THE CONTROLLER
;	   BUFFER.
;
;	2. S100 -- SET THIS "TRUE" IF YOU ARE USING A CONTROLLER WITH
;	   AN S-100 BUS INTERFACE (STS). SET THIS "FALSE" IF YOU USING
;	   A CONTROLLER WITH A RIBBON CABLE INTERFACE (STR).
;
;
;
FALSE	EQU	0		;DEFINE FALSE
TRUE	EQU	NOT FALSE	;DEFINE TRUE
;
Z80	EQU	TRUE		;WE ARE USING A Z-80
;
S100	EQU	TRUE		;S-100 CONTROLLER






;	---EQUATES---
;
;
STRT	EQU	62*1024		;PROGRAM START ADDRESS
OFFSET	EQU	400H-STRT	;DDT OFFSET VALUE
WBOOT	EQU	0		;WARM BOOT
RPT	EQU	80H		;REPEAT
INDEX	EQU	10H		;INDEX BIT
VSA	EQU	8		;SEEK VERIFY START ADDRESS
VCA	EQU	1BH		;SEEK VERIFY COMPARE ADDRESS
LMSK	EQU	0FFH		;LSB MASK
;
MAXHD	EQU	2		;NUMBER OF LOGICAL DISK DRIVES
MAXCYL	EQU	612/2		;NUMBER OF LOGICAL CYLINDERS/DRIVE
LZONE	EQU	656		;LANDING ZONE
HEDMSK	EQU	1		;HEAD MASK
MAXSEC	EQU	32		;NUMBER OF SECTORS PER TRACK
;
;   HARD DISK/CPM INTERFACE
;
BLKSIZ	EQU	2048		;NUMBER OF BYTES PER BLOCK
CPMSPT	EQU	2*MAXSEC	;NUMBER OF CPM SECTORS PER TRACK
;
;   BIT FLAGS
;
F@RDOP	EQU	1		;READ OPERATION
F@RIP	EQU	2		;READ IN PROGRESS
F@WIP	EQU	4		;WRITE IN PROGRESS
;
;   DRIVE COMMANDS
;
SKOUT	EQU	1		;SEEK DIRECTION = OUTWARD
SKCMD	EQU	3		;SEEK
NOPC	EQU	40H		;NO PRE-COMPENSATION
LOWRT	EQU	80H		;LOW WRITE CURRENT
;
;   DRIVE STATUS
;
READY	EQU	1		;DRIVE READY
WRTFLT	EQU	2		;WRITE FAULT
TK00	EQU	4		;TRACK ZERO
RAWINDX	EQU	20H		;RAW INDEX
;
;   CONTROLLER COMMANDS
;
BANK0	EQU	0		;BANK 0 SELECT
BANK1	EQU	1		;BANK 1 SELECT
DBENB	EQU	2		;DATA BUFFER ENABLE
CBENB	EQU	4		;COMPARE BUFFER ENABLE
START	EQU	8		;START COMMAND
;
;   CONTROLLER STATUS
;
INDEX	EQU	10H		;DELAYED INDEX
SECTOR	EQU	20H		;SECTOR
;
;   DRIVE/CONTROLLER I/O
;
	IF	S100	;IF S-100 BUS
CBASE	EQU	70H		;BASE ADR OF THE CONTROLLER
DRCSR	EQU	CBASE		;DRIVE COMMAND/STATUS
EXTCMD	EQU	CBASE+1		;EXTENDED COMMNAND REGISTER
LOSC	EQU	CBASE+2		;SEEK COUNT, LSB
HISC	EQU	CBASE+3		;  * MSB
CTCSR	EQU	CBASE+4		;CONTROLLER COMMAND/STATUS
CTBFR	EQU	CBASE+5		;CONTROLLER BUFFER ADDRESS
CTDP	EQU	CBASE+6		;CONTROLLER DATA PORT
	ELSE		;IF RIBBON CABLE INTERFACE
CBASE	EQU	80H		;BASE ADR OF THE CONTROLLER
DRCSR	EQU	CBASE+7		;DRIVE COMMAND/STATUS
EXTCMD	EQU	CBASE+6		;EXTENDED COMMNAND REGISTER
LOSC	EQU	CBASE+5		;SEEK COUNT, LSB
HISC	EQU	CBASE+4		;  * MSB
CTCSR	EQU	CBASE+3		;CONTROLLER COMMAND/STATUS
CTBFR	EQU	CBASE+2		;CONTROLLER BUFFER ADDRESS
CTDP	EQU	CBASE+1		;CONTROLLER DATA PORT
	ENDIF
;
;   ASCII CHARACTERS
;
CR	EQU	0DH		;CARRIAGE RETURN
LF	EQU	0AH		;LINE FEED

	PAGE


;	============
;	** MACROS **
;	============




;	---SET FLAG BIT---
;
;
SETBIT	MACRO	BIT
	MVI	A,BIT		;; A = BIT TO SET
	CALL	@BSET		;; SET THE BIT
	ENDM




;	---RESET FLAG BIT---
;
;
CLRBIT	MACRO	BIT
?Y	SET	NOT BIT
	MVI	A,?Y		;; A = CLEAR MASK
	CALL	@BRES		;; RESET THE BIT
	ENDM




;	---TEST A FLAG BIT---
;
;
TSTBIT	MACRO	BIT
	MVI	A,BIT		;; A = BIT UNDER TEST
	CALL	@BTST		;; TEST THE BIT
	ENDM




;	---Z80 BLOCK INPUT---
;
;
INIR	MACRO
	DB	0EDH,0B2H
	ENDM




;	---Z80 BLOCK OUTPUT---
;
;
OUTIR	MACRO
	DB	0EDH,0B3H
	ENDM

	PAGE

	ORG	STRT


;	============================
;	** ENTRY POINT JUMP TABLE **
;	============================
;
;
	JMP	BOOT		;  0 - COLD BOOT
DLA:	JMP	HOME		;  1 - TRACK ZERO SEEK
	JMP	SETDR		;  2 - SET DRIVE #
	JMP	SETTK		;  3 - SET TRACK ADR
	JMP	SETSEC		;  4 - SET SECTOR ADR
	JMP	SETDMA		;  5 - SET BUFFER ADDRESS
	JMP	READ		;  6 - READ A SECTOR
	JMP	WRITE		;  7 - WRITE A SECTOR
LOT:	JMP	$		;  8 - LIST OUTPUT READY TEST
	JMP	SXR		;  9 - SECTOR XLATE ROUTINE
;
;   SIGN-ON MESSAGE
;
HELLO:	DB	CR,LF,LF
	DB	'MHBED -- Vers 1.0',CR,LF
	DB	'18-Apr-83',CR,LF,'$'

	PAGE



;	======================
;	** DISK DEFINITIONS **
;	======================






;	---DISK PARAMETERS---
;
;
DPHDR:	DW	0,0		;HARD DISK, LOGICAL DRIVE 0
	DW	0,0
	DW	DIRBFR,HDBLK
	DW	LAST,HALL0
;
	DW	0,0		;HARD DISK, LOGICAL DRIVE 1
	DW	0,0
	DW	DIRBFR,HDBLK
	DW	LAST,HALL1






;	---HARD DISK PARAMETER BLOCK---
;
;
;   THIS TABLE DEFINES HARD DISK CONFIGURATION.
;
;	1. THE MS-3012 IS CONFIGURED AS TWO LOGICAL 5 MEGABYTE DRIVES.
;
;	2. THE CPM LOGICAL BLOCK SIZE IS 2048 BYTES.
;
;	3. THERE ARE 1024 DIRECTORY ENTRIES PER LOGICAL DRIVE.
;
;	4. THE FIRST TWO TRACKS ARE RESERVED FOR THE SYSTEM. THIS WILL
;	   ALLOW AN AGGREGATE CCP/BDOS/CBIOS SIZE OF 16 K-BYTES.
;
;
;
HDBLK:	DW	2*MAXSEC	;SPT: SECTORS PER TRACK
	DB	4		;BSH: BLOCK SHIFT FACTOR
	DB	15		;BLM: BLOCK MASK
	DB	0		;EXM: NULL MASK
	DW	2439		;DSM: (DISK SIZE) - 1
	DW	1023		;DRM: (DIR SIZE) - 1
	DB	0FFH		;AL0: DIRECTORY ALLOCATION BITS, MSB
	DB	0FFH		;AL1:   * LSB
	DW	0		;CKS: CHECK SIZE
	DW	2		;OFF: TRACK OFFSET

	PAGE


;	=====================
;	** CBIOS FUNCTIONS **
;	=====================




;	---COLD BOOT---
;
;
BOOT:	STA	FAD		;1ST AVAIL DRIVE NR
	LHLD	1		;WARM BOOT LINK ADR
	LXI	D,7*3
	DAD	D		;CALC ADR OF CBIOS F/D LINKS
	PUSH	H		;..SAVE IT
	LXI	D,X@TKZ
	CALL	MLNK		;GET/SAVE LINKS TO F/D DRIVER
	LHLD	X@LOT+1		;GET ADR OF LIST OUT TEST
	SHLD	LOT+1		;UPDATE MY OWN POINTER
	LXI	H,DLA		;DRIVER LINKS ADRS
	POP	D		;D/E = CBIOS F/D LINK ADRS
	CALL	MLNK		;OVERLAY CBIOS LINKS
; INITIALIZE STORAGE
	LXI	H,FLAGS		;START OF VARIABLES
	MVI	B,FLGSIZ	;NR OF VARIABLES
	XRA	A
BT1:	MOV	M,A
	INX	H
	DCR	B
	JNZ	BT1
	JMP	WBOOT		;EXIT, WARM BOOT SYSTEM




;	---SECTOR TRANSLATE ROUTINE---
;
;
SXR:	CALL	DRTST		;DRIVE TEST
	JC	X@SXR		;JIF DO F/D XLATE
	XCHG			;H/L = ADR OF XLATE TABLE
	MOV	A,L
	ORA	H		;TEST TABLE ADR
	JNZ	SXR1		;JIF DO XLATE
	DAD	B		;DON'T XLATE -- USE SECT ADR AS IS
	RET
;
SXR1:	DAD	B		;INDEX INTO THE TABLE
	MOV	L,M		;L = XLATED SECTOR ADR
	MVI	H,0		;H = 0 FOR 16-BIT VALUE
	RET




;	---HOME---
;
;
HOME:	CALL	DRTST		;DRIVE TEST
	JC	X@TKZ		;JIF DO F/D HOME
	CPI	MAXHD
	JNC	SEF		;JIF INVALID DRIVE #
	LXI	H,DOTKZ
	MOV	A,M		;GET FORCE TKZ FLAG
	MVI	M,1		;SET FLAG FOR NEXT TIME
	ORA	A
	JNZ	XSTZ		;JIF SEEK TO ZERO
	LXI	H,0
	SHLD	R@TRK		;SET REQ TRACK = ZERO
	JMP	XTKZ		;DO TKZ 1ST TIME
;
;   SET ERROR FLAG
;
SEF:	MVI	A,1		;A = ERROR CODE
	ORA	A		;SET 8080/Z80 FLAGS
	RET




;	---READ---
;
;
READ:	CALL	DRTST		;DRIVE TEST
	JC	X@RD		;JIF DO F/D READ
	CPI	MAXHD
	JNC	SEF		;JIF INVALID DRIVE NR
	JMP	HDREAD		;READ FROM H/D




;	---WRITE---
;
;
WRITE:	CALL	DRTST		;DRIVE TEST
	JC	X@WRT		;JIF DO F/D WRITE
	CPI	MAXHD
	JNC	SEF		;JIF INVALID DRIVE NR
	JMP	HDWRT		;WRITE TO H/D




;	---SET DRIVE NUMBER---
;
;
SETDR:	MOV	A,C		;A = NEW DRIVE #
	STA	R@DSK		;..SAVE IT
	CALL	DRTST		;DRIVE TEST
	JC	X@SEL		;DO F/D SLCT
	LXI	H,0
	CPI	MAXHD
	RNC			;RIF INVALID DRIVE #
	MOV	L,A
	DAD	H		;SHIFT H/L LEFT 4
	DAD	H		;   (HL = HL*16)
	DAD	H
	DAD	H
	LXI	D,DPHDR		;DISK PARM HDR
	DAD	D		;H/L = DPHDR FOR SELECTED DRIVE
	RET




;	---SET SECTOR ADR---
;
;
SETSEC:	MOV	A,C
	STA	R@SEC		;SAVE SECTOR ADR
; SET REAL SECTOR ADR FOR THE H/D DRIVER
	ANI	NOT 1		;CLEAR LOW ORDER BIT
	RRC			;THERE ARE 2 CPM SECTORS PER H/D SECT
	STA	RSA		;SAVE REAL SECTOR ADR
	JMP	X@SEC		;PASS INFO TO F/D DRIVER, EXIT





;	---SET TRACK ADDRESS---
;
;
SETTK:	CALL	DRTST		;DRIVE TEST
	JC	X@TK		;JIF F/D
	MOV	H,B
	MOV	L,C
	SHLD	R@TRK		;SAVE TRACK ADR
	RET




;	---SET DMA ADDRESS---
;
;
SETDMA:	MOV	H,B
	MOV	L,C
	SHLD	DMADR		;SAVE DMA ADR
	JMP	X@DMA		;PASS INFO TO F/D DRIVER, EXIT


	PAGE


;	======================================
;	** HARD DISK BLOCK/DEBLOCK ROUTINES **
;	======================================




;	---HARD DISK READ---
;
;
HDREAD:	XRA	A
	STA	ERFLG		;CLEAR THE ERROR FLAG
	SETBIT	F@RDOP		;SET READ OPERATION FLAG
	TSTBIT	F@WIP		;WRITE IN PROGRESS ?
	CNZ	XWRT		;YES, WRITE DATA BEFORE READ
	CLRBIT	F@WIP		;RESET 'WIP' FLAG
	CALL	TSTHST		;HOST = REQ ?
	JNZ	HDRD1		;NO, READ A BLOCK
	TSTBIT	F@RIP		;PRIOR BLOCK READ ?
	JNZ	HDRD2		;YES, JUST EXTRACT DATA FROM BFR
HDRD1:	CALL	SETHST		;MAKE HOST=REQ
	CALL	XREAD		;READ A BLOCK
HDRD2:	CALL	GETDMA		;GET DMA ADR, SET POINTERS
	CALL	BFRIN		;MOVE DATA TO RAM
	SETBIT	F@RIP		;SET READ-IN-PROGRESS FLAG
	LDA	ERFLG		;ERROR FLAG
	RET




;	---HARD DISK WRITE---
;
;
HDWRT:	XRA	A
	STA	ERFLG		;CLEAR THE ERROR FLAG
	CLRBIT	F@RIP		;CLEAR READ-IN-PROGRESS FLAG
	MOV	A,C		;A = WRITE MODE
	DCR	A
	JZ	WDIR		;DO DIRECTORY WRITE
	JM	WNORM		;DO NORMAL WRITE
;
;   UNALLOCATED WRITE
;
WUN:	TSTBIT	F@WIP		;WRITE IN PROGRESS ?
	CNZ	XWRT		;YES, WRITE DATA IN BFR
; SET UNALLOC RECORD PARAMETERS
	MVI	A,BLKSIZ/128
	STA	U@CNT		;SET UNALLOC RECORD COUNT
	CALL	DRTST		;GET REQ DRIVE
	STA	U@DSK		;UPDATE DRIVE NUM
	LDA	R@SEC
	STA	U@SEC		;UPDATE SECTOR ADR
	LHLD	R@DSK
	SHLD	U@DSK		;UPDATE DRIVE & SECTOR
	LHLD	R@TRK
	SHLD	U@TRK		;UPDATE TRACK ADR
	CALL	SETHST		;SET HOST = REQ
	CALL	BUMP		;BUMP UNALC PARMS FOR NEXT PASS

; XFER DATA TO CTLR BFR
WXFER:	CLRBIT	F@RDOP		;CLEAR READ OPER FLAG
	CALL	GETDMA		;GET DMA ADR, SET FOR WRITE
	CALL	BFROUT		;MOVE DATA TO THE BFR
	SETBIT	F@WIP		;SET WRITE-IN-PROGRESS FLAG
	LDA	ERFLG		;ERROR FLAG
	RET
;
;   NORMAL WRITE
;
WNORM:	LDA	U@CNT		;UNALC RECORD COUNT
	ORA	A
	JZ	WALC		;JIF DO ALLOC WRITE
	LXI	H,U@TRK
	CALL	DSKCMP		;UNALC DSK/TRK = REQ DSK/TRK ?
	JNZ	WALC		;NO, DO ALLOC WRITE
	LDA	R@SEC
	CMP	M		;UNALC SECT = REQ SECT ?
	JNZ	WALC		;NO, DO ALLOC WRITE
	CALL	BUMP		;BUMP UNALC PARMS FOR NEXT PASS
	CALL	TSTHST		;HOST = REQ ?
	JZ	WN1		;YES, CON'T TO FILL THE BFR
	TSTBIT	F@WIP		;WRITE IN PROGRESS ?
	CNZ	XWRT		;YES, WRITE OLD DATA ONTO DISK
	CALL	SETHST		;MAKE HOST = REQ
;
WN1:	TSTBIT	F@RDOP		;INTERVENING READ ?
	CNZ	XREAD		;YES, READ OLD UNALC DATA
	JMP	WXFER		;MOVE DATA TO BFR, EXIT
;
;   ALLOCATED WRITE
;
WALC:	XRA	A
	STA	U@CNT		;CLEAR UNALC RECORD COUNT
	CALL	TSTHST		;HOST = REQ ?
	JZ	WXFER		;YES -  MOVE DATA TO BFR, EXIT
	TSTBIT	F@WIP		;WRITE IN PROGRESS ?
	CNZ	XWRT		;YES, WRITE OLD DATA ONTO DISK
	CALL	SETHST		;MAKE HOST = REQ
	CALL	XREAD		;READ IN ALLOCATED DATA
	JMP	WXFER		;MOVE NEW DATA IN BFR, EXIT
;
;   DIRECTORY WRITE
;
WDIR:	XRA	A
	STA	U@CNT		;CLEAR UNALC RECORD COUNT
	CLRBIT	F@RDOP		;RESET 'RDOP' FLAG
	TSTBIT	F@WIP		;WRITE IN PROGRESS ?
	CNZ	XWRT		;YES, WRITE OLD DATA ONTO THE DISK
	CLRBIT	F@WIP		;CLEAR 'WIP' FLAG
	CALL	SETHST		;MAKE HOST = REQ
	CALL	XREAD		;READ DIR DATA
	RNZ			;RIF READ ERROR
	CALL	GETDMA		;GET DMA ADR, SET POINTERS
	CALL	BFROUT		;MOVE NEW DIR DATA INTO BFR
	JMP	XWRT		;WRITE DIR DATA, EXIT




;	---TEST HOST---
;
;   DETERMINES IF THE HOST DISK ADDRESS IS
;   THE SAME AS THE REQUESTED DISK ADDRESS.
;
;
TSTHST:	LXI	H,H@TRK
	CALL	DSKCMP		;TRACK & DRIVE THE SAME ?
	RNZ			;RIF NO
	LDA	RSA
	CMP	M		;SECTOR THE SAME ?
	RET			;IF A = 0 THEN THEY MATCH




;	---SET HOST---
;
;   SETS THE HOST DISK ADDRESS TO BE THE
;   SAME AS THE REQUESTED DISK ADDRESS.
;
;
SETHST:	LDA	R@DSK
	STA	H@DSK		;DRIVE #
	LHLD	R@TRK
	SHLD	H@TRK		;TRACK ADR
	LDA	RSA
	STA	H@SEC		;SECTOR ADR
	RET




;	---BUMP---
;
;   BUMPS PARAMETERS FOR UNALLOCATED WRITES.
;   PARMS ARE CHANGED FOR THE NEXT PASS THRU
;   THE CODE (NOT THE CURRENT PASS).
;
;
BUMP:	LXI	H,U@CNT		;UNALC RECORD COUNT
	DCR	M		;  DECR IT
	DCX	H		;H/L = U@SEC
	INR	M		;  INCR IT
	MOV	A,M
	CPI	CPMSPT		;CPM SECTORS PER TRACK
	RC			;RIF STAY ON SAME TRACK
; OVERFLOW TO NEXT TRACK
	MVI	M,0		;RESET SECTOR ADR
	LHLD	U@TRK
	INX	H		;INCR TRACK ADR
	SHLD	U@TRK
	RET




;	---GET DMA ADDRESS---
;
;   SETS THE CONTROLLER BUFFER ADDRESS TO THE CORRECT
;   STARTING POINT. ALSO SETS B=128 & H/L=DMADR.
;
;
GETDMA:	LHLD	DMADR		;DMA ADR
	IF	Z80
	MVI	C,CTDP		;FOR BLOCK I/O
	ENDIF
	MVI	B,128		;B = COUNT
	MVI	A,DBENB
	OUT	CTCSR		;ENB DATA BFR
	LDA	R@SEC		;REQUESTED SECTOR
	RRC
	MVI	A,0
	JNC	GET1		;JIF USE 1ST HALF OF BFR
	MOV	A,B		;   USE 2ND HALF OF BFR
GET1:	OUT	CTBFR		;SET CTLR DATA BFR ADR
	RET




;	---BUFFER INPUT---
;
;   READ 128 BYTES FROM THE CONTROLLER BUFFER
;
;
BFRIN	EQU	$
	IN	CTDP		;PRIME DATA INPUT
	NOP			;TIME DELAY
	IF	Z80		;IF USING A Z-80
	INIR			;BLOCK INPUT
	RET
	ELSE			;IF USING AN 8080
BFRIN1:	IN	CTDP		;GET DATA
	MOV	M,A		;PUT DATA INTO RAM
	INX	H
	DCR	B
	JNZ	BFRIN1
	RET
	ENDIF




;	---BUFFER OUTPUT---
;
;   OUTPUT 128 BYTES TO THE CONTROLLER BUFFER
;
;
BFROUT	EQU	$
	IF	Z80		;IF USING A Z-80
	OUTIR			;BLOCK OUTPUT
	RET
	ELSE			;IF USING AN 8080
	MOV	A,M		;GET DATA FROM RAM
	OUT	CTDP		;PUT DATA INTO BFR
	INX	H
	DCR	B
	JNZ	BFROUT
	RET
	ENDIF


	PAGE



;	======================================
;	** HARD DISK I/O & SUPPORT ROUTINES **
;	======================================




;	---READ A BLOCK---
;
;
XREAD:	LXI	H,RTBL		;READ CMD TBL
	CALL	DORW		;READ
;
XR1:	MVI	A,0
	RZ			;RIF READ/WRITE OK
	INR	A
	STA	ERFLG		;SET ERROR FLAG
	RET




;	---WRITE A BLOCK---
;
;
XWRT:	LXI	H,WTBL		;WRITE CMD TBL
	CALL	DORW		;WRITE A SECTOR
	JMP	XR1		;SET ERROR FLAG




;	---EXECUTE READ/WRITE COMMANDS---
;
;
DORW:	SHLD	CTA		;SAVE CMD TBL ADR
	CALL	XSEK		;SEEK TO NEW TRACK (IF REQUIRED)
	RNZ			;RIF SEEK FAILED
	CALL	XSEL		;HEAD SELECT
	LHLD	CTA		;GET CMD TBL ADR
;
DO0:	MOV	A,M
	STA	RETRY		;SET RETRY COUNT
	INX	H
	MOV	A,M
	OUT	CTCSR		;ENB CMP BFR
	INX	H
	MOV	A,M
	OUT	CTBFR		;SET CMP BFR ADR
	INX	H
	SHLD	CTA		;SAVE CMD TBL ADR
;
	LXI	H,RCA		;REAL CYL ADR
	MVI	B,3
DO1:	MOV	A,M
	OUT	CTDP		;PUT HDR INFO INTO CMP BFR
	INX	H
	DCR	B
	JNZ	DO1
	LDA	H@SEC
	OUT	CTDP		;SET SECT ADR FOR COMPARE
;
DO2:	CALL	XRDY		;DRIVE READY ?
	RNZ			;  RIF NO
	LHLD	CTA		;CMD TBL ADR
	MOV	A,M		;A = CNTL BANK
	INX	H
	MOV	B,A
	OUT	CTCSR		;SLCT CNTL BANK
	MOV	A,M
	OUT	CTBFR		;SET START ADR
	INX	H
	MOV	A,B
	ORI	START
	OUT	CTCSR		;START R/W CMD
;
DO3:	CALL	WFD		;WAIT FOR READ/WRITE TO FINISH
	RC			;ABORT, TIME-OUT ERROR
	ANA	M		;TEST CTLR STATUS (0=OK)
	MOV	B,A
	IN	DRCSR		;DRIVE STATUS
	ANI	WRTFLT
	CNZ	CLRDF		;CIF CLEAR DRIVE FAULT
	ORA	B		;SET/CLEAR ERROR FLAG (0=OK)
	RZ			;RIF READ/WRITE OK
	LXI	H,RETRY
	DCR	M		;DECR RETRY COUNT
	JNZ	DO2		;JIF RETRY READ/WRITE
	JMP	SEF		;SET ERROR FLAG,EXIT




;	---WAIT FOR DONE---
;
;
WFD:	PUSH	H		;SAVE H/L
	LXI	H,0		;DELAY COUNT
;
WFD1:	IN	CTCSR		;CTLR STATUS
	RRC
	JC	WFD2		;JIF DONE
	DCX	H		;DECR DELAY COUNT
	MOV	A,H
	ORA	L
	JNZ	WFD1		;JIF CON'T WAITING
; TIME-OUT ERROR
	OUT	CTCSR		;STOP CTLR
	POP	H		;RESTORE H/L
	MVI	A,1
	ORA	A		;SET CPM ERROR FLAG
	STC			;SET INTERNAL ERROR FLAG
	RET
;
WFD2:	POP	H		;RESTORE H/L
	IN	CTCSR		;GET CTLR STATUS
	MOV	B,A	
	XRA	A
	OUT	CTCSR		;STOP CTLR
	MOV	A,B
	ORA	A		;CLEAR CY
	RET




;	---RESTORE--
;
;
XTKZ:	LXI	H,0
	SHLD	RCA		;INIT REAL CYL ADR
	CALL	TZT		;TRACK ZERO TEST
	RZ			;RIF ALREADY AT TRACK ZERO
;
	LXI	H,511		;# CYLS WE CAN SEEK
	CALL	RTZ		;SEEK OUT
	RC			;ABORT, DRIVE NOT RDY
	RZ			;RIF AT TRACK 0
	LXI	H,LZONE+10-511
	CALL	RTZ		;TRY IT AGAIN
	RC			;ABORT, DRIVE NOT RDY
	RZ			;RIF FOUND TRACK ZERO THIS TIME
	JMP	SEF		;ABORT, RESTORE FAILED
;
;   SEEK OUTWARD
;
RTZ:	CALL	XRDY		;DRIVE READY ?
	STC
	RNZ			;ABORT, DRIVE NOT RDY
	MOV	A,L
	OUT	LOSC		;SET LSB OF SEEK COUNT
	MOV	A,H
	OUT	HISC		;SET MSB
	MVI	A,SKOUT
	OUT	EXTCMD		;SET SEEK DIRC = OUTWARD
	MVI	A,SKCMD
	OUT	DRCSR		;ISSUE SEEK CMD
	CALL	WSC		;WAIT FOR SEEK DONE
	RC			;ABORT, RESTORE FAILED TO FINISH
;
TZT:	IN	DRCSR		;DRIVE STATUS
	ANI	TK00		;TRACK ZERO INDICATOR
	XRI	TK00		;..MAKE IT LO-TRUE
	RET




;	---SEEK TO TRACK ZERO---
;
;
XSTZ:	TSTBIT	F@WIP
	JNZ	XSTZ1		;JIF WRITE IN PROGRESS
	STA	FLAGS		;MAKE HOST INACTIVE
XSTZ1:	LXI	H,0
	SHLD	R@TRK		;SET TK ADR = 0
	XRA	A		;SET FLAG = NO ERRORS
	RET




;	---SEEK---
;
;
XSEK:	MVI	A,3
	STA	SKRTC		;SET RETRY COUNT
;
XSEK1:	LHLD	H@TRK		;REQUESTED TRACK
	MVI	B,1		;2 HEADS
	CALL	DRS		;SHIFT H/L RIGHT 'N' TIMES
	LXI	D,MAXCYL	;# OF CYLS
	CALL	TSUB		;TEST REQUESTED CYL ADR
	RAL
	JNC	SEF		;JIF GOT INVALID CYL ADR
	PUSH	H		;SAVE H/L
	LXI	H,FAD		;1ST AVAIL DRIVE
	LDA	H@DSK		;HOST DISK
	SUB	M
	POP	H		;RESTORE H/L
	JZ	XSEK2		;JIF LOGICAL DRIVE ZERO
	DAD	D		;2ND DRIVE USES INNER HALF OF DISK
;
XSEK2:	XCHG			;D/E = NEW CYL ADR
	LHLD	RCA		;REAL (CURR) CYL ADR
	XCHG
	SHLD	RCA		;UPDATE REAL CYL ADR
	XCHG
	CALL	SUBT		; (NEW CYL) - (OLD CYL)
	RZ			;RIF SAME CYL
	MVI	B,1		;DIRC = OUT
	RAL
	JNC	XSEK3		;JIF SEEK OUTWARD
; SEEK INWARD
	MVI	B,3		;DIRC = INWARD
	MOV	A,L		;MAKE SEEK COUNT POSITIVE
	CMA			  ;
	MOV	L,A		  ;
	MOV	A,H		  ;
	CMA			  ;
	MOV	H,A		  ;
	INX	H		  ;
;
XSEK3:	MOV	A,B		;A = SEEK DIRC
	STA	SKDIR		;..SAVE IT
	LXI	D,512
	CALL	TSUB
	RAL
	JNC	XSEK4		;JIF DOUBLE-PUMP (SEEK > 511 CYLS)
	CALL	PSK		;SEEK =< 511 CYLS
	RNZ			;ABORT, SEEK FAILED
	JMP	XSEK5
; DOUBLE PUMP SEEKS
XSEK4:	DCX	D		;D/E = 511
	CALL	SUBT		; (H/L) - (D/E)
	SHLD	RSKNT		;SAVE RESIDUAL SEEK CNT
	XCHG			;H/L = 511
	CALL	PSK		;SEEK 511 CYLS
	RNZ			;ABORT, SEEK FAILED
	LHLD	RSKNT		;RESIDUAL SEEK COUNT
	CALL	PSK		;SEEK REMAINDER
	RNZ
; SEEK VERIFY
XSEK5:	MVI	A,3
	STA	VSRTC		;SET RETRY COUNT
	MVI	A,CBENB
	OUT	CTCSR		;ENABLE BANK ZERO CMP BFR
	MVI	A,VCA
	OUT	CTBFR		;SET CMP BFR ADR
	LHLD	RCA		;REAL (CURR) CYL ADR
	MOV	A,L
	OUT	CTDP		;SET CYL ADR, LSB
	MOV	A,H
	OUT	CTDP		;SET CYL ADR, MSB
XSEK6:	MVI	A,VSA
	OUT	CTBFR		;SET M/CODE START ADR
	MVI	A,START
	OUT	CTCSR		;START VERIFY
	CALL	WFD		;WAIT FOR DONE
	ANI	0CH		;TEST CTLR STATUS
	RZ			;RIF VERIFY OK
	LXI	H,VSRTC
	DCR	M		;DECR RETRY COUNT
	JNZ	XSEK6		;JIF RETRY SEEK VERIFY
; VERIFY FAILED
	CALL	XTKZ		;RESTORE
	LXI	H,SKRTC
	DCR	M		;DECR RETRY COUNT
	JNZ	XSEK1		;JIF RETRY SEEK
; UNRECOVERABLE SEEK ERROR
	ORI	1		;SET ERROR FLAG
	RET			;ABORT
;
;   PARTIAL SEEK
;
PSK:	CALL	XRDY		;DRIVE READY ?
	RNZ			;ABORT, DRIVE NOT RDY
	MOV	A,L
	OUT	LOSC		;SET SEEK COUNT, LSB
	MOV	A,H
	OUT	HISC		;   * MSB
	LDA	SKDIR
	OUT	EXTCMD		;SET SEEK DIRECTION
	MVI	A,3
	OUT	DRCSR		;ISSUE SEEK CMD
;  ---> FALL THRU TO 'WSC' <---




;	---WAIT FOR SEEK COMPLETE---
;
;
WSC:	PUSH	H		;SAVE REGS
	PUSH	B
	LXI	H,0		;TIME-OUT DELAY COUNT
	MVI	B,3		;..DITTO
;
WSC1:	IN	DRCSR		;DRIVE STATUS
	RAL
	JC	WSC2		;JIF SEEK DONE
	DCX	H		;DECR DELAY COUNT
	MOV	A,H
	ORA	L
	JNZ	WSC1		;JIF CON'T WAITING
	DCR	B
	JNZ	WSC1
; TIME-OUT ERROR
	POP	B		;RESTORE REGS
	POP	H
	MVI	A,1
	ORA	A		;SET CPM ERROR FLAG
	STC			;SET INTERNAL ERROR FLAG
	RET
;
WSC2:	POP	B		;RESTORE REGS
	POP	H
	IN	DRCSR		;DRIVE STATUS
	ANI	WRTFLT		;TEST FOR DRIVE FAULT
	RZ			;RIF OK
; A FAULT HERE NORMALLY INDICATES POWER SUPPLY PROBLEMS.
; FALL THRU TO 'CLRDF'.





;	---CLEAR DRIVE FAULT---
;
;
CLRDF:	XRA	A
	OUT	EXTCMD		;DE-SELECT THE DRIVE
				;  * FALL THRU TO 'XSEL' TO
				;    RE-SELECT THE DRIVE!





;	---HEAD SELECT---
;
;
XSEL:	LDA	H@TRK		;REQUESTED TRACK
	ANI	HEDMSK		;HEAD MASK
	STA	RHD		;SAVE REAL HEAD #
	ADD	A		;SHIFT HEAD # LEFT TWICE FOR H/W
	ADD	A
	ORI	1		;TO MAINTAIN DRIVE SLCT
	OUT	EXTCMD		;SELECT HEAD 0...N
	RET




;	---DRIVE READY TEST---
;
;
XRDY:	IN	DRCSR		;DRIVE STATUS
	ANI	1		;DRIVE READY BIT
	XRI	1		;INVERT IT
	RET			;IF A = 0, DRIVE RDY





;	---COMMAND TABLES---
;
;
;   WRITE
;
WTBL:	DB	5		;RETRY COUNT
	DB	5		;CMP BFR ENB
	DB	0E6H		;CMP BFR ADR
	DB	BANK1		;CNTL BANK
	DB	0D3H		;START ADR
	DB	0EH		;STATUS MASK
;
;   READ
;
RTBL:	DB	10		;RETRY COUNT
	DB	4		;CMP BFR ENB
	DB	0EAH		;CMP BFR ADR
	DB	BANK0		;CNTL BANK
	DB	0D7H		;START ADDRESS
	DB	0EH		;STATUS MASK




;	---DISK ADDRESS COMPARE---
;
;   COMPARES EITHER THE HOST DISK ADDRESS OR THE UNALLOCATED
;   DISK ADDRESS AGAINST THE REQUESTED DISK ADDRESS.
;
;
DSKCMP:	LXI	D,R@TRK		;REQUESTED TRACK
	MVI	B,3
DC1:	LDAX	D
	CMP	M
	RNZ			;RIF ADRS NOT EQUAL
	INX	H
	INX	D
	DCR	B
	JNZ	DC1
	RET


	PAGE


;	======================================
;	** SUBROUTINES, MESSAGES, & STORAGE **
;	======================================




;	---SET FLAG BIT---
;
;
@BSET:	PUSH	H
	LXI	H,FLAGS		;BIT FLAGS
	ORA	M		;OLD BITS
	MOV	M,A
	POP	H
	RET




;	---RESET BIT FLAG---
;
;
@BRES:	PUSH	H
	LXI	H,FLAGS		;BIT FLAGS
	ANA	M		;KILL UNWANTED BIT
	MOV	M,A
	POP	H
	RET




;	---TEST BIT FLAG---
;
;
@BTST:	PUSH	H
	LXI	H,FLAGS		;BIT FLAGS
	ANA	M		;TEST THE BIT
	POP	H
	RET




;	---DOUBLE RIGHT SHIFT---
;
;   SHIFTS THE CONTENTS OF H/L RIGHT 'N' TIMES.
;
DRS:	PUSH	PSW		;SAVE 'A'
;
DRS1:	MOV	A,H		;MSB
	ORA	A		;CLEAR CY
	RAR
	MOV	H,A
	MOV	A,L		;LSB
	RAR
	MOV	L,A
	DCR	B		;B = SHIFT COUNT
	JNZ	DRS1
	POP	PSW		;RESTORE 'A'
	RET




;	---MOVE LINKS---
;
;
MLNK:	MVI	B,9*3		;9 LINKS
;
ML1:	MOV	A,M		;GET
	STAX	D		;PUT
	INX	H
	INX	D
	DCR	B
	JNZ	ML1
	RET




;	---DRIVE TEST---
;
;
DRTST:	LXI	H,FAD		;1ST AVAIL DRIVE #
	LDA	R@DSK		;REQUSTED DISK
	SUB	M		;IF CY=1 THEN MUST BE F/D
	RET			;   OTHERWISE WILL BE H/D




;	---16 BIT SUBTRACTION---
;
;
SUBT:	MOV	A,L
	SUB	E		;LSB
	MOV	L,A
	MOV	A,H
	SBB	D		;MSB
	MOV	H,A
	ORA	L		;TEST FOR ZERO RESULTS
	MOV	A,H		;FOR EXTERNAL USE
	RET




;	---16 BIT COMPARISON TEST---
;
;
TSUB:	PUSH	H		;SAVE H/L
	CALL	SUBT		; (H/L) - (D/E)
	POP	H		;RESTORE H/L
	RET




;	---PROGRAM STORAGE---
;
;
;
FLAGS:	DS	1		;BIT FLAGS
;
R@TRK:	DS	2		;REQUESTED TRACK ADDRESS
R@DSK:	DS	1		;   DRIVE #
R@SEC:	DS	1		;   SECTOR ADDRESS
;
U@TRK:	DS	2		;UNALLOCATED TRACK ADDRESS
U@DSK:	DS	1		;   DRIVE #
U@SEC:	DS	1		;   SECTOR ADDRESS
U@CNT:	DS	1		;   RECORD COUNT
;
H@TRK:	DS	2		;HOST (SHUGART) TRACK ADDRESS
H@DSK:	DS	1		;   DRIVE #
H@SEC:	DS	1		;   SECTOR ADDRESS
;
RCA:	DS	2		;REAL CYLINDER ADDRESS
RHD:	DS	1		;  HEAD
RSA:	DS	1		;  SECTOR
;
SECNT:	DS	1		;SECTOR COUNT (FOR WARM BOOT)
RETRY:	DS	1		;RETRY COUNT
CTA:	DS	2		;COMMAND TABLE ADDRESS
ERFLG:	DS	1		;ERROR FLAG
DMADR:	DS	2		;BUFFER (DMA) ADDRESS
WRTMODE: DS	1		;WRITE MODE
TEMP:	DS	2		;TEMPORARY STORAGE
DOTKZ:	DS	1		;FORCE 'TKZ' FLAG
SKRTC:	DS	1		;SEEK RETRY COUNT
VSRTC:	DS	1		;SEEK VERIFY RETRY COUNT
SKDIR:	DS	1		;SEEK DIRECTION
RSKNT:	DS	2		;RESIDUAL SEEK COUNT
;
FLGSIZ	EQU	$-FLAGS		;DEFINES SIZE OF VARIABLE STORAGE
;
FAD:	DS	1		;FIRST AVAILABLE DRIVE NUMBER



;	---CBIOS LINK STORAGE---
;
;
X@TKZ:	DS	3		;HOME
X@SEL:	DS	3		;DRIVE SELECT
X@TK:	DS	3		;SET TRACK ADDRESS
X@SEC:	DS	3		;SET SECTOR ADDRESS
X@DMA:	DS	3		;SET BUFFER ADDRESS
X@RD:	DS	3		;READ
X@WRT:	DS	3		;WRITE
X@LOT:	DS	3		;LIST OUT TEST
X@SXR:	DS	3		;SECTOR XLATE




;	---BDOS STORAGE---
;
;
DIRBFR:	DS	128		;DIRECTORY BUFFER
;
HALL0:	DS	306		;BIT MAP, DRIVE 0
HALL1:	DS	306		;..DRIVE 1
;
LAST	EQU	$



	END
