	TITLE	'MHBOOT -- MINISCRIBE MS-3012 CPM 2.2 BOOTSTRAP'

;	04-15-83



;   MHBOOT is used to boot a CP/M System from the first two tracks
;   of a MiniScribe MS-3012 Hard Disk Drive. An XCOMP STS/STR Hard
;   Disk Controller is used.
;
;   You may wish to add some code to the error routine ERROR in
;   order to report errors to the operator.





;	---PROGRAM CONFIGURATION---
;
;   THE FOLLOWING STATEMENTS ARE USED TO CONFIGURE THE PROGRAM.
;
;	1. TEST -- SET THIS "FALSE" BEFORE USING THE PROGRAM.
;
;	2. S100 -- SET THIS "TRUE" IF YOU ARE USING A CONTROLLER WITH
;	   AN S-100 BUS INTERFACE (STS). SET THIS "FALSE" IF YOU ARE
;	   USING A CONTROLLER WITH A RIBBON CABLE INTERFACE (STR).
;
;
FALSE	EQU	0		;DEFINE FALSE
TRUE	EQU	NOT FALSE	;DEFINE TRUE
;
TEST	EQU	FALSE		;NOT TESTING
;
S100	EQU	TRUE		;S-100 CONTROLLER




;	---EQUATES---
;
;
MSIZE	EQU	62		;MEMORY SIZE IN K-BYTES
;
BIAS	EQU	(MSIZE-20)*1024	;SYSTEM BIAS
CPM	EQU	BIAS+3400H	;CPM LOAD ADDRESS
BIOS	EQU	CPM+1600H	;COLD BOOT ENTRY POINT
;
	IF	TEST	;  ** IF WE ARE TESTING **
STRT	EQU	400H		;PROGRAM START ADDRESS
	ELSE		;  ** IF NORMAL **
STRT	EQU	0		;PROGRAM START ADDRESS
	ENDIF
;
STACK	EQU	STRT+100H	;STACK POINTER
NSECT	EQU	30		;NUMBER SECTORS TO READ
MAXSEC	EQU	32		;NUMBER SECTORS PER TRACK
;
ROM	EQU	0F000H		;BASE ADR OF ROM
DEBUG	EQU	ROM		;DEBUG ENTRY POINT
TRAP	EQU	ROM+1EH		;DEBUG TRAP ADDRESS
;
RPT	EQU	80H		;REPEAT
INDEX	EQU	10H		;INDEX BIT
RCA	EQU	0EAH		;READ COMPARE ADDRESS
RSA	EQU	0D7H		;READ START ADDRESS
;
;   DRIVE COMMANDS
;
SKCMD	EQU	3		;SEEK
NOPC	EQU	40H		;NO PRE-COMPENSATION
LOWRT	EQU	80H		;LOW WRITE CURRENT
;
;   DRIVE STATUS
;
READY	EQU	1		;DRIVE READY
WRTFLT	EQU	2		;WRITE FAULT
TK00	EQU	4		;TRACK ZERO
RAWINDX	EQU	20H		;RAW INDEX
;
;   CONTROLLER COMMANDS
;
BANK0	EQU	0		;BANK 0 SELECT
BANK1	EQU	1		;BANK 1 SELECT
DBENB	EQU	2		;DATA BUFFER ENABLE
CBENB	EQU	4		;COMPARE BUFFER ENABLE
START	EQU	8		;START COMMAND
;
;   DRIVE/CONTROLLER I/O
;
;
	IF	S100	;IF S-100 COMPUTER
CBASE	EQU	70H		;BASE ADR OF THE CONTROLLER
DRCSR	EQU	CBASE		;DRIVE COMMAND/STATUS
EXTCMD	EQU	CBASE+1		;EXTENDED COMMNAND REGISTER
LOSC	EQU	CBASE+2		;SEEK COUNT, LSB
HISC	EQU	CBASE+3		;  * MSB
CTCSR	EQU	CBASE+4		;CONTROLLER COMMAND/STATUS
CTBFR	EQU	CBASE+5		;CONTROLLER BUFFER ADDRESS
CTDP	EQU	CBASE+6		;CONTROLLER DATA PORT
	ELSE		;IF RIBBON CABLE INTERFACE
CBASE	EQU	80H		;BASE ADR OF THE CONTROLLER
DRCSR	EQU	CBASE+7		;DRIVE COMMAND/STATUS
EXTCMD	EQU	CBASE+6		;EXTENDED COMMNAND REGISTER
LOSC	EQU	CBASE+5		;SEEK COUNT, LSB
HISC	EQU	CBASE+4		;  * MSB
CTCSR	EQU	CBASE+3		;CONTROLLER COMMAND/STATUS
CTBFR	EQU	CBASE+2		;CONTROLLER BUFFER ADDRESS
CTDP	EQU	CBASE+1		;CONTROLLER DATA PORT
	ENDIF

	PAGE

	ORG	STRT



;	---PROGRAM ENTRY POINT---
;
;
BOOT:	LXI	SP,STACK	;SET SP
	LXI	H,CPM
	SHLD	DMADR		;SET DMA ADR
	LXI	H,100H
	SHLD	HEAD		;SET HEAD=0 & SECTOR=1
	MVI	A,NSECT
	STA	SCNT		;NUM SECTS TO BOOT
;
;    READ LOOP
;
LOOP:	CALL	READ		;READ A SECTOR
	JNZ	ERROR		;JIF READ FAILED
;
	MVI	A,DBENB
	OUT	CTCSR		;ENB DATA BFR
	XRA	A
	OUT	CTBFR		;SET BFR ADR = 0
	MOV	B,A		;B = COUNT (256)
	LHLD	DMADR		;H/L = DMA ADR
	IN	CTDP		;PRIME INPUT DATA
	NOP			;TIME DELAY
L1:	IN	CTDP		;GET DATA FROM BFR
	MOV	M,A		;PUT DATA INTO RAM
	INX	H
	DCR	B
	JNZ	L1
;
	SHLD	DMADR		;SAVE NEW DMA ADR
	LXI	H,SECTOR
	INR	M		;INCR SECTOR ADR
	MOV	A,M
	CPI	MAXSEC
	JC	L2		;JIF CON'T ON SAME TRACK
	MVI	M,0		;RESET SECTOR ADR
	DCX	H	
	INR	M		;INCR HEAD NR
	CALL	HDSEL		;SLCT NEW HEAD
	INX	H
;
L2:	INX	H
	DCR	M		;DECR LOOP COUNT
	JNZ	LOOP		;JIF MORE TO DO
	IF	TEST	;  ** IF WE ARE TESTING **
	JMP	DEBUG		;GO TO DEBUG
	ELSE		;  ** IF NORMAL **
	JMP	BIOS		;COLD BOOT
	ENDIF




;	---READ A SECTOR---
;
;
READ:	MVI	D,10		;D = RETRY COUNT
	MVI	A,CBENB
	OUT	CTCSR		;SLCT BANK0 CMP BFR
	MVI	A,RCA+2
	OUT	CTBFR		;SET BFR ADR
	LDA	HEAD
	OUT	CTDP		;SET HEAD # FOR HEADER COMPARE
	LDA	SECTOR
	OUT	CTDP		;    ** DITTO SECTOR ADR
;
RD1:	CALL	RDY		;DRIVE READ ?
	RNZ			;RIF DRIVE NOT RDY
	XRA	A
	OUT	CTCSR		;SLCT BANK0
	MVI	A,RSA
	OUT	CTBFR		;SET CMD BFR ADR
	MVI	A,START
	OUT	CTCSR		;START READ
	LXI	H,0		;DELAY COUNT
;
RD2:	IN	CTCSR		;CTLR STATUS
	RRC
	JC	RD3		;JIF READ DONE
	DCX	H		;DECR RETRY COUNT
	MOV	A,H
	ORA	L
	JNZ	RD2		;JIF CON'T WAITING
	OUT	CTCSR		;STOP CTLR
	JMP	RD4		;ABORT, TIME-OUT ERROR
;
RD3:	NOP			;DELAY FOR H/W
	IN	CTCSR		;GET NON-CHANGING STATUS
	ANI	0EH		;TEST CTLR STATUS
	MOV	B,A		;  SAVE RESULTS
	XRA	A
	OUT	CTCSR		;STOP CTLR
	IN	DRCSR		;DRIVE STATUS
	ANI	WRTFLT
	CNZ	CLRDF		;CIF CLEAR DRIVE FAULT
	ORA	B		;SET/CLEAR ERROR FLAG (0=OK)
	RZ			;RIF READ/WRITE OK
	DCR	D		;DECR RETRY COUNT
	JNZ	RD1		;JIF RETRY READ
; READ ERROR
RD4:	MVI	A,2		;A = ERROR CODE
	ORA	A		;SET 8080 FLAGS
	RET




;	---DRIVE READY TEST---
;
;
RDY:	IN	DRCSR		;DRIVE STATUS
	ANI	READY		;DRIVE READY BIT
	XRI	READY		;  MAKE IT LO-TRUE
	RZ			;RIF DRIVE READY
	MVI	A,1		;A = ERROR CODE
	ORA	A		;SET 8080 FLAGS
	RET




;	---HEAD SELECT---
;
;
HDSEL:	LDA	HEAD
	RLC
	RLC			;SHIFT FOR H/L
	ORI	1		;TO MAINTAIN DRIVE SLCT
	OUT	EXTCMD		;SLCT NEW HEAD
	RET




;	---CLEAR DRIVE FAULT---
;
;
CLRDF:	XRA	A
	OUT	EXTCMD		;DE-SELECT THE DRIVE
	MVI	A,1
	OUT	EXTCMD		;RE-SELECT THE DRIVE
	RET




;	---UNRECOVERABLE ERRORS---
;
;   UNRECOVERABLE ERRORS COME HERE WITH AN ERROR
;   CODE IN 'A'.
;
;	A = 1	Drive not Ready
;	  = 2	Unrecoverable Read Error
;
;
ERROR	EQU	$
	IF	TEST		; ** IF WE ARE TESTING **
	CALL	TRAP		;DEBUG TRAP ADR
	ENDIF
	JMP	ERROR		;   ** ADD SOME CODE HERE **




;	---STORAGE---
;
;
DMADR:	DS	2		;DMA ADDRESS
HEAD:	DS	1		;HEAD NUMBER
SECTOR:	DS	1		;SECTOR ADDRESS
SCNT:	DS	1		;SECTOR COUNT



	END
