	TITLE	'MHBT -- CONTROLLER BUFFER TEST (MINISCRIBE VERSION)'

;	04-19-83



;   This program tests the buffer memory of an XCOMP STS/STR
;   Hard Disk Controller. A PASS/FAIL indication will be
;   given to the operator.




;	---PROGRAM CONFIGURATION---
;
;   THE FOLLOWING STATMENTS ARE USED TO CONFIGURE THE PROGRAM.
;
;	1. S100 -- SET THIS 'TRUE' IF YOU ARE USING A CONTROLLER WITH
;	   AN S-100 BUS INTERFACE (STS). SET THIS 'FALSE' IF YOU ARE
;	   USING A CONTROLLER WITH A RIBBON CABLE INTERFACE (STR).
;
;	2. Z80 -- SET THIS 'TRUE' IF YOU ARE USING A Z80 MICROPROCESSOR.
;	   SET THIS 'FALSE' IF YOU ARE USING AN 8080 PROCESSOR.
;
;
FALSE	EQU	0		;DEFINE FALSE
TRUE	EQU	NOT FALSE	;DEFINE TRUE
;
S100	EQU	TRUE		;S-100 CONTROLLER
;
Z80	EQU	TRUE		;USING Z-80 MICROPROCESSOR




;	---EQUATES---
;
;
STRT	EQU	100H		;PROGRAM START ADDRESS
WBOOT	EQU	0		;CPM WARM BOOT ADDRESS
BDOS	EQU	5		;BDOS ENTRY POINT
RPT	EQU	80H		;REPEAT
;
;   CONTROLLER COMMANDS
;
BANK0	EQU	0		;BANK 0
BANK1	EQU	1		;BANK 1
DBENB	EQU	2		;DATA BUFFER
;
;   CONTROLLER I/O
;
	IF	S100	; IF S-100 BUS COMPUTER
CBASE	EQU	70H		;BASE ADR OF THE CONTROLLER
DRCSR	EQU	CBASE		;DRIVE COMMAND/STATUS
EXTCMD	EQU	CBASE+1		;EXTENDED COMMNAND REGISTER
LOSC	EQU	CBASE+2		;SEEK COUNT, LSB
HISC	EQU	CBASE+3		;  * MSB
CTCSR	EQU	CBASE+4		;CONTROLLER COMMAND/STATUS
CTBFR	EQU	CBASE+5		;CONTROLLER BUFFER ADDRESS
CTDP	EQU	CBASE+6		;CONTROLLER DATA PORT
	ELSE		; IF RIBBON CABLE INTERFACE
CBASE	EQU	80H		;BASE ADR OF THE CONTROLLER
DRCSR	EQU	CBASE+7		;DRIVE COMMAND/STATUS
EXTCMD	EQU	CBASE+6		;EXTENDED COMMNAND REGISTER
LOSC	EQU	CBASE+5		;SEEK COUNT, LSB
HISC	EQU	CBASE+4		;  * MSB
CTCSR	EQU	CBASE+3		;CONTROLLER COMMAND/STATUS
CTBFR	EQU	CBASE+2		;CONTROLLER BUFFER ADDRESS
CTDP	EQU	CBASE+1		;CONTROLLER DATA PORT
	ENDIF
;
;   ASCII CHARACTERS 
;
CR	EQU	0DH		;CARRIAGE RETURN
LF	EQU	0AH		;LINE FEED
BELL	EQU	7		;DING!
ESC	EQU	1BH		;ESCAPE KEY
;
;   CPM FUNCTION CODES
;
F@CI	EQU	1		;CONSOLE INPUT
F@CO	EQU	2		;CONSOLE OUTPUT
F@PRT	EQU	9		;PRINT MESSAGES
F@REQ	EQU	11		;GET CONSOLE STATUS

	PAGE

;	============
;	** MACROS **
;	============




;	---CALL CPM---
;
;
CPM	MACRO	FC
	MVI	C,FC		;; LOAD FUNCTION CODE
	CALL	BDOS		;; GOTO CPM
	ENDM




;	---PRINT MACRO---
;
;
PRNT	MACRO	MSG
	LXI	D,MSG		;; ADR OF MSG
	CPM	F@PRT		;; PRINT THE MSG
	ENDM




;	---Z80 BLOCK INPUT---
;
;
INIR	MACRO
	DB	0EDH,0B2H
	ENDM




;	---Z80 BLOCK OUTPUT---
;
;
OUTIR	MACRO
	DB	0EDH,0B3H
	ENDM

	PAGE

	ORG	STRT




;	---PROGRAM ENTRY POINT---
;
;
;
BGN:	LXI	SP,$
	XRA	A
	OUT	CTCSR	
	OUT	EXTCMD
	OUT	DRCSR		;STOP ANY DISK ACTIVITY
	PRNT	MSG1		;SAY HELLO
	LXI	H,1
	SHLD	SEED		;SET SEED=1 AND PASS=0
;
;   BUFFER MEMORY TEST
;
BMT:	CALL	FILL		;FILL BFR WITH PSEUDO RANDOM DATA
	CALL	PUT		;MOVE DATA TO CTLR BFR
	CALL	GET		;READ THE DATA BACK
	CALL	TST		;VERIFY DATA JUST READ
	JNZ	MEMBAD		;JIF BFR TEST FAILED
	CALL	STOP		;STOP TESTING ?
	JNZ	XIT		;..JIF YES
	LXI	H,SEED
	MOV	C,M
	CALL	RNG		;CHANGE SEED
	MOV	M,C		;  SAVE NEW SEED
	INX	H
	INR	M		;INCR PASS COUNT
	JZ	MEMOK		;JIF TEST DONE (BFR OK)
	MOV	A,M		;A = PASS
	CALL	PPC		;PRINT PASS COUNT
	JMP	BMT		;DO NEXT PASS
; MEMORY OK
MEMOK:	PRNT	MSG2		;PRINT OK MSG
	JMP	BAT		;DO BFR ADR TEST
; MEMORY BAD
MEMBAD:	PRNT	MSG3		;PRINT FAIL MESSAGE
;
;   BUFFER ADDRESS TEST
;
BAT:	PRNT	MSG4		; 'BFR ADR TEST'
	MVI	A,DBENB
	OUT	CTCSR		;ENABLE DATA BFR
	XRA	A
	OUT	CTBFR		;INIT BFR ADR
BAT1:	OUT	CTDP		;FILL DATA BFR = INCR CNT
	INR	A
	JNZ	BAT1
;
	LXI	H,0FF01H
	SHLD	SEED		;SET SEED = 1, PASS CNT = 255
	MOV	C,L		;INIT ADR
BAT2:	MVI	B,255		;INIT LOOP CNT
BAT3:	MOV	A,C
	OUT	CTBFR		;SET RANDOM BFR ADR
	IN	CTDP		;PRIME INPUT DATA
	NOP
	IN	CTDP		;GET BFR DATA
	CMP	C		;DATA SHOULD MATCH ADR
	JNZ	ADRBAD		;JIF TEST FAILED
	CALL	RNG		;ROLL NEW RAN NUM
	DCR	B
	JNZ	BAT3		;JIF MORE TO DO
	CALL	STOP		;STOP TESTING ?
	JNZ	XIT		;..JIF YES
	LXI	H,SEED
	MOV	C,M
	CALL	RNG		;ROLL NEW SEED
	MOV	M,C		;  SAVE IT
	INX	H
	DCR	M		;DECR PASS CNT
	JZ	ADROK		;JIF DONE (TEST OK)
	MVI	A,255
	SUB	M		;CALC PASS COUNT
	CALL	PPC		;..PRINT IT
	JMP	BAT2		;DO NEXT PASS
; ADR TEST OK
ADROK:	PRNT	MSG5		;SAY OK
	JMP	XIT
; ADR TEST FAILED
ADRBAD:	PRNT	MSG6		;WE ARE NOT OK
;
;   PROGRAM DONE, EXIT
;
XIT:	LXI	H,WRTCODE
	CALL	SQUIRT		;LOAD/RELOAD WRITE MICROCODE
	LXI	H,RDCODE
	CALL	SQUIRT		;LOAD/RELOAD READ MICROCODE
	MVI	A,1
	OUT	EXTCMD		;RESELECT THE DRIVE
	INR	A
	OUT	DRCSR		;ENABLE THE INTERFACE
	JMP	WBOOT		;EXIT




;	---FILL DATA BUFFER---
;
;
FILL:	LXI	H,BFR1		;BFR ADR
	LXI	D,512		;BFR SIZE
	LDA	SEED		;RAN NUM SEED
	MOV	C,A
;
FILL1:	MOV	M,C		;PUT RAN DATA INTO BFR
	CALL	RNG		;ROLL NEW RAN NUM
	INX	H
	DCX	D
	MOV	A,D
	ORA	E
	JNZ	FILL1
	RET




;	---TEST BUFFER---
;
;
TST:	LXI	H,BFR1
	LXI	D,BFR2
	LXI	B,512
;
TST1:	LDAX	D		;GET READ BACK DATA
	CMP	M		;COMPARE AGAINST ORIG
	RNZ			;RIF VERIFY ERROR
	INX	H
	INX	D
	DCX	B
	MOV	A,B
	ORA	C
	JNZ	TST1
	RET




;	---MOVE DATA TO CONTROLLER BUFFER---
;
;
PUT:	LXI	H,BFR1		;BFR ADR
	MVI	C,CTDP		;FOR Z80 I/O
	MVI	B,128
	MVI	A,BANK0
	CALL	PUT1		;FILL BANK0
	MVI	B,128
	MVI	A,BANK1
	CALL	PUT1		;FILL BANK1
	MVI	A,DBENB		;FOR DATA BFR
;
PUT1:	OUT	CTCSR		;SELECT BFR
	XRA	A
	OUT	CTBFR		;INIT BFR ADR
;
PUT2	EQU	$
	IF	Z80		;IF USING Z80
	OUTIR
	ELSE			;IF USING 8080
	MOV	A,M		;GET DATA FROM BFR
	OUT	CTDP		;MOVE TO CTLR
	INX	H
	DCR	B
	JNZ	PUT2
	ENDIF
	RET




;	---GET DATA FROM CONTROLLER BUFFER---
;
;
GET:	LXI	H,BFR2		;BFR ADR
	MVI	C,CTDP		;FOR Z80 I/O
	MVI	B,128
	MVI	A,BANK0
	CALL	GET1		;GET DATA FROM BANK0
	MVI	B,128
	MVI	A,BANK1
	CALL	GET1		;GET DATA FROM BANK1
	MVI	A,DBENB		;FOR DATA BFR
;
GET1:	OUT	CTCSR		;SELECT BANK
	XRA	A
	OUT	CTBFR		;INIT BFR ADR
	IN	CTDP		;PRIME INPUT DATA
	NOP
;
GET2	EQU	$
	IF	Z80		;IF USING Z80
	INIR
	ELSE			;IF USING 8080
	IN	CTDP
	MOV	M,A
	INX	H
	DCR	B
	JNZ	GET2
	ENDIF
	RET

	PAGE


;	======================================
;	** SUBROUTINES, MESSAGES, & STORAGE **
;	======================================




;	---RANDOM NUMBER GENERATOR---
;
;   THIS ROUTINE IS USED TO PRODUCE A PSEUDO RANDOM
;   NUMBER WHICH REPEATS EVERY 255 BYTES. THIS IS
;   USEFULL FOR TESTING PATTERN SENSITIVITY AS WELL
;   AS FINDING BUFFER ADDRESSING PROBLEMS.
;
;
RNG:	MOV	A,C
	RLC
	JNC	RNG1
	XRI	1CH
RNG1:	MOV	C,A
	RET




;	---STOP TEST---
;
;   TEST FOR 'ESC' KEY. IF FOUND, ABORT TEST.
;
;
STOP:	CPM	F@REQ		;GET CONSOLE STATUS
	ORA	A		;..TEST IT
	RZ			;RIF DATA NOT AVAIL
	CPM	F@CI		;GET INPUT CHAR
	ANI	7FH		;KILL PARITY BIT
	CPI	ESC		;'ESC' KEY ?
	JZ	STP1		;JIF GOT 'ESC' KEY
	XRA	A		;SET FLAG = CON'T TEST
	RET
;
STP1:	PRNT	MSG8		; 'TEST ABORTED'
	ORI	1		;SET FLAG = STOP TEST
	RET




;	---PRINT PASS COUNT---
;
;
PPC:	PUSH	PSW		;SAVE PASS COUNT
	PRNT	MSG7		; 'PASS='
	POP	PSW		;A = PASS COUNT
; FALL THRU TO 'P2D' TO PRINT PASS COUNT




;	---PRINT TWO DIGITS---
;
;   CONVERTS BINARY DATA (IN 'A') TO 2 ASCII
;   HEX CHARACTERS & PRINTS THOSE CHARACTERS.
;
;
P2D:	PUSH	D		;SAVE D/E
	PUSH	PSW
	RRC			;MOVE MSD TO LSD
	RRC
	RRC
	RRC
	CALL	P1D		;CONVERT MSD & PRINT IT
	POP	PSW
	CALL	P1D		;  * LSD
	POP	D		;RESTORE D/E
	RET
;
P1D:	ANI	0FH		;KILL MSD
	ADI	'0'
	CPI	'9'+1
	JC	P1DA		;JIF CHAR 0-9
	ADI	7		;CHAR A-F
P1DA:	MOV	E,A
	CPM	F@CO		;PRINT THE DIGIT
	RET




;	---SQUIRT MICROCODE---
;
;
SQUIRT:	MOV	A,M
	OUT	CTCSR		;SLCT BANK
	INX	H
	MOV	A,M
	OUT	CTBFR		;SET CTLR BFR ADR
	INX	H
;
SQ1:	MOV	A,M
	ORA	A
	RZ			;RIF DONE
	INX	H		;INCR TBL ADR
	MOV	B,A		;B = COUNT
	RAL
	JC	SQ3		;JIF REPEAT
;
SQ2:	MOV	A,M
	OUT	CTDP		;OUTPUT CONTROL INFO
	INX	H
	DCR	B
	JNZ	SQ2
	JMP	SQ1		;DO NEXT STRING
;
SQ3:	ORA	A		;CLEAR CY
	RAR
	MOV	B,A		;B = COUNT
	MOV	A,M
	INX	H
SQ4:	OUT	CTDP		;OUTPUT REPEAT CONTROL INFO
	DCR	B
	JNZ	SQ4
	JMP	SQ1		;DO NEXT STRING




;	---WRITE MICROCODE---
;
;
WRTCODE	EQU	$		;  MINISCRIBE     06-09-81
;
	DB	1		
	DB	0D3H		
	DB	2,40H		
	DB	41H		
	DB	RPT+3,40H	
	DB	RPT+12,6DH	
	DB	1,63H		
	DB	RPT+4,65H	
	DB	RPT+2,67H	
	DB	2,49H		
	DB	40H		
	DB	RPT+17,50H	
	DB	5,5BH		
	DB	0DDH		
	DB	5DH		
	DB	50H		
	DB	0FH		
	DB	0		




;	---READ MICROCODE---
;
;
RDCODE	EQU	$		;  MINISCRIBE     06-09-81
;
	DB	0		
	DB	0D7H		
	DB	2,40H		
	DB	41H		
	DB	RPT+3,40H	
	DB	RPT+12,6DH	
	DB	1,63H		
	DB	RPT+4,65H	
	DB	RPT+2,67H	
	DB	1,49H		
	DB	RPT+2,40H	
	DB	RPT+12,6DH	
	DB	3,63H		
	DB	0E7H		
	DB	67H		
	DB	RPT+7,0FH	
;
	DB	2,40H		
	DB	41H		
	DB	RPT+3,40H	
	DB	RPT+12,6DH	
	DB	1,63H		
	DB	RPT+2,65H	
	DB	RPT+2,60H	
	DB	RPT+2,67H	
	DB	1,0FH		
	DB	0		





;	---MESSAGES---
;
;
MSG1:	DB	CR,LF,LF,LF
	DB	'MHBT -- Vers 1.0',CR,LF
	DB	'19-Apr-83'
	DB	CR,LF,LF,LF
	DB	'Buffer Memory Test'
	DB	CR,LF,'$'
;
MSG2:	DB	CR,LF,'* Buffer Memory Test OK$'
;
MSG3:	DB	BELL,CR,LF,'* Buffer Memory Test Failed$'
;
MSG4:	DB	CR,LF,LF,LF
	DB	'Buffer Address Test'
	DB	CR,LF,'$'
;
MSG5:	DB	CR,LF,'* Buffer Address Test OK'
	DB	CR,LF,LF,'$'
;
MSG6:	DB	BELL,CR,LF,'* Buffer Address Test Failed'
	DB	CR,LF,LF,'$'
;
MSG7:	DB	CR,'Pass=$'
;
MSG8	DB	BELL,CR,LF,LF,LF
	DB	'* Test Aborted!'
	DB	CR,LF,LF,'$'




;	---STORAGE---
;
;
SEED:	DS	1		;RANDOM NUMBER SEED
PASS:	DS	1		;PASS COUNT
;
BFR1:	DS	512
BFR2:	DS	512



	END
