	TITLE	'MHGET -- READ (BOOT) CPM FROM A MINISCRIBE MS-3012'

;	04-15-83	** A **


;   MHGET is used to read (boot) CPM from a MiniScribe MS-3012 hard
;   disk drive. An XCOMP STS/STR controller is used.




;	---PROGRAM CONFIGURATION---
;
;
;   THE FOLLOWING STATEMENTS ARE USED TO CONFIGURE THE PROGRAM.
;
;	1. TEST -- SET THIS 'FALSE' BEFORE USING THIS PROGRAM.
;
;	2. S100 -- SET THIS 'TRUE' IF YOU ARE USING A CONTROLLER WITH
;	   AN S-100 BUS COMPUTER (STS). SET THIS 'FALSE' IF YOU ARE
;	   USING A CONTROLLER WITH A RIBBON CABLE INTERFACE (STR).
;
;
;
FALSE	EQU	0		;DEFINE FALSE
TRUE	EQU	NOT FALSE	;DEFINE TRUE
;
TEST	EQU	FALSE		;NOT TESTING
;
S100	EQU	TRUE		;S-100 CONTROLLER






;	---EQUATES---
;
;
MSIZE	EQU	62		;MEMORY SIZE IN K-BYTES
;
BIAS	EQU	(MSIZE-20)*1024	;SYSTEM BIAS
CPM	EQU	BIAS+3400H	;CPM LOAD ADDRESS
BIOS	EQU	CPM+1600H	;COLD BOOT ENTRY POINT
;
STRT	EQU	100H		;PROGRAM START ADDRESS
STACK	EQU	100H		;STACK POINTER
LMSK	EQU	0FFH		;LSB MASK
NSECT	EQU	30		;NUMBER SECTORS TO READ
MAXSEC	EQU	32		;NUMBER SECTORS PER TRACK
RPT	EQU	80H		;REPEAT
INDEX	EQU	10H		;INDEX BIT
LZONE	EQU	656		;LANDING ZONE
;
ROM	EQU	0F000H		;BASE ADR OF ROM
DEBUG	EQU	ROM		;DEBUG ENTRY POINT
TRAP	EQU	ROM+1EH		;DEBUG TRAP ADDRESS
;
;   ASCII CHARACTERS
;
CR	EQU	0DH		;CARRIAGE RETURN
LF	EQU	0AH		;LINE FEED
BELL	EQU	7		;DING!
;
;   CRT I/O
;
CRTS	EQU	9		;STATUS
CRTD	EQU	8		;DATA
CRTBE	EQU	1		;BUFFER EMPTY
CRTDA	EQU	2		;DATA AVAILABLE
;
;   DRIVE COMMANDS
;
SKCMD	EQU	3		;SEEK
SKOUT	EQU	1		;SEEK DIRECTION = OUTWARD
NOPC	EQU	40H		;NO PRE-COMPENSATION
LOWRT	EQU	80H		;LOW WRITE CURRENT
;
;   DRIVE STATUS
;
READY	EQU	1		;DRIVE READY
WRTFLT	EQU	2		;WRITE FAULT
TK00	EQU	4		;TRACK ZERO
RAWINDX	EQU	20H		;RAW INDEX
;
;   CONTROLLER COMMANDS
;
BANK0	EQU	0		;BANK 0 SELECT
BANK1	EQU	1		;BANK 1 SELECT
DBENB	EQU	2		;DATA BUFFER ENABLE
CBENB	EQU	4		;COMPARE BUFFER ENABLE
START	EQU	8		;START COMMAND
;
;   DRIVE/CONTROLLER I/O
;
	IF	S100	;IF S-100 COMPUTER
CBASE	EQU	70H		;BASE ADR OF THE CONTROLLER
DRCSR	EQU	CBASE		;DRIVE COMMAND/STATUS
EXTCMD	EQU	CBASE+1		;EXTENDED COMMNAND REGISTER
LOSC	EQU	CBASE+2		;SEEK COUNT, LSB
HISC	EQU	CBASE+3		;  * MSB
CTCSR	EQU	CBASE+4		;CONTROLLER COMMAND/STATUS
CTBFR	EQU	CBASE+5		;CONTROLLER BUFFER ADDRESS
CTDP	EQU	CBASE+6		;CONTROLLER DATA PORT
	ELSE		;IF RIBBON CABLE INTERFACE
CBASE	EQU	80H		;BASE ADR OF THE CONTROLLER
DRCSR	EQU	CBASE+7		;DRIVE COMMAND/STATUS
EXTCMD	EQU	CBASE+6		;EXTENDED COMMNAND REGISTER
LOSC	EQU	CBASE+5		;SEEK COUNT, LSB
HISC	EQU	CBASE+4		;  * MSB
CTCSR	EQU	CBASE+3		;CONTROLLER COMMAND/STATUS
CTBFR	EQU	CBASE+2		;CONTROLLER BUFFER ADDRESS
CTDP	EQU	CBASE+1		;CONTROLLER DATA PORT
	ENDIF

	PAGE

	ORG	STRT



;	---PROGRAM ENTRY POINT---
;
;
GET:	LXI	SP,STACK
	XRA	A
	OUT	CTCSR
	OUT	DRCSR
	OUT	EXTCMD		;STOP ANY PRIOR ACTIVITY
	LXI	H,RDCODE
	CALL	SQUIRT		;SET READ M/CODE
	LXI	H,WRTCODE
	CALL	SQUIRT		;SET WRITE M/CODE
	MVI	A,1
	OUT	EXTCMD		;SLCT HEAD ZERO
	INR	A
	OUT	DRCSR		;ENB SECTOR/INDEX TO DATA BOARD
;
	CALL	TKZ		;RESTORE
	LXI	H,CPM
	SHLD	DMADR		;INIT DMA ADR
	LXI	H,0
	SHLD	CYL		;INIT CYL ADR
	INR	H
	SHLD	HEAD		;SET HEAD=0 & SECTOR=1
	MVI	A,NSECT
	STA	SCNT		;NUM SECTS TO BOOT
;
;   READ LOOP
;
LOOP:	LXI	H,RTBL
	CALL	DORW		;READ A SECTOR
;
	MVI	A,DBENB
	OUT	CTCSR		;ENB DATA BFR
	XRA	A
	OUT	CTBFR		;SET BFR ADR = 0
	MOV	B,A		;B = COUNT (256)
	LHLD	DMADR		;ADR OF THE DATA
	IN	CTDP		;PRIME INPUT DATA
	NOP			;TIME DELAY
L1:	IN	CTDP		;GET DATA FROM BFR
	MOV	M,A		;PUT DATA INTO RAM
	INX	H
	DCR	B
	JNZ	L1
	SHLD	DMADR		;SAVE NEW DMA ADR
;
	LXI	H,SECTOR
	INR	M		;INCR SECTOR ADR
	MOV	A,M
	CPI	MAXSEC
	JC	L2		;JIF CON'T ON SAME TRACK
	MVI	M,0		;RESET SECTOR ADR
	DCX	H	
	INR	M		;INCR HEAD NR
	CALL	HDSEL		;SLCT NEW HEAD
	INX	H
;
L2:	INX	H
	DCR	M		;DECR SECTOR COUNT
	JNZ	LOOP		;JIF NOT DONE
;
	IF	TEST
	JMP	DEBUG		;IF WE ARE TESTING
	ELSE
	JMP	BIOS		;RUN CPM
	ENDIF





;	---EXECUTE READ/WRITE COMMANDS---
;
;
DORW:	MOV	A,M
	STA	RETRY		;SET RETRY COUNT
	INX	H
	MOV	A,M
	OUT	CTCSR		;ENB CMP BFR
	INX	H
	MOV	A,M
	OUT	CTBFR		;SET CMP BFR ADR
	INX	H
	SHLD	CTA		;SAVE CMD TBL ADR
;
	LXI	H,CYL
	MVI	B,4
DO1:	MOV	A,M
	OUT	CTDP		;PUT HDR INFO INTO CMP BFR
	INX	H
	DCR	B
	JNZ	DO1
;
DO2:	CALL	RDY		;DRIVE READY ?  (RIF YES)
	LHLD	CTA		;CMD TBL ADR
	MOV	A,M		;A = CNTL BANK
	INX	H
	MOV	B,A
	OUT	CTCSR		;SLCT CNTL BANK
	MOV	A,M
	OUT	CTBFR		;SET START ADR
	INX	H
	MOV	A,B
	ORI	START
	OUT	CTCSR		;START R/W CMD
	PUSH	H		;SAVE H/L
	LXI	H,0		;DELAY COUNT
;
DO3:	IN	CTCSR		;CTLR STATUS
	RRC
	JC	DO4		;JIF READ DONE
	DCX	H		;DECR DELAY COUNT
	MOV	A,H
	ORA	L
	JNZ	DO3		;JIF CON'T WAITING
	OUT	CTCSR		;STOP THE CTLR
	POP	H		;RESTORE H/L
	JMP	DO5		;ABORT, TIME-OUT ERROR
;
DO4:	POP	H		;RESTORE H/L
	IN	CTCSR		;GET NON-CHANGING STATUS
	MOV	B,A		;   SAVE IT
	XRA	A
	OUT	CTCSR		;STOP CTLR
	MOV	A,B
	ANA	M		;TEST CTLR STATUS (0=OK)
	MOV	B,A
	IN	DRCSR		;DRIVE STATUS
	ANI	WRTFLT
	CNZ	CLRDF		;CIF CLEAR DRIVE FAULT
	ORA	B		;SET/CLEAR ERROR FLAG (0=OK)
	RZ			;RIF READ/WRITE OK
	LXI	H,RETRY
	DCR	M		;DECR RETRY CNT
	JNZ	DO2		;JIF RETRY READ/WRITE
; READ ERROR
DO5:	LXI	H,MSG3		;ADR OF ERR MSG
	JMP	PEM		;ABORT, READ FAILED




;	---RESTORE--
;
;
TKZ:	CALL	TZT		;TRACK ZERO TEST
	RZ			;RIF ALREADY AT TRACK ZERO
;
	LXI	H,511		;# CYLS WE CAN SEEK
	CALL	RTZ		;SEEK OUT
	RZ			;RIF AT TRACK 0
	LXI	H,LZONE+10-511
	CALL	RTZ		;TRY IT AGAIN
	RZ			;RIF FOUND TRACK ZERO THIS TIME
; RESTORE FAILED
	LXI	H,MSG2		;H/L = ADR OF ERR MSG
	JMP	PEM		;ABORT, RESTORE FAILED
;
;   SEEK OUTWARD
;
RTZ:	CALL	RDY		;DRIVE READY ?  (RIF YES)
	MOV	A,L
	OUT	LOSC		;SET LSB OF SEEK COUNT
	MOV	A,H
	OUT	HISC		;SET MSB
	MVI	A,SKOUT
	OUT	EXTCMD		;SET SEEK DIRC = OUTWARD
	MVI	A,SKCMD
	OUT	DRCSR		;ISSUE SEEK CMD
	CALL	WSC		;WAIT FOR SEEK DONE
;
TZT:	IN	DRCSR		;DRIVE STATUS
	ANI	TK00		;TRACK ZERO INDICATOR
	XRI	TK00		;..MAKE IT LO-TRUE
	RET




;	---WAIT FOR SEEK COMPLETE---
;
;
WSC:	PUSH	B		;SAVE B/C
	LXI	H,0		;DELAY COUNT
	MVI	B,3		;..DITTO
;
WSC1:	IN	DRCSR		;DRIVE STATUS
	RAL
	JC	WSC2		;JIF SEEK DONE
	DCX	H		;DECR DELAY COUNT
	MOV	A,H
	ORA	L
	JNZ	WSC1		;JIF CON'T WAITING
	DCR	B
	JNZ	WSC1		;JIF CON'T WAITING
	POP	B		;RESTORE B/C
	LXI	H,MSG2
	JMP	PEM		;ABORT, RESTORE FAILED TO FINISH
;
WSC2:	POP	B		;RESTORE B/C
	XRA	A		;SET FLAG = SEEK FINISHED
	RET




;	---DRIVE READY TEST---
;
;
RDY:	IN	DRCSR		;DRIVE STATUS
	ANI	READY		;DRIVE RDY BIT
	RNZ			;RIF DRIVE READY
	LXI	H,MSG1		;ADR OF ERR MSG
	JMP	PEM		;ABORT, DRIVE NOT READY




;	---HEAD SELECT---
;
;
HDSEL:	LDA	HEAD
	RLC
	RLC			;SHIFT FOR H/L
	ORI	1		;TO MAINTAIN DRIVE SLCT
	OUT	EXTCMD		;SLCT NEW HEAD
	RET




;	---CLEAR DRIVE FAULT---
;
;
CLRDF:	XRA	A
	OUT	EXTCMD		;DE-SELECT THE DRIVE
	MVI	A,1
	OUT	EXTCMD		;RE-SELECT THE DRIVE
	RET




;	---READ COMMAND TABLE---
;
;
RTBL:	DB	10		;RETRY COUNT
	DB	4		;CMP BFR ENB
	DB	0EAH		;CMP BFR ADR
	DB	BANK0		;CNTL BANK
	DB	0D7H		;START ADDRESS
	DB	0EH		;STATUS MASK




;	---SQUIRT MICROCODE---
;
;
SQUIRT:	MOV	A,M
	OUT	CTCSR		;SLCT BANK
	INX	H
	MOV	A,M
	OUT	CTBFR		;SET CTLR BFR ADR
	INX	H
;
SQ1:	MOV	A,M
	ORA	A
	RZ			;RIF DONE
	INX	H		;INCR TBL ADR
	MOV	B,A		;B = COUNT
	RAL
	JC	SQ3		;JIF REPEAT
;
SQ2:	MOV	A,M
	OUT	CTDP		;OUTPUT CONTROL INFO
	INX	H
	DCR	B
	JNZ	SQ2
	JMP	SQ1		;DO NEXT STRING
;
SQ3:	ORA	A		;CLEAR CY
	RAR
	MOV	B,A		;B = COUNT
	MOV	A,M
	INX	H
SQ4:	OUT	CTDP		;OUTPUT REPEAT CONTROL INFO
	DCR	B
	JNZ	SQ4
	JMP	SQ1		;DO NEXT STRING





;	---WRITE MICROCODE---
;
;
WRTCODE	EQU	$		;  MINISCRIBE		06-09-81
;
	DB	1		
	DB	0D3H		
	DB	2,40H		
	DB	41H		
	DB	RPT+3,40H	
	DB	RPT+12,6DH	
	DB	1,63H		
	DB	RPT+4,65H	
	DB	RPT+2,67H	
	DB	2,49H		
	DB	40H		
	DB	RPT+17,50H	
	DB	5,5BH		
	DB	0DDH		
	DB	5DH		
	DB	50H		
	DB	0FH		
	DB	0		




;	---READ MICROCODE---
;
;
RDCODE	EQU	$		;  MINISCRIBE		06-09-81
;
	DB	0		
	DB	0D7H		
	DB	2,40H		
	DB	41H		
	DB	RPT+3,40H	
	DB	RPT+12,6DH	
	DB	1,63H		
	DB	RPT+4,65H	
	DB	RPT+2,67H	
	DB	1,49H		
	DB	RPT+2,40H	
	DB	RPT+12,6DH	
	DB	3,63H		
	DB	0E7H		
	DB	67H		
	DB	RPT+7,0FH	
;
	DB	2,40H		
	DB	41H		
	DB	RPT+3,40H	
	DB	RPT+12,6DH	
	DB	1,63H		
	DB	RPT+2,65H	
	DB	RPT+2,60H	
	DB	RPT+2,67H	
	DB	1,0FH		
	DB	0		





;	---PRINT ERROR MESSAGES---
;
;   UNRECOVERABLE ERRORS COME HERE TO PRINT
;   ERROR MESSAGES.
;
;
PEM:	LXI	SP,STACK	;RESET STACK ADR
	CALL	PRINT		;PRINT ERROR MSG
	CALL	CI		;WAIT FOR THE OPER
	IF	TEST		;IF WE ARE TESTING
	CPI	3		;CNTL-C ?
	CZ	TRAP		;YES, TRAP TO DEBUG
	ENDIF
	LXI	H,CRLF
	CALL	PRINT		;DO CR.LF
	JMP	LOOP		;TRY TO BOOT AGAIN




;	---PRINT MESSAGES---
;
;
PRINT:	MOV	A,M		;A = PRINT DATA
	INX	H		;POINT TO NEXT CHAR
	ORA	A
	RZ			;ZERO ENDS MSG
	MOV	C,A
	CALL	CO		;PRINT A CHAR
	JMP	PRINT




;	---CONSOLE INPUT---
;
;
CI:	IN	CRTS		;CRT STATUS
	ANI	CRTDA
	JZ	CI		;WAIT FOR INPUT CHAR
	IN	CRTD		;INPUT DATA
	ANI	7FH		;KILL PARITY BIT
	RET




;	---CONSOLE OUTPUT---
;
;
CO:	IN	CRTS		;CRT STATUS
	ANI	CRTBE
	JZ	CO		;WAIT FOR BFR EMPTY
	MOV	A,C
	OUT	CRTD		;OUTPUT DATA
	RET




;	---MESSAGES---
;
;
MSG1:	DB	BELL,CR,LF,'* Drive not Ready ',0
;
MSG2:	DB	BELL,CR,LF,'* Restore Failed ',0
;
MSG3:	DB	BELL,CR,LF,'* Read Failed ',0
;
CRLF:	DB	CR,LF,0




;	---STORAGE---
;
;
DMADR:	DS	2		;DMA ADDRESS
CYL:	DS	2		;CYLINDER ADDRESS
HEAD:	DS	1		;HEAD NUMBER
SECTOR:	DS	1		;SECTOR ADDRESS
SCNT:	DS	1		;SECTOR COUNT
RETRY:	DS	1		;RETRY COUNT
CTA:	DS	2		;COMMAND TABLE ADRESS




	END
