	TITLE	'MHIOS -- MINISCRIBE MS-3012 CBIOS FOR CPM 2.2'

;	04-16-83




;   MHIOS is the I/O Driver for CPM 2.2 using a MiniScribe MS-3012 Hard
;   Disk Drive & an XCOMP STS/STR Hard Disk Controller. The drive is
;   configured as two logical 5 Megabyte Drives. MHIOS also supports two
;   floppy disk drives using a CCS 2422 F/D Controller (single density
;   mode only).





;	---PROGRAM CONFIGURATION---
;
;
;   THE FOLLOWING STATMENTS ARE USED TO CONFIGURE THE PROGRAM.
;
;	1. TEST -- SET THIS 'FALSE' BEFORE USING THE PROGRAM.
;
;	2. Z80 -- SET THIS "TRUE" IF YOU ARE USING A Z-80 MICROPROCESSOR.
;	   THIS WILL SPEED UP DATA TRANSFERS TO/FROM THE CONTROLLER
;	   BUFFER.
;
;	3. S100 -- SET THIS "TRUE" IF YOU ARE USING A CONTROLLER WITH
;	   AN S-100 BUS INTERFACE (STS). SET THIS "FALSE" IF YOU ARE
;	   USING A CONTROLLER WITH A RIBBON CABLE INTERFACE (STR).
;
;
;
;
FALSE	EQU	0		;DEFINE FALSE
TRUE	EQU	NOT FALSE	;DEFINE TRUE
;
TEST	EQU	FALSE		;NOT TESTING
;
Z80	EQU	TRUE		;WE ARE USING A Z-80
;
S100	EQU	TRUE		;S-100 CONTROLLER





;	---EQUATES---
;
;
MSIZE	EQU	62		;MEMORY SIZE IN K-BYTES
;
BIAS	EQU	(MSIZE-20)*1024	;SYSTEM BIAS
CPM	EQU	BIAS+3400H  	;BASE ADR OF CPM
BDOS	EQU	CPM+806H  	;LINK TO BDOS
BIOS	EQU	CPM+1600H	;CBIOS START ADDRESS
OFFSET	EQU	1F80H-BIOS	;DDT SYSGEN OFFSET
;
DEBUG	EQU	0F000H		;BASE ADR OF ROM DEBUG
TRAP	EQU	DEBUG+1EH	;DEBUG TRAP ADDRESS
LMSK	EQU	0FFH		;LSB MASK
;
NSECT	EQU	(BIOS-CPM)/128	;NUMBER SECTORS TO WARM BOOT
IOBYT	EQU	3		;I/O SYSTEM CONFIGURATION
CDISK	EQU	4		;CURRENT DISK ADDRESS (USED BY CPM)
BUFF	EQU	80H		;FCB BUFFER
RPT	EQU	80H		;REPEAT
INDEX	EQU	10H		;INDEX BIT
VSA	EQU	8		;SEEK VERIFY START ADDRESS
VCA	EQU	1BH		;SEEK VERIFY COMPARE ADDRESS
;
MAXHD	EQU	2		;NUMBER OF LOGICAL DISK DRIVES
MAXCYL	EQU	612/2		;NUMBER OF CYLINDERS PER LOGICAL DRIVE
LZONE	EQU	656		;LANDING ZONE
HEDMSK	EQU	1		;HEAD MASK
MAXSEC	EQU	32		;NUMBER OF SECTORS PER TRACK
;
MAXFD	EQU	2		;NUMBER OF FLOPPY DISK DRIVES
LLD	EQU	MAXHD+MAXFD	;LAST LOGICAL DRIVE
;
;   CRT I/O
;
CRTS	EQU	9		;STATUS
CRTD	EQU	8		;DATA
CRTBE	EQU	1		;BUFFER EMPTY
CRTDA	EQU	2		;DATA AVAILABLE
;
;   LINE PRINTER I/O
;
LPS	EQU	0BH		;STATUS
LPD	EQU	0AH		;DATA
LPXR	EQU	84H		;XMTR READY
;
;   HARD DISK/CPM INTERFACE
;
BLKSIZ	EQU	2048		;NUMBER OF BYTES PER BLOCK
CPMSPT	EQU	2*MAXSEC	;NUMBER OF CPM SECTORS PER TRACK
;
;   BIT FLAGS
;
F@RDOP	EQU	1		;READ OPERATION
F@RIP	EQU	2		;READ IN PROGRESS
F@WIP	EQU	4		;WRITE IN PROGRESS
;
;   DRIVE COMMANDS
;
SKCMD	EQU	3		;SEEK
SKOUT	EQU	1		;SEEK DIRECTION = OUTWARD
NOPC	EQU	40H		;NO PRE-COMPENSATION
LOWRT	EQU	80H		;LOW WRITE CURRENT
;
;   DRIVE STATUS
;
READY	EQU	1		;DRIVE READY
WRTFLT	EQU	2		;WRITE FAULT
TK00	EQU	4		;TRACK ZERO
RAWINDX	EQU	20H		;RAW INDEX
;
;   CONTROLLER COMMANDS
;
BANK0	EQU	0		;BANK 0 SELECT
BANK1	EQU	1		;BANK 1 SELECT
DBENB	EQU	2		;DATA BUFFER ENABLE
CBENB	EQU	4		;COMPARE BUFFER ENABLE
START	EQU	8		;START COMMAND
;
;   CONTROLLER STATUS
;
INDEX	EQU	10H		;DELAYED INDEX
SECTOR	EQU	20H		;SECTOR
;
;   DRIVE/CONTROLLER I/O
;
	IF	S100	;IF S-100 BUS
CBASE	EQU	70H		;BASE ADR OF THE CONTROLLER
DRCSR	EQU	CBASE		;DRIVE COMMAND/STATUS
EXTCMD	EQU	CBASE+1		;EXTENDED COMMNAND REGISTER
LOSC	EQU	CBASE+2		;SEEK COUNT, LSB
HISC	EQU	CBASE+3		;  * MSB
CTCSR	EQU	CBASE+4		;CONTROLLER COMMAND/STATUS
CTBFR	EQU	CBASE+5		;CONTROLLER BUFFER ADDRESS
CTDP	EQU	CBASE+6		;CONTROLLER DATA PORT
	ELSE		;IF RIBBON CABLE INTERFACE
CBASE	EQU	80H		;BASE ADR OF THE CONTROLLER
DRCSR	EQU	CBASE+7		;DRIVE COMMAND/STATUS
EXTCMD	EQU	CBASE+6		;EXTENDED COMMNAND REGISTER
LOSC	EQU	CBASE+5		;SEEK COUNT, LSB
HISC	EQU	CBASE+4		;  * MSB
CTCSR	EQU	CBASE+3		;CONTROLLER COMMAND/STATUS
CTBFR	EQU	CBASE+2		;CONTROLLER BUFFER ADDRESS
CTDP	EQU	CBASE+1		;CONTROLLER DATA PORT
	ENDIF
;
;
;   ASCII CHARACTERS
;
CR	EQU	0DH		;CARRIAGE RETURN
LF	EQU	0AH		;LINE FEED
BELL	EQU	7		;DING
EOF	EQU	1AH		;CPM END-OF-FILE
;
;   FLOPPY DISK EQUATES
;
FD0	EQU	11H		;DRIVE ZERO
MAXI	EQU	10H		;MAXI (8") DISK
AUTO	EQU	80H		;AUTO WAIT
;
;   1791 I/O
;
DISK	EQU	0F8H		;BASE ADR OF F/D CONTROLLER
;
D@CSR	EQU	DISK		;COMMAND/STATUS
D@TK	EQU	DISK+1		;TRACK ADDRESS
D@SEC	EQU	DISK+2		;SECTOR ADDRESS
D@DP	EQU	DISK+3		;DATA PORT
D@CP1	EQU	DISK+4		;CONTROL PORT #1
D@CP2	EQU	DISK+5		;CONTROL PORT #2
;
;   DISK COMMANDS
;
STEP	EQU	1		;STEP RATE = 6 MSEC
C@TKZ	EQU	8+STEP		;TRACK ZERO SEEK
C@SEEK	EQU	1CH+STEP	;TRACK SEEK
C@SUN	EQU	10H		;SEEK (UNLOAD HEAD)
C@RD	EQU	80H		;SECTOR READ
C@WRT	EQU	0A0H		;SECTOR WRITE
;
;   DISK STATUS  (AFTER COMMAND EXECUTION)
;
NRB	EQU	80H		;NOT READY BIT
WPROT	EQU	40H		;WRITE PROTECT BIT
S@TKZ	EQU	9CH		;TRACK ZERO SEEK
S@SEEK	EQU	98H		;SEEK
S@RD	EQU	9CH		;READ
S@WRT	EQU	0ECH		;WRITE


	PAGE


;	============
;	** MACROS **
;	============




;	---SET FLAG BIT---
;
;
SETBIT	MACRO	BIT
	MVI	A,BIT		;; A = BIT TO SET
	CALL	@BSET		;; SET THE BIT
	ENDM




;	---RESET FLAG BIT---
;
;
CLRBIT	MACRO	BIT
?Y	SET	NOT BIT
	MVI	A,?Y		;; A = CLEAR MASK
	CALL	@BRES		;; RESET THE BIT
	ENDM




;	---TEST A FLAG BIT---
;
;
TSTBIT	MACRO	BIT
	MVI	A,BIT		;; A = BIT UNDER TEST
	CALL	@BTST		;; TEST THE BIT
	ENDM




;	---Z80 BLOCK INPUT---
;
;
INIR	MACRO
	DB	0EDH,0B2H
	ENDM




;	---Z80 BLOCK OUTPUT---
;
;
OUTIR	MACRO
	DB	0EDH,0B3H
	ENDM


	PAGE

	ORG	BIOS


;	---JUMP TABLE---
;
;
	JMP	BOOT		;  0 - COLD BOOT
WBX:	JMP	WBOOT		;  1 - WARM BOOT
	JMP	CSTS		;  2 - CONSOLE STATUS REQUEST
	JMP	CI		;  3 - CONSOLE INPUT
	JMP	CO		;  4 - CONSOLE OUTPUT
	JMP	LO		;  5 - LIST OUTPUT
	JMP	PO		;  6 - PUNCH OUTPUT
	JMP	RDR		;  7 - READER INPUT
	JMP	HOME		;  8 - TRACK ZERO SEEK
	JMP	SETDR		;  9 - SET DRIVE #
	JMP	SETTK		; 10 - SET TRACK ADR
	JMP	SETSEC		; 11 - SET SECTOR ADR
	JMP	SETDMA		; 12 - SET BUFFER ADDRESS
	JMP	READ		; 13 - READ A SECTOR
	JMP	WRITE		; 14 - WRITE A SECTOR
	JMP	LOTST		; 15 - LIST OUTPUT READY TEST
	JMP	SXR		; 16 - SECTOR XLATE ROUTINE




;	---SIGN ON MESSAGE---
;
;
;
HELLO:	DB	CR,LF,LF,LF,LF
	DB	MSIZE/10 + '0'
	DB	MSIZE MOD 10  + '0'
	DB	'K MHIOS '
	DB	CR,LF,'CP/M Vers 2.2'
	DB	CR,LF,'16-Apr-83'
	DB	CR,LF,0


	PAGE

;	======================
;	** DISK DEFINITIONS **
;	======================




;	---DISK PARAMETERS---
;
;
DPHDR:	DW	0,0		;HARD DISK, LOGICAL DRIVE 0
	DW	0,0
	DW	DIRBFR,HDBLK
	DW	LAST,HALL0
;
	DW	0,0		;HARD DISK, LOGICAL DRIVE 1
	DW	0,0
	DW	DIRBFR,HDBLK
	DW	LAST,HALL1
;
	DW	XLATE,0		;F/D DRIVE 0
	DW	0,0
	DW	DIRBFR,FDBLK
	DW	FCHK0,FALL0
;
	DW	XLATE,0		;F/D DRIVE 1
	DW	0,0
	DW	DIRBFR,FDBLK
	DW	FCHK1,FALL1




;	---HARD DISK PARAMETER BLOCK---
;
;
;   THIS TABLE DEFINES HARD DISK CONFIGURATION.
;
;	1. THE MS-3012 IS CONGIGURED AS TWO LOGICAL 5 MEGABYTE DRIVE.
;
;	2. THE CPM LOGICAL BLOCK SIZE IS 2048 BYTES.
;
;	3. THERE ARE 1024 DIRECTORY ENTRIES PER LOGICAL DRIVE.
;
;	4. THE FIRST TWO TRACKS ARE RESERVED FOR THE SYSTEM. THIS WILL
;	   ALLOW AN AGGREGATE CCP/BDOS/CBIOS SIZE OF 16 K-BYTES.
;
;
;
HDBLK:	DW	2*MAXSEC	;SPT: SECTORS PER TRACK
	DB	4		;BSH: BLOCK SHIFT FACTOR
	DB	15		;BLM: BLOCK MASK
	DB	0		;EXM: NULL MASK
	DW	2439		;DSM: (DISK SIZE) - 1
	DW	1023		;DRM: (DIR SIZE) - 1
	DB	0FFH		;AL0: DIRECTORY ALLOCATION BITS, MSB
	DB	0FFH		;AL1:   * LSB
	DW	0		;CKS: CHECK SIZE
	DW	2		;OFF: TRACK OFFSET




;	---FLOPPY DISK PARAMETER BLOCK---
;
;
;   THIS TABLE DEFINES HOW THE FLOPPY DISK IS
;   CONFIGURED. CURRENTLY THE F/D IS SET UP AS
;   A NORMAL SINGLE DENSITY DISK. THIS WILL
;   MAINTAIN COMPATIBILITY WITH PREVIOUS VERSIONS
;   OF CPM.
;
;
;
FDBLK:	DW	26		;SPT: SECTORS PER TRACK
	DB	3		;BSH: BLOCK SHIFT FACTOR
	DB	7		;BLM: BLOCK MASK
	DB	0		;EXM: NULL MASK
	DW	242		;DSM: (DISK SIZE) - 1
	DW	63		;DRM: (DIR SIZE) - 1
	DB	0C0H		;AL0: DIRECTORY ALLOCATION BITS, MSB
	DB	0		;AL1:   * LSB
	DW	16		;CKS: CHECK SIZE
	DW	2		;OFF: TRACK OFFSET




;	---FLOPPY DISK TRANSLATION TABLE---
;
;
;
XLATE:	DB	1,7,13,19	;  1...4
	DB	25,5,11,17	;  5...8
	DB	23,3,9,15	;  9...12
	DB	21,2,8,14	; 13...16
	DB	20,26,6,12	; 17...20
	DB	18,24,4,10	; 21...24
	DB	16,22		; 25 & 26

	PAGE


;	=====================
;	** CBIOS FUNCTIONS **
;	=====================




;	---COLD BOOT---
;
;
BOOT:	LXI	SP,BUFF		;USE LOW MEMORY
	LXI	H,HELLO
	CALL	PRNT		;SAY HELLO
	CALL	INIT		;INIT FLAGS & VARIABLES
	STA	CDISK		;FOR USE BY CPM
	STA	CFD		;SET CURR F/D = DRIVE 0
	MVI	A,FD0
	OUT	D@CP1		;SLCT F/D DRIVE 0
	STA	RFD		;..SAVE FOR FUTURE USE
	MVI	A,0D0H
	OUT	D@CP2		;8", SNGL DENS
	JMP	XIT		;GOTO CPM




;	---WARM BOOT---
;
;
WBOOT:	LXI	SP,BUFF
	XRA	A
	STA	R@DSK		;SLCT DRIVE ZERO
;
	CALL	XSTZ		;SEEK TO TRACK ZERO
	LXI	H,CPM		;CPM LOAD ADR
	SHLD	DMADR		;  SAVE IT
	MVI	C,2
	CALL	SETSEC		;SET SECTOR ADR
	MVI	A,NSECT		; # SECTORS TO WARM BOOT
;
WB1:	STA	SECNT		;SAVE THE SECTOR COUNT
	CALL	HDREAD		;READ A SECTOR FROM THE HARD DISK
	ORA	A
	JZ	WB2		;JIF READ OK
	LXI	H,MSG1		;  'CAN'T BOOT'
	CALL	PRNT		;PRINT ERROR MSG
	CALL	CI		;WAIT FOR OPER
	CALL	CRLF
	JMP	WBOOT		;TRY ANOTHER BOOT
;
WB2:	LHLD	DMADR
	LXI	D,128
	DAD	D		;BUMP DMA ADR
	SHLD	DMADR
	LXI	H,R@SEC
	MOV	C,M
	INR	C		;INCR SECT ADR
	CALL	SETSEC		;SET NEW SECT ADR
	LDA	SECNT		;A = SECTOR COUNT
	DCR	A
	JNZ	WB1		;JIF MORE TO READ




;	---EXIT TO CPM---
;
;
XIT:	LXI	H,BUFF
	SHLD	DMADR		;INIT DMA ADR
	MVI	A,0C3H		;JMP INST
	STA	0
	LXI	H,WBX
	SHLD	1		;SET WARM BOOT LINK
	LXI	H,BDOS
	STA	5
	SHLD	6		;SET BDOS LINK
	LDA	CDISK		;CURRENT LOGGED-IN DISK & USER
	MOV	C,A
	JMP	CPM		;GOTO CPM




;	---CONSOLE INPUT---
;
;
CI:	IN	CRTS		;CRT STATUS
	ANI	CRTDA		;DATA AVAIL
	JZ	CI		;WAIT FOR DATA
	IN	CRTD		;CRT DATA
	ANI	7FH		;KILL PARITY BIT
	IF	TEST		;IF WE ARE TESTING
	CPI	'~'
	RNZ			;RIF NOT DEBUG TRAP CHAR
	CALL	TRAP		;TRAP TO ROM DEBUG
	JMP	CI		;GET NEXT CHAR
	ELSE			;IF NOT TESTING
	RET
	ENDIF




;	---CONSOLE OUTPUT---
;
;
CO:	IN	CRTS		;CRT STATUS
	ANI	CRTBE		;BFR EMPTY
	JZ	CO		;WAIT FOR EMPTY
	MOV	A,C
	OUT	CRTD		;OUTPUT DATA
	RET





;	---CONSOLE STATUS---
;
;
CSTS:	IN	CRTS		;CRT STATUS
	ANI	CRTDA		;DATA AVAIL
	RZ			;RIF NOT AVAIL
	ORI	-1		;SET AVAIL FLAG
	RET





;	---LIST OUTPUT---
;
;
LO:	IN	LPS		;L/P STATUS
	ANI	LPXR		;XMTR RDY
	XRI	LPXR		;INVERT BOTH BITS
	JNZ	LO		;WAIT FOR RDY
	MOV	A,C
	OUT	LPD		;OUTPUT PRINT DATA
	RET





;	---READER INPUT---
;
;
RDR:	MVI	A,EOF		;NO RDR -- SET 'EOF' FLAG
	RET





;	---PUNCH OUT---
;
;
PO:	RET			;  * NOT IMPLEMENTED *





;	---LIST READY TEST---
;
;
LOTST:	IN	LPS		;L/P STATUS
	ANI	LPXR		;XMTR RDY
	XRI	LPXR		;INVERT BOTH BITS
	MVI	A,0
	RNZ			;RIF NOT RDY
	ORI	-1		;SET READY FLAG
	RET





;	---SECTOR TRANSLATE ROUTINE---
;
;
SXR:	XCHG			;H/L = ADR OF XLATE TABLE
	MOV	A,L
	ORA	H		;TEST TABLE ADR
	JNZ	SXR1		;JIF DO XLATE
	DAD	B		;DON'T XLATE -- USE SECT ADR AS IS
	RET
;
SXR1:	DAD	B		;INDEX INTO THE TABLE
	MOV	L,M		;L = XLATED SECTOR ADR
	MVI	H,0		;H = 0 FOR 16-BIT VALUE
	RET





;	---HOME---
;
;
HOME:	LDA	R@DSK		;DRIVE #
	CPI	MAXHD
	JC	XSTZ		;JIF REZERO HARD DISK
	CPI	LLD
	JC	FTKZ		;JIF REZERO FLOPPY
;
;   SET ERROR FLAG
;
SEF:	MVI	A,1		;A = ERROR CODE
	ORA	A		;SET 8080/Z80 FLAGS
	RET





;	---READ---
;
;
READ:	LDA	R@DSK		;DRIVE #
	CPI	MAXHD
	JC	HDREAD		;JIF READ FROM H/D
	CPI	LLD
	JC	FREAD		;JIF READ FROM F/D
	JMP	SEF		;ABORT, SELECT ERROR





;	---WRITE---
;
;
WRITE:	LDA	R@DSK		;DRIVE #
	CPI	MAXHD
	JC	HDWRT		;JIF WRITE ONTO H/D
	CPI	LLD
	JC	FWRT		;JIF WRITE ONTO F/D
	JMP	SEF		;ABORT, SELECT ERROR





;	---SET DRIVE NUMBER---
;
;
SETDR:	MOV	A,C		;A = NEW DRIVE #
	STA	R@DSK		;  SAVE IT
	LXI	H,0
	CPI	LLD
	RNC			;RIF INVALID DRIVE #
	MOV	L,A
	DAD	H		;SHIFT H/L LEFT 4
	DAD	H		;   (HL = HL*16)
	DAD	H
	DAD	H
	LXI	D,DPHDR		;DISK PARM HDR
	DAD	D		;H/L = DPHDR FOR SELECTED DRIVE
	RET





;	---SET SECTOR ADR---
;
;
SETSEC:	MOV	A,C
	STA	R@SEC		;SAVE SECTOR ADR
; SET REAL SECTOR ADR FOR THE H/D DRIVER
	ANI	NOT 1		;CLEAR LOW ORDER BIT
	RRC			;THERE ARE 2 CPM SECTORS PER H/D SECT
	STA	RSA		;SAVE REAL SECTOR ADR
	RET





;	---SET TRACK ADDRESS---
;
;
SETTK:	MOV	H,B
	MOV	L,C
	SHLD	R@TRK		;SAVE TRACK ADR
	RET





;	---SET DMA ADDRESS---
;
;
SETDMA:	MOV	H,B
	MOV	L,C
	SHLD	DMADR		;SAVE DMA ADR
	RET

	PAGE


;	======================================
;	** HARD DISK BLOCK/DEBLOCK ROUTINES **
;	======================================




;	---HARD DISK READ---
;
;
HDREAD:	XRA	A
	STA	ERFLG		;CLEAR THE ERROR FLAG
	SETBIT	F@RDOP		;SET READ OPERATION FLAG
	TSTBIT	F@WIP		;WRITE IN PROGRESS ?
	CNZ	XWRT		;YES, WRITE DATA BEFORE READ
	CLRBIT	F@WIP		;RESET 'WIP' FLAG
	CALL	TSTHST		;HOST = REQ ?
	JNZ	HDRD1		;NO, READ A BLOCK
	TSTBIT	F@RIP		;PRIOR BLOCK READ ?
	JNZ	HDRD2		;YES, JUST EXTRACT DATA FROM BFR
HDRD1:	CALL	SETHST		;MAKE HOST=REQ
	CALL	XREAD		;READ A BLOCK
HDRD2:	CALL	GETDMA		;GET DMA ADR, SET POINTERS
	CALL	BFRIN		;MOVE DATA TO RAM
	SETBIT	F@RIP		;SET READ-IN-PROGRESS FLAG
	LDA	ERFLG		;ERROR FLAG
	RET




;	---HARD DISK WRITE---
;
;
HDWRT:	XRA	A
	STA	ERFLG		;CLEAR THE ERROR FLAG
	CLRBIT	F@RIP		;CLEAR READ-IN-PROGRESS FLAG
	MOV	A,C		;A = WRITE MODE
	DCR	A
	JZ	WDIR		;DO DIRECTORY WRITE
	JM	WNORM		;DO NORMAL WRITE
;
;   UNALLOCATED WRITE
;
WUN:	TSTBIT	F@WIP		;WRITE IN PROGRESS ?
	CNZ	XWRT		;YES, WRITE DATA IN BFR
; SET UNALLOC RECORD PARAMETERS
	MVI	A,BLKSIZ/128
	STA	U@CNT		;SET UNALLOC RECORD COUNT
	LHLD	R@DSK
	SHLD	U@DSK		;UPDATE DRIVE & SECTOR
	LHLD	R@TRK
	SHLD	U@TRK		;UPDATE TRACK ADR
	CALL	SETHST		;SET HOST = REQ
	CALL	BUMP		;BUMP UNALC PARMS FOR NEXT PASS

; XFER DATA TO CTLR BFR
WXFER:	CLRBIT	F@RDOP		;CLEAR READ OPER FLAG
	CALL	GETDMA		;GET DMA ADR, SET FOR WRITE
	CALL	BFROUT		;MOVE DATA TO THE BFR
	SETBIT	F@WIP		;SET WRITE-IN-PROGRESS FLAG
	LDA	ERFLG		;ERROR FLAG
	RET
;
;   NORMAL WRITE
;
WNORM:	LDA	U@CNT		;UNALC RECORD COUNT
	ORA	A
	JZ	WALC		;JIF DO ALLOC WRITE
	LXI	H,U@TRK
	CALL	DSKCMP		;UNALC DSK/TRK = REQ DSK/TRK ?
	JNZ	WALC		;NO, DO ALLOC WRITE
	LDA	R@SEC
	CMP	M		;UNALC SECT = REQ SECT ?
	JNZ	WALC		;NO, DO ALLOC WRITE
	CALL	BUMP		;BUMP UNALC PARMS FOR NEXT PASS
	CALL	TSTHST		;HOST = REQ ?
	JZ	WN1		;YES, CON'T TO FILL THE BFR
	TSTBIT	F@WIP		;WRITE IN PROGRESS ?
	CNZ	XWRT		;YES, WRITE OLD DATA ONTO DISK
	CALL	SETHST		;MAKE HOST = REQ
;
WN1:	TSTBIT	F@RDOP		;INTERVENING READ ?
	CNZ	XREAD		;YES, READ OLD UNALC DATA
	JMP	WXFER		;MOVE DATA TO BFR, EXIT
;
;   ALLOCATED WRITE
;
WALC:	XRA	A
	STA	U@CNT		;CLEAR UNALC RECORD COUNT
	CALL	TSTHST		;HOST = REQ ?
	JZ	WXFER		;YES -  MOVE DATA TO BFR, EXIT
	TSTBIT	F@WIP		;WRITE IN PROGRESS ?
	CNZ	XWRT		;YES, WRITE OLD DATA ONTO DISK
	CALL	SETHST		;MAKE HOST = REQ
	CALL	XREAD		;READ IN ALLOCATED DATA
	JMP	WXFER		;MOVE NEW DATA IN BFR, EXIT
;
;   DIRECTORY WRITE
;
WDIR:	XRA	A
	STA	U@CNT		;CLEAR UNALC RECORD COUNT
	CLRBIT	F@RDOP		;RESET 'RDOP' FLAG
	TSTBIT	F@WIP		;WRITE IN PROGRESS ?
	CNZ	XWRT		;YES, WRITE OLD DATA ONTO THE DISK
	CLRBIT	F@WIP		;CLEAR 'WIP' FLAG
	CALL	SETHST		;MAKE HOST = REQ
	CALL	XREAD		;READ DIR DATA
	RNZ			;RIF READ ERROR
	CALL	GETDMA		;GET DMA ADR, SET POINTERS
	CALL	BFROUT		;MOVE NEW DIR DATA INTO BFR
	JMP	XWRT		;WRITE DIR DATA, EXIT




;	---TEST HOST---
;
;   DETERMINES IF THE HOST DISK ADDRESS IS
;   THE SAME AS THE REQUESTED DISK ADDRESS.
;
;
TSTHST:	LXI	H,H@TRK
	CALL	DSKCMP		;TRACK & DRIVE THE SAME ?
	RNZ			;RIF NO
	LDA	RSA
	CMP	M		;SECTOR THE SAME ?
	RET			;IF A = 0 THEN THEY MATCH




;	---SET HOST---
;
;   SETS THE HOST DISK ADDRESS TO BE THE
;   SAME AS THE REQUESTED DISK ADDRESS.
;
;
SETHST:	LDA	R@DSK
	STA	H@DSK		;DRIVE #
	LHLD	R@TRK
	SHLD	H@TRK		;TRACK ADR
	LDA	RSA
	STA	H@SEC		;SECTOR ADR
	RET




;	---BUMP---
;
;   BUMPS PARAMETERS FOR UNALLOCATED WRITES.
;   PARMS ARE CHANGED FOR THE NEXT PASS THRU
;   THE CODE (NOT THE CURRENT PASS).
;
;
BUMP:	LXI	H,U@CNT		;UNALC RECORD COUNT
	DCR	M		;  DECR IT
	DCX	H		;H/L = U@SEC
	INR	M		;  INCR IT
	MOV	A,M
	CPI	CPMSPT		;CPM SECTORS PER TRACK
	RC			;RIF STAY ON SAME TRACK
; OVERFLOW TO NEXT TRACK
	MVI	M,0		;RESET SECTOR ADR
	LHLD	U@TRK
	INX	H		;INCR TRACK ADR
	SHLD	U@TRK
	RET




;	---GET DMA ADDRESS---
;
;   SETS THE CONTROLLER BUFFER ADDRESS TO THE CORRECT
;   STARTING POINT. ALSO SETS B=128 & H/L=DMADR.
;
;
GETDMA:	LHLD	DMADR		;DMA ADR
	IF	Z80
	MVI	C,CTDP		;FOR BLOCK I/O
	ENDIF
	MVI	B,128		;B = COUNT
	MVI	A,DBENB
	OUT	CTCSR		;ENB DATA BFR
	LDA	R@SEC		;REQUESTED SECTOR
	RRC
	MVI	A,0
	JNC	GET1		;JIF USE 1ST HALF OF BFR
	MOV	A,B		;   USE 2ND HALF OF BFR
GET1:	OUT	CTBFR		;SET CTLR DATA BFR ADR
	RET




;	---BUFFER INPUT---
;
;   READ 128 BYTES FROM THE CONTROLLER BUFFER
;
;
BFRIN	EQU	$
	IN	CTDP		;PRIME DATA INPUT
	NOP			;TIME DELAY
	IF	Z80		;IF USING A Z-80
	INIR			;BLOCK INPUT
	RET
	ELSE			;IF USING AN 8080
BFRIN1:	IN	CTDP		;GET DATA
	MOV	M,A		;PUT DATA INTO RAM
	INX	H
	DCR	B
	JNZ	BFRIN1
	RET
	ENDIF




;	---BUFFER OUTPUT---
;
;   OUTPUT 128 BYTES TO THE CONTROLLER BUFFER
;
;
BFROUT	EQU	$
	IF	Z80		;IF USING A Z-80
	OUTIR			;BLOCK OUTPUT
	RET
	ELSE			;IF USING AN 8080
	MOV	A,M		;GET DATA FROM RAM
	OUT	CTDP		;PUT DATA INTO BFR
	INX	H
	DCR	B
	JNZ	BFROUT
	RET
	ENDIF




;	---DISK ADDRESS COMPARE---
;
;   COMPARES EITHER THE HOST DISK ADDRESS OR THE UNALLOCATED
;   DISK ADDRESS AGAINST THE REQUESTED DISK ADDRESS.
;
;
DSKCMP:	LXI	D,R@TRK		;REQUESTED TRACK
	MVI	B,3
DC1:	LDAX	D
	CMP	M
	RNZ			;RIF ADRS NOT EQUAL
	INX	H
	INX	D
	DCR	B
	JNZ	DC1
	RET


	PAGE



;	======================================
;	** HARD DISK I/O & SUPPORT ROUTINES **
;	======================================




;	---READ A BLOCK---
;
;
XREAD:	LXI	H,RTBL		;READ CMD TBL
	CALL	DORW		;READ
;
XR1:	MVI	A,0
	RZ			;RIF READ/WRITE OK
	INR	A
	STA	ERFLG		;SET ERROR FLAG
	RET




;	---WRITE A BLOCK---
;
;
XWRT:	LXI	H,WTBL		;WRITE CMD TBL
	CALL	DORW		;WRITE A SECTOR
	JMP	XR1		;SET ERROR FLAG




;	---EXECUTE READ/WRITE COMMANDS---
;
;
DORW:	SHLD	CTA		;SAVE CMD TBL ADR
	CALL	XSEK		;SEEK TO NEW TRACK (IF REQUIRED)
	RNZ			;RIF SEEK FAILED
	CALL	XSEL		;HEAD SELECT
	LHLD	CTA		;GET CMD TBL ADR
;
DO0:	MOV	A,M
	STA	RETRY		;SET RETRY COUNT
	INX	H
	MOV	A,M
	OUT	CTCSR		;ENB CMP BFR
	INX	H
	MOV	A,M
	OUT	CTBFR		;SET CMP BFR ADR
	INX	H
	SHLD	CTA		;SAVE CMD TBL ADR
;
	LXI	H,RCA		;REAL CYL ADR
	MVI	B,3
DO1:	MOV	A,M
	OUT	CTDP		;PUT HDR INFO INTO CMP BFR
	INX	H
	DCR	B
	JNZ	DO1
	LDA	H@SEC
	OUT	CTDP		;SET SECT ADR FOR COMPARE
;
DO2:	CALL	XRDY		;DRIVE READY ?
	RNZ			;  RIF NO
	LHLD	CTA		;CMD TBL ADR
	MOV	A,M		;A = CNTL BANK
	INX	H
	MOV	B,A
	OUT	CTCSR		;SLCT CNTL BANK
	MOV	A,M
	OUT	CTBFR		;SET START ADR
	INX	H
	MOV	A,B
	ORI	START
	OUT	CTCSR		;START R/W CMD
;
DO3:	CALL	WFD		;WAIT FOR READ/WRITE TO FINISH
	RC			;ABORT, TIME-OUT ERROR
	ANA	M		;TEST CTLR STATUS (0=OK)
	MOV	B,A
	IN	DRCSR		;DRIVE STATUS
	ANI	WRTFLT
	CNZ	CLRDF		;CIF CLEAR DRIVE FAULT
	ORA	B		;SET/CLEAR ERROR FLAG (0=OK)
	RZ			;RIF READ/WRITE OK
	LXI	H,RETRY
	DCR	M		;DECR RETRY COUNT
	JNZ	DO2		;JIF RETRY READ/WRITE
	JMP	SEF		;SET ERROR FLAG, EXIT




;	---WAIT FOR DONE---
;
;
WFD:	PUSH	H		;SAVE H/L
	LXI	H,0		;TIME-OUT DELAY COUNT
;
WFD1:	IN	CTCSR		;CTLR STATUS
	RRC
	JC	WFD2		;JIF DONE
	DCX	H		;DECR DELAY COUNT
	MOV	A,H
	ORA	L
	JNZ	WFD1		;JIF CON'T WAITING
; TIME-OUT ERROR
	OUT	CTCSR		;STOP THE CTLR
	POP	H		;RESTORE H/L
	MVI	A,1
	ORA	A		;SET CPM ERROR FLAG
	STC			;SET INTERNAL ERROR FLAG
	RET
;
WFD2:	POP	H		;RESET H/L
	IN	CTCSR		;GET NON-CHANGING STATUS
	MOV	B,A	
	XRA	A
	OUT	CTCSR		;STOP CTLR
	MOV	A,B
	RET




;	---RESTORE--
;
;
XTKZ:	LXI	H,0
	SHLD	RCA		;INIT REAL CYL ADR
	CALL	TZT		;TRACK ZERO TEST
	RZ			;RIF ALREADY AT TRACK ZERO
;
	LXI	H,511		;# CYLS WE CAN SEEK
	CALL	RTZ		;SEEK OUT
	RC			;ABORT, DRIVE NOT RDY
	RZ			;RIF AT TRACK 0
	LXI	H,LZONE+10-511
	CALL	RTZ		;TRY IT AGAIN
	RC			;ABORT, DRIVE NOT RDY
	RZ			;RIF FOUND TRACK ZERO THIS TIME
	JMP	SEF		;ABORT, RESTORE FAILED
;
;   SEEK OUTWARD
;
RTZ:	CALL	XRDY		;DRIVE READY ?
	STC
	RNZ			;ABORT, DRIVE NOT RDY
	MOV	A,L
	OUT	LOSC		;SET LSB OF SEEK COUNT
	MOV	A,H
	OUT	HISC		;SET MSB
	MVI	A,SKOUT
	OUT	EXTCMD		;SET SEEK DIRC = OUTWARD
	MVI	A,SKCMD
	OUT	DRCSR		;ISSUE SEEK CMD
	CALL	WSC		;WAIT FOR SEEK DONE
	RC			;ABORT, RESTORE FAILED TO FINISH
;
TZT:	IN	DRCSR		;DRIVE STATUS
	ANI	TK00		;TRACK ZERO INDICATOR
	XRI	TK00		;..MAKE IT LO-TRUE
	RET




;	---SEEK TO TRACK ZERO---
;
;
XSTZ:	TSTBIT	F@WIP
	JNZ	XSTZ1		;JIF WRITE IN PROGRESS
	STA	FLAGS		;MAKE HOST INACTIVE
XSTZ1:	LXI	H,0
	SHLD	R@TRK		;SET TK ADR = 0
	XRA	A		;SET FLAG = NO ERRORS
	RET




;	---SEEK---
;
;
XSEK:	MVI	A,3
	STA	SKRTC		;SET RETRY COUNT
;
XSEK1:	LHLD	H@TRK		;REQUESTED TRACK
	MVI	B,1		;2 HEADS
	CALL	DRS		;SHIFT H/L RIGHT 'N' TIMES
	LXI	D,MAXCYL	;# OF CYLS
	CALL	TSUB		;TEST REQUESTED CYL ADR
	RAL
	JNC	SEF		;JIF GOT INVALID CYL ADR
	LDA	H@DSK		;HOST DISK
	ORA	A
	JZ	XSEK2		;JIF LOGICAL DRIVE ZERO
	DAD	D		;DRIVE ONE USES INNER HALF OF DISK
;
XSEK2:	XCHG			;D/E = NEW CYL ADR
	LHLD	RCA		;REAL (CURR) CYL ADR
	XCHG
	SHLD	RCA		;UPDATE REAL CYL ADR
	XCHG
	CALL	SUBT		; (NEW CYL) - (OLD CYL)
	RZ			;RIF SAME CYL
	MVI	B,1		;DIRC = OUT
	RAL
	JNC	XSEK3		;JIF SEEK OUTWARD
; SEEK INWARD
	MVI	B,3		;DIRC = INWARD
	MOV	A,L		;MAKE SEEK COUNT POSITIVE
	CMA			  ;
	MOV	L,A		  ;
	MOV	A,H		  ;
	CMA			  ;
	MOV	H,A		  ;
	INX	H		  ;
;
XSEK3:	MOV	A,B		;A = SEEK DIRC
	STA	SKDIR		;..SAVE IT
	LXI	D,512
	CALL	TSUB
	RAL
	JNC	XSEK4		;JIF DOUBLE-PUMP (SEEK > 511 CYLS)
	CALL	PSK		;SEEK =< 511 CYLS
	RNZ			;ABORT, SEEK FAILED
	JMP	XSEK5
; DOUBLE PUMP SEEKS
XSEK4:	DCX	D		;D/E = 511
	CALL	SUBT		; (H/L) - (D/E)
	SHLD	RSKNT		;SAVE RESIDUAL SEEK CNT
	XCHG			;H/L = 511
	CALL	PSK		;SEEK 511 CYLS
	RNZ			;ABORT, SEEK FAILED
	LHLD	RSKNT		;RESIDUAL SEEK COUNT
	CALL	PSK		;SEEK REMAINDER
	RNZ
; SEEK VERIFY
XSEK5:	MVI	A,3
	STA	VSRTC		;SET RETRY COUNT
	MVI	A,CBENB
	OUT	CTCSR		;ENABLE BANK ZERO CMP BFR
	MVI	A,VCA
	OUT	CTBFR		;SET CMP BFR ADR
	LHLD	RCA		;REAL (CURR) CYL ADR
	MOV	A,L
	OUT	CTDP		;SET CYL ADR, LSB
	MOV	A,H
	OUT	CTDP		;SET CYL ADR, MSB
XSEK6:	MVI	A,VSA
	OUT	CTBFR		;SET M/CODE START ADR
	MVI	A,START
	OUT	CTCSR		;START VERIFY
	CALL	WFD		;WAIT FOR DONE
	ANI	0CH		;TEST CTLR STATUS
	RZ			;RIF VERIFY OK
	LXI	H,VSRTC
	DCR	M		;DECR RETRY COUNT
	JNZ	XSEK6		;JIF RETRY SEEK VERIFY
; VERIFY FAILED
	CALL	XTKZ		;RESTORE
	LXI	H,SKRTC
	DCR	M		;DECR RETRY COUNT
	JNZ	XSEK1		;JIF RETRY SEEK
; UNRECOVERABLE SEEK ERROR
	ORI	1		;SET ERROR FLAG
	RET			;ABORT
;
;   PARTIAL SEEK
;
PSK:	CALL	XRDY		;DRIVE READY ?
	RNZ			;ABORT, DRIVE NOT RDY
	MOV	A,L
	OUT	LOSC		;SET SEEK COUNT, LSB
	MOV	A,H
	OUT	HISC		;   * MSB
	LDA	SKDIR
	OUT	EXTCMD		;SET SEEK DIRECTION
	MVI	A,3
	OUT	DRCSR		;ISSUE SEEK CMD
;  ---> FALL THRU TO 'WSC' <---




;	---WAIT FOR SEEK COMPLETE---
;
;
WSC:	PUSH	H		;SAVE REGS
	PUSH	B
	LXI	H,0		;TIME-OUT DELAY COUNT
	MVI	B,3		;..DITTO
;
WSC1:	IN	DRCSR		;DRIVE STATUS
	RAL
	JC	WSC2		;JIF SEEK DONE
	DCX	H		;DECR DELAY COUNT
	MOV	A,H
	ORA	L
	JNZ	WSC1		;JIF CON'T WAITING
	DCR	B
	JNZ	WSC1
; TIME-OUT ERROR
	POP	B		;RESTORE REGS
	POP	H
	MVI	A,1
	ORA	A		;SET CPM ERROR FLAG
	STC			;SET INTERNAL ERROR FLAG
	RET
;
WSC2:	POP	B		;RESTORE REGS
	POP	H
	XRA	A		;SET FLAG = OK
	RET





;	---CLEAR DRIVE FAULT---
;
;
CLRDF:	XRA	A
	OUT	EXTCMD		;DE-SELECT THE DRIVE
				;  * FALL THRU TO 'XSEL' TO
				;    RE-SELECT THE DRIVE!





;	---HEAD SELECT---
;
;
XSEL:	LDA	H@TRK		;REQUESTED TRACK
	ANI	HEDMSK		;HEAD MASK
	STA	RHD		;SAVE REAL HEAD #
	ADD	A		;SHIFT HEAD # LEFT TWICE FOR H/W
	ADD	A
	ORI	1		;TO MAINTAIN DRIVE SLCT
	OUT	EXTCMD		;SELECT HEAD 0/1
	RET




;	---DRIVE READY TEST---
;
;
XRDY:	IN	DRCSR		;DRIVE STATUS
	ANI	1		;DRIVE READY BIT
	XRI	1		;INVERT IT
	RET			;IF A = 0, DRIVE RDY





;	---COMMAND TABLES---
;
;
;   WRITE
;
WTBL:	DB	5		;RETRY COUNT
	DB	5		;CMP BFR ENB
	DB	0E6H		;CMP BFR ADR
	DB	BANK1		;CNTL BANK
	DB	0D3H		;START ADR
	DB	0EH		;STATUS MASK
;
;   READ
;
RTBL:	DB	10		;RETRY COUNT
	DB	4		;CMP BFR ENB
	DB	0EAH		;CMP BFR ADR
	DB	BANK0		;CNTL BANK
	DB	0D7H		;START ADDRESS
	DB	0EH		;STATUS MASK


	PAGE


;	==========================
;	** FLOPPY DISK ROUTINES **
;	==========================




;	---SECTOR READ---
;
;
FREAD:	CALL	FSEK		;DO TRACK SEEK
	RNZ			;RIF HAD SEEK ERROR
	MVI	D,10		;C = RETRY COUNT
;
FRD1:	IN	D@DP		;CLEAR 'DRQ'
	LDA	RFD		;F/D DRIVE #
	ORI	AUTO		;TURN ON AUTO WAIT BIT
	OUT	D@CP1		;TURN ON AUTO WAIT
	LHLD	DMADR		;H/L = DMA ADR
	MVI	B,128
	MVI	A,C@RD
	OUT	D@CSR		;ISSUE READ CMD
FRD2:	IN	D@DP		;INPUT DATA FROM FDC
	MOV	M,A		;PUT DATA INTO RAM
	INX	H		;INCR ADR
	DCR	B
	JNZ	FRD2
	CALL	FDW		;WAIT FOR CMD TO FINISH
	ANI	S@RD		;TEST FOR  READ ERRORS
	RZ			;RIF READ OK
	ANI	NRB		;TEST FOR NOT READY
	RNZ			;RIF DRIVE NOT READY
	DCR	D		;DECR RETRY COUNT
	JNZ	FRD1		;RETRY READ
	JMP	SEF		;SET ERROR FLAG, EXIT



;	---SECTOR WRITE---
;
;
FWRT:	CALL	FSEK		;DO TRACK SEEK
	RNZ			;RIF HAD SEEK ERROR
	MVI	D,10		;C = RETRY COUNT
;
FWRT1:	IN	D@DP		;CLEAR 'DRQ'
	LDA	RFD		;DRIVE #
	ORI	AUTO		;TURN ON AUTO WAIT BIT
	OUT	D@CP1		;TURN ON AUTO WAIT
	LHLD	DMADR		;H/L = DMA ADR
	MVI	B,128
	MVI	A,C@WRT
	OUT	D@CSR		;ISSUE WRITE COMMAND
FWRT2:	MOV	A,M		;GET DATA FROM RAM
	OUT	D@DP		;OUTPUT TO FDC
	INX	H		;INCR ADR
	DCR	B
	JNZ	FWRT2
	CALL	FDW		;WAIT FOR CMD TO FINISH
	ANI	S@WRT		;FOR WRITE ERRORS
	RZ			;RIF WRITE OK
	ANI	NRB + WPROT	;NOT RDY & CAN'T WRITE
	RNZ			;RIF DRIVE NOT READY
	DCR	D		;DECR RETRY COUNT
	JNZ	FWRT1		;RETRY WRITE
	JMP	SEF		;SET ERROR FLAG, EXIT




;	---TRACK ZERO SEEK---
;
;
FTKZ:	CALL	FSEL		;SLCT NEW DRIVE (IF REQUIRED)
	RNZ			;RIF HAD ERROR
	CALL	FRDY		;ENSURE READY TO SEEK
	RNZ			;EXIT, DRIVE NOT READY
	MVI	A,C@TKZ
	CALL	DOCMD		;DO SEEK, WAIT FOR DONE
	ANI	S@TKZ		;TEST FOR ERRORS
	XRI	4		;INVERT +TKZ BIT
	RET



;	---TRACK SEEK---
;
;
FSEK:	CALL	FSEL		;SLCT NEW DRIVE (IF REQUIRED)
	RNZ			;RIF HAD ERROR
	CALL	FRDY		;ENSURE DRIVE RDY TO SEEK
	RNZ			;ABORT, DRIVE NOT RDY
	LDA	R@SEC
	OUT	D@SEC		;SET SECTOR ADR
	IN	D@TK		;CURR TK ADR
	LXI	H,R@TRK
	CMP	M		;SAME TRACK ?
	RZ			;YES, DON'T MOVE HEAD
	MVI	D,3		;SET RETRY COUNT
;
FSEK1:	MOV	A,M
	OUT	D@DP		;SET TK ADR FOR SEEK
	MVI	A,C@SEEK
	CALL	DOCMD		;DO SEEK, WAIT FOR DONE
	ANI	S@SEEK		;TEST FOR ERRORS
	RZ			;EXIT SEEK OK
	MOV	E,M		;SAVE CURRENT TRACK ADR
	CALL	FTKZ		;DO TRACK ZERO SEEK
	MOV	M,E		;RESTORE TRACK ADR
	RNZ			;EXIT IF TKZ ERROR
	DCR	D		;DECR RETRY COUNT
	JNZ	FSEK1		;RE-SEEK
	JMP	SEF		;SET ERROR FLAG, EXIT




;	---DRIVE SELECT---
;
;
FSEL:	LDA	R@DSK		;REQUESTED DISK
	SUI	MAXHD		;ADJUST FOR F/D
	CPI	MAXFD
	JNC	SEF		;JIF INVALID DRIVE
	LXI	H,CFD
	MOV	B,M		;B = CURR DRIVE #
	CMP	B
	RZ			;RIF SAME DRIVE
	MOV	M,A		;UPDATE CURR DRIVE #
	MOV	C,A		;C = NEW DR #
	MOV	L,A		;L = SHIFT COUNT (DRIVE #)
	INR	L
	XRA	A
	STC
FSEL1:	RAL			;SHIFT DRIVE SLCT BIT
	DCR	L
	JNZ	FSEL1
	ORI	MAXI		;FOR 8" DRIVES
	OUT	D@CP1		;SLCT NEW DRIVE
	STA	RFD		;  SAVE FOR FUTURE USE
	LXI	H,TKSAV		;TK ADR SAVE AREA
	MOV	E,B
	MVI	D,0
	DAD	D		;H/L = OLD DR SAVE
	IN	D@TK		;GET CURR TK ADR
	MOV	M,A		;SAVE TK ADR OF OLD DR
	LXI	H,TKSAV
	MOV	E,C
	DAD	D		;H/L = NEW DR SAVE
	MOV	A,M		;A = TK ADR OF NEW DRIVE
	OUT	D@TK		;SET 1791 TK REG
	OUT	D@DP		;PUT TK ADR INTO DATA REG
	CALL	FRDY		;DRIVE READY ?
	RNZ			;NO, EXIT
	MVI	A,C@SUN+STEP	;YES, DO SEEK TO SAME TRACK
	CALL	DOCMD		;   TO UNLOAD THE HEAD
	XRA	A		;NO ERRORS !
	RET




;	---EXECUTE CONTROLLER COMMANDS---
;
;
DOCMD:	OUT	D@CSR		;ISSUE 1791 CMD
				;  * FALL THRU TO 'FDW'




;	---WAIT FOR DONE---
;
;
FDW:	IN	D@CP1		;CNTL PORT #1
	RRC			;CY = INTRQ
	JNC	FDW		;WAIT FOR DONE
	IN	D@CSR		;GET 1791 STATUS
	RET




;	---READY TEST---
;
;
FRDY:	IN	D@CSR		;1791 STATUS
	ANI	80H		;TEST READY BIT
	RET


	PAGE


;	======================================
;	** SUBROUTINES, MESSAGES, & STORAGE **
;	======================================




;	---INITIALIZE FLAGS & VARIABLES---
;
;
INIT:	LXI	H,FLAGS
	MVI	B,FLGSIZ
	XRA	A
INIT1:	MOV	M,A		;CLEAR FLAGS & VARIABLES
	INX	H
	DCR	B
	JNZ	INIT1
	RET




;	---SET FLAG BIT---
;
;
@BSET:	PUSH	H
	LXI	H,FLAGS		;BIT FLAGS
	ORA	M		;OLD BITS
	MOV	M,A
	POP	H
	RET




;	---RESET BIT FLAG---
;
;
@BRES:	PUSH	H
	LXI	H,FLAGS		;BIT FLAGS
	ANA	M		;KILL UNWANTED BIT
	MOV	M,A
	POP	H
	RET




;	---TEST BIT FLAG---
;
;
@BTST:	PUSH	H
	LXI	H,FLAGS		;BIT FLAGS
	ANA	M		;TEST THE BIT
	POP	H
	RET




;	---DOUBLE RIGHT SHIFT---
;
;   SHIFTS THE CONTENTS OF H/L RIGHT 'N' TIMES.
;
DRS:	PUSH	PSW		;SAVE 'A'
;
DRS1:	MOV	A,H		;MSB
	ORA	A		;CLEAR CY
	RAR
	MOV	H,A
	MOV	A,L		;LSB
	RAR
	MOV	L,A
	DCR	B		;B = SHIFT COUNT
	JNZ	DRS1
	POP	PSW		;RESTORE 'A'
	RET





;	---PRINT ROUTINE---
;
;
PRNT:	MOV	A,M		;PRINT CHAR
	INX	H		;FOR NEXT CHAR
	ORA	A
	RZ			;RIF DONE
	MOV	C,A
	CALL	CO		;PRINT A CHAR
	JMP	PRNT		;DO NEXT CHAR




;	---CARRIAGE RETURN, LINE FEED----
;
;
CRLF:	MVI	C,CR
	CALL	CO		;PRINT A CR
	MVI	C,LF
	JMP	CO		;PRINT A LF, EXIT




;	---16 BIT SUBTRACTION---
;
;
SUBT:	MOV	A,L
	SUB	E		;LSB
	MOV	L,A
	MOV	A,H
	SBB	D		;MSB
	MOV	H,A
	ORA	L		;TEST FOR ZERO RESULTS
	MOV	A,H		;FOR EXTERNAL USE
	RET




;	---16 BIT COMPARISON TEST---
;
;
TSUB:	PUSH	H		;SAVE H/L
	CALL	SUBT		; (H/L) - (D/E)
	POP	H		;RESTORE H/L
	RET




;	---MESSAGES---
;
;
MSG1:	DB	CR,LF,BELL
	DB	'* Warm Boot Failed...Hit any key to Retry ',0



CODEND	EQU	$		;DEFINES END OF CODE






;	---PROGRAM STORAGE---
;
;
;
FLAGS:	DS	1		;BIT FLAGS
;
R@TRK:	DS	2		;REQUESTED TRACK ADDRESS
R@DSK:	DS	1		;   DRIVE #
R@SEC:	DS	1		;   SECTOR ADDRESS
;
U@TRK:	DS	2		;UNALLOCATED TRACK ADDRESS
U@DSK:	DS	1		;   DRIVE #
U@SEC:	DS	1		;   SECTOR ADDRESS
U@CNT:	DS	1		;   RECORD COUNT
;
H@TRK:	DS	2		;HOST (MS) TRACK ADDRESS
H@DSK:	DS	1		;   DRIVE #
H@SEC:	DS	1		;   SECTOR ADDRESS
;
RCA:	DS	2		;REAL CYLINDER ADDRESS
RHD:	DS	1		;  HEAD
RSA:	DS	1		;  SECTOR
;
SECNT:	DS	1		;SECTOR COUNT (FOR WARM BOOT)
RETRY:	DS	1		;RETRY COUNT
CTA:	DS	2		;COMMAND TABLE ADDRESS
ERFLG:	DS	1		;ERROR FLAG
TKSAV:	DS	4		;TRACK ADDRESS SAVE AREA (F/D)
DMADR:	DS	2		;BUFFER (DMA) ADDRESS
WRTMODE: DS	1		;WRITE MODE
RFD:	DS	1		;REAL F/D DRIVE SELECT INFO
SKRTC:	DS	1		;SEEK RETRY COUNT
VSRTC:	DS	1		;SEEK VERIFY RETRY COUNT
SKDIR:	DS	1		;SEEK DIRECTION
RSKNT:	DS	2		;RESIDUAL SEEK COUNT
;
FLGSIZ	EQU	$-FLAGS		;DEFINES SIZE OF VARIABLE STORAGE
;
CFD:	DS	1		;CURRENTLY SELECT F/D


	IF	TEST		;IF WE ARE TESTING
PATCH:	DS	64		;CREATE PATCH AREA FOR DEBUG
	ENDIF




;	---BDOS STORAGE---
;
;
DIRBFR:	DS	128		;DIRECTORY BUFFER
;
FALL0:	DS	31		;F/D ALLOCATION BUFFER, DRIVE 0
FALL1:	DS	31		;   DRIVE 1
FCHK0:	DS	16		;F/D CHECK BUFFER, DRIVE 0
FCHK1:	DS	16		;   DRIVE 1
;
HALL0:	DS	306		;BIT MAP, DRIVE 0
HALL1:	DS	306		;..DRIVE 1
;
LAST	EQU	$



	END
