	TITLE	'MHOFF -- MOVE HEADS TO LANDING ZONE (MS-3012)'

;	04-19-83



;   MHOFF is used to position the heads of a MiniScribe MS-3012 hard
;   disk drive over the Landing Zone. This program may be run before
;   removing power from the drive. This program **MUST** be run before
;   moving or shipping the drive. An  XCOMP STS/STR hard disk controller
;   is used.




;	---PROGRAM CONFIGURATION---
;
;
;   THE FOLLOWING STATEMENTS ARE USED TO CONFIGURE THE PROGRAM.
;
;	1. TEST -- SET THIS 'FALSE' BEFORE USING THE PROGRAM.
;
;	2. S100 -- SET THIS 'TRUE' IF YOU ARE USING A CONTROLLER WITH
;	   AN S-100 BUS INTERFACE (STS). SET THIS 'FALSE' IF YOU ARE
;	   USING A CONTROLLER WITH THE RIBBON CABLE INTERFACE (STR).
;
;
;
FALSE	EQU	0		;DEFINE FALSE
TRUE	EQU	NOT FALSE	;..TRUE
;
TEST	EQU	FALSE		;NOT TESTING
;
S100	EQU	TRUE		;S-100 CONTROLLER




;	---EQUATES---
;
;
STRT	EQU	100H		;PROGRAM START ADDRESS
STACK	EQU	100H		;STACK ADDRESS
LMSK	EQU	0FFH		;LSB MASK
WBOOT	EQU	0		;CPM WARM BOOT
BDOS	EQU	5		;BDOS ENTRY POINT
;
LZONE	EQU	656		;LANDING ZONE
;
ROMCTL	EQU	0FH		;ROM CONTROL PORT
ROMON	EQU	80H		;ENABLES ROM
DEBUG	EQU	0F000H		;ROM DEBUG ENTRY POINT
;
;   DRIVE COMMANDS
;
SKCMD	EQU	3		;SEEK COMMAND
SKOUT	EQU	1		;SEEK DIRECTION = OUTWARD
SKIN	EQU	3		;..INWARD
NOPC	EQU	40H		;NO PRE-COMPENSATION
LOWRT	EQU	80H		;LOW WRITE CURRENT
;
;   DRIVE STATUS
;
READY	EQU	1		;DRIVE READY
WRTFLT	EQU	2		;WRITE FAULT
TK00	EQU	4		;TRACK ZERO
RAWINDX	EQU	20H		;RAW INDEX
;
;   CONTROLLER COMMANDS
;
BANK0	EQU	0		;BANK 0 SELECT
BANK1	EQU	1		;BANK 1 SELECT
DBENB	EQU	2		;DATA BUFFER ENABLE
CBENB	EQU	4		;COMPARE BUFFER ENABLE
START	EQU	8		;START COMMAND
;
;   CONTROLLER STATUS
;
INDEX	EQU	10H		;DELAYED INDEX
SECTOR	EQU	20H		;SECTOR
;
;   DRIVE/CONTROLLER I/O
;
	IF	S100	; IF S-100 BUS COMPUTER
CBASE	EQU	70H		;BASE ADR OF THE CONTROLLER
DRCSR	EQU	CBASE		;DRIVE COMMAND/STATUS
EXTCMD	EQU	CBASE+1		;EXTENDED COMMNAND REGISTER
LOSC	EQU	CBASE+2		;SEEK COUNT, LSB
HISC	EQU	CBASE+3		;  * MSB
CTCSR	EQU	CBASE+4		;CONTROLLER COMMAND/STATUS
CTBFR	EQU	CBASE+5		;CONTROLLER BUFFER ADDRESS
CTDP	EQU	CBASE+6		;CONTROLLER DATA PORT
	ELSE		; IF RIBBON CABLE INTERFACE
CBASE	EQU	80H		;BASE ADR OF THE CONTROLLER
DRCSR	EQU	CBASE+7		;DRIVE COMMAND/STATUS
EXTCMD	EQU	CBASE+6		;EXTENDED COMMNAND REGISTER
LOSC	EQU	CBASE+5		;SEEK COUNT, LSB
HISC	EQU	CBASE+4		;  * MSB
CTCSR	EQU	CBASE+3		;CONTROLLER COMMAND/STATUS
CTBFR	EQU	CBASE+2		;CONTROLLER BUFFER ADDRESS
CTDP	EQU	CBASE+1		;CONTROLLER DATA PORT
	ENDIF
;
;   ASCII CHARACTERS 
;
CR	EQU	0DH		;CARRIAGE RETURN
LF	EQU	0AH		;LINE FEED
BELL	EQU	7		;DING!
ESC	EQU	1BH		;ESCAPE KEY
;
;   CPM FUNCTION CODES
;
F@CI	EQU	1		;CONSOLE INPUT
F@CO	EQU	2		;CONSOLE OUTPUT
F@PRT	EQU	9		;PRINT MESSAGES

	PAGE



;	============
;	** MACROS **
;	============



;	---CALL BDOS---
;
;
CPM	MACRO	FC
	MVI	C,FC		;; C = FUNCTION CODE
	CALL	BDOS		;; DO CPM FUNCTION
	ENDM




;	---PRINT MESSAGES---
;
;
PRNT	MACRO	MSG
	LXI	D,MSG		;; ADR OF THE MSG
	CPM	F@PRT		;; PRINT THE MSG
	ENDM

	PAGE


	ORG	STRT



;	---PROGRAM ENTRY POINT---
;
;
BGN:	LXI	SP,STACK
	PRNT	MSG1		;SAY HELLO
	CALL	COFF		;STOP ANY PRIOR ACTIVITY
	MVI	A,1
	OUT	EXTCMD		;SLCT THE DRIVE
	INR	A
	OUT	DRCSR		;ENABLE THE INTERFACE
;
LOOP:	CALL	TKZ		;RESTORE
	JNZ	ERROR		;JIF RESTORE FAILED
	CALL	SEEK		;SEEK TO INNER CYL
	JNZ	ERROR		;JIF SEEK FAILED
;
;   HEADS ARE NOW IN SHIP POSITION
;
DONE:	PRNT	MSG2		;SAY GOODBYE
	CALL	COFF		;TURN OFF DRIVE/CTLR
	IF	TEST	;; IF WE ARE TESTING
	JMP	0		;WARM BOOT
	ELSE		;; NORMAL MODE
	DI			;INHIBIT INTERRUPTS
	JMP	$		;DON'T DO ANYTHING ELSE
	ENDIF
;
;   ERROR RETRY ROUTINE
;
ERROR:	PRNT	MSG4		; 'TYPE ANY CHAR...'
	CPM	F@CI		;WAIT FOR OPER
	CPI	3
	JZ	WBOOT		;ABORT
	PRNT	MSG5		;DO CR.LF
	JMP	LOOP		;RETRY




;	---REZERO---
;
;
TKZ:	CALL	TZT		;TRACK ZERO TEST
	RZ			;RIF AT TRACK ZERO
;
	LXI	H,511		;# CYLS WE CAN SEEK
	CALL	RTZ		;SEEK OUT
	RZ			;RIF AT TRACK 0
	LXI	H,LZONE+10-511
	CALL	RTZ		;TRY IT AGAIN
	RZ			;RIF FOUND TRACK ZERO THIS TIME
; PRINT ERROR MSG
	PRNT	MSG6		; 'RESTORE FAILED'
	ORI	1		;SET ERROR FLAG
	RET
;
;   SEEK OUTWARD
;
RTZ:	CALL	RDY		;DRIVE READY ?  (RIF YES)
	MOV	A,L
	OUT	LOSC		;SET LSB OF SEEK COUNT
	MOV	A,H
	OUT	HISC		;SET MSB
	MVI	A,SKOUT
	OUT	EXTCMD		;SET SEEK DIRC = OUTWARD
	MVI	A,SKCMD
	OUT	DRCSR		;ISSUE SEEK CMD
	CALL	WSC		;WAIT FOR SEEK DONE
;
TZT:	IN	DRCSR		;DRIVE STATUS
	ANI	TK00		;TRACK ZERO INDICATOR
	XRI	TK00		;MAKE BIT LO-TRUE
	RET




;	---SEEK---
;
;
SEEK:	LXI	H,511
	CALL	PSK		;SEEK 511 CYLS
	LXI	H,LZONE-1-511
;
;   PARTIAL SEEK
;
PSK:	CALL	RDY		;DRIVE READY ?  (RIF YES)
	MOV	A,L
	OUT	LOSC		;SET SEEK COUNT, LSB
	MOV	A,H
	OUT	HISC		;   * MSB
	MVI	A,3
	OUT	EXTCMD		;SET SEEK DIRECTION = INWARD
	OUT	DRCSR		;ISSUE SEEK CMD
;
;   WAIT FOR SEEK COMPLETE
;
WSC:	IN	DRCSR		;DRIVE STATUS
	RAL
	JNC	WSC		;WAIT FOR SEEK DONE
	XRA	A
	RET




;	---DRIVE READY TEST---
;
;
RDY:	IN	DRCSR		;DRIVE STATUS
	ANI	1
	XRI	1
	RZ			;RIF DRIVE RDY
	PRNT	MSG3		; 'DRIVE NOT RDY'
	ORI	1		;SET ERROR FLAG
	RET




;	---TURN DRIVE/CONTROLLER OFF---
;
;
COFF:	XRA	A
	OUT	CTCSR		;STOP ANY PRIOR ACTIVITY
	OUT	DRCSR		;DISABLE THE INTERFACE
	OUT	EXTCMD		;DE-SELECT THE DRIVE
	RET




;	---MESSAGES---
;
;
MSG1:	DB	CR,LF,LF
	DB	'MHOFF -- Vers 1.0',CR,LF
	DB	'19-Apr-83',CR,LF,'$'
;
MSG2:	DB	CR,LF,'* Heads are over Landing Zone',CR,LF
	DB	'  OK to remove Drive Power!',CR,LF,'$'
;
MSG3:	DB	BELL,CR,LF,'* Drive not Ready',CR,LF,'$'
;
MSG4:	DB	CR,LF,'  Type CTRL-C to ABORT or any other character to RETRY $'
;
MSG5:	DB	CR,LF,'$'
;
MSG6:	DB	BELL,CR,LF,'* Restore Failed',CR,LF,'$'




	END
