	TITLE	'MHRDT -- MINISCRIBE MS-3012 RANDOM DATA TEST'

;	04-12-83



;   MHRDT is a random Read/Write test designed to ensure proper
;   operation of XCOMP controllers when used with a MiniScribe
;   MS-3012 hard disk drive.
;
;
;   NOTE -- Hitting the ESC key will terminate testing.






;	---PROGRAM CONFIGURATION---
;
;   THE FOLLOWING STATEMENTS ARE USED TO CONFIGURE THE PROGRAM.
;
;	1. TEST -- SET THIS 'FALSE' BEFORE USING THE PROGRAM.
;
;	2. S100 -- SET THIS 'TRUE' IF YOU ARE USING A CONTROLLER WITH
;	   AN S-100 BUS INTERFACE (STS). SET THIS 'FALSE' IF YOU ARE
;	   USING A CONTROLLER WITH A RIBBON CABLE INTERFACE (STR).
;
;
;
FALSE	EQU	0
TRUE	EQU	NOT FALSE
;
TEST	EQU	FALSE		;NOT TESTING
;
S100	EQU	TRUE		;S-100 CONTROLLER




;	---EQUATES---
;
;
STRT	EQU	100H		;PROGRAM START ADDRESS
STACK	EQU	100H		;STACK POINTER
LMSK	EQU	0FFH		;LSB MASK
WBOOT	EQU	0		;CPM WARM BOOT
BDOS	EQU	5		;BDOS ENTRY POINT
DEBUG	EQU	0F000H		;DEBUG ENTRY POINT
RNCON	EQU	3619H		;CONSTANT FOR RANDOM NUMBER GENERATOR
WORST	EQU	0B6D9H		;WORST CASE DATA PATTERN
;
VSA	EQU	8		;SEEK VERIFY START ADDRESS
VCA	EQU	1BH		;SEEK VERIFY COMPARE ADDRESS
;
WRTCNT	EQU	10		;WRITE RETRY COUNT
RDCNT	EQU	10		;READ RETRY COUNT
;
DR0	EQU	41H		;DRIVE 0, HEAD 0
DATA	EQU	0E5H		;WRITE FILL DATA
RPT	EQU	80H		;REPEAT
INDEX	EQU	10H		;INDEX BIT
;
MAXHD	EQU	2		;NUMBER OF HEADS
MAXCYL	EQU	612		;NUMBER OF CYLINDERS
LZONE	EQU	656		;LANDING ZONE
MAXSEC	EQU	32		;NUMBER OF SECTORS PER TRACK
;
;   DRIVE COMMANDS
;
STEPIN	EQU	3		;STEP DIRECTION = INWARD
STPOUT	EQU	1		;..OUTWARD
SKCMD	EQU	3		;SEEK COMMAND
NOPC	EQU	40H		;NO PRE-COMPENSATION
LOWRT	EQU	80H		;LOW WRITE CURRENT
;
;   DRIVE STATUS
;
READY	EQU	1		;DRIVE READY
WRTFLT	EQU	2		;WRITE FAULT
TK00	EQU	4		;TRACK ZERO
RAWINDX	EQU	20H		;RAW INDEX
;
;   CONTROLLER COMMANDS
;
BANK0	EQU	0		;BANK 0 SELECT
BANK1	EQU	1		;BANK 1 SELECT
DBENB	EQU	2		;DATA BUFFER ENABLE
CBENB	EQU	4		;COMPARE BUFFER ENABLE
START	EQU	8		;START COMMAND
;
;   DRIVE/CONTROLLER I/O
;
	IF	S100	; IF S-100 BUS INTERFACE
CBASE	EQU	70H		;BASE ADR OF THE CONTROLLER
DRCSR	EQU	CBASE		;DRIVE COMMAND/STATUS
EXTCMD	EQU	CBASE+1		;EXTENDED COMMNAND REGISTER
LOSC	EQU	CBASE+2		;SEEK COUNT, LSB
HISC	EQU	CBASE+3		;  * MSB
CTCSR	EQU	CBASE+4		;CONTROLLER COMMAND/STATUS
CTBFR	EQU	CBASE+5		;CONTROLLER BUFFER ADDRESS
CTDP	EQU	CBASE+6		;CONTROLLER DATA PORT
	ELSE		; IF RIBBON CABLE INTERFACE
CBASE	EQU	80H		;BASE ADR OF THE CONTROLLER
DRCSR	EQU	CBASE+7		;DRIVE COMMAND/STATUS
EXTCMD	EQU	CBASE+6		;EXTENDED COMMNAND REGISTER
LOSC	EQU	CBASE+5		;SEEK COUNT, LSB
HISC	EQU	CBASE+4		;  * MSB
CTCSR	EQU	CBASE+3		;CONTROLLER COMMAND/STATUS
CTBFR	EQU	CBASE+2		;CONTROLLER BUFFER ADDRESS
CTDP	EQU	CBASE+1		;CONTROLLER DATA PORT
	ENDIF
;
;   ASCII CHARACTERS 
;
CR	EQU	0DH		;CARRIAGE RETURN
LF	EQU	0AH		;LINE FEED
BELL	EQU	7		;DING!
ESC	EQU	1BH		;ESCAPE KEY
;
;   CPM FUNCTION CODES
;
F@CI	EQU	1		;CONSOLE INPUT
F@CO	EQU	2		;CONSOLE OUTPUT
F@PRNT	EQU	9		;PRINT MESSAGES
F@RCB	EQU	10		;READ CONSOLE BUFFER
F@REQ	EQU	11		;GET CONSOLE STATUS

	PAGE



;	============
;	** MACROS **
;	============



;	---CALL BDOS---
;
;
CPM	MACRO	FC
	CALL	SYSTEM		;; SYSTEM CALL ROUTINE
	DB	FC		;; CPM FUNCTION CODE
	ENDM




;	---PRINT MESSAGES---
;
;
PRNT	MACRO	MSG
	LXI	D,MSG		;; ADR OF THE MSG
	CPM	F@PRNT		;; PRINT THE MSG
	ENDM

	PAGE

	ORG	STRT	



;	==========================
;	** PROGRAM ENTRY POINT **
;	==========================
;
;
;
BGN:	LXI	SP,STACK	;SET SP
	CALL	COFF		;STOP ANY PRIOR ACTIVITY
	PRNT	MSG1		;SAY HELLO
	MVI	A,DR0
	OUT	EXTCMD		;SLCT DRIVE 0, HEAD 0, NO PRE-COMP
	MVI	A,2
	OUT	DRCSR		;ENABLE INDEX/SECTOR/CLOCKS TO DATA BOARD
	CALL	INZVAR		;INIT VARIABLES
	CALL	SRW		;SET READ/WRITE MICROCODE
;
;   COMMAND LOOP
;
LOOP:	LXI	SP,STACK	;RESET STACK PNTR
	PRNT	MSG1A		;PROMPT OPER
;
LP1:	CPM	F@REQ		;STATUS REQUEST
	JNZ	LP2		;JIF OPER HIT KEY
	LHLD	SEED
	INX	H		;INCR RANDOM NUMBER
	SHLD	SEED
	JMP	LP1
;
LP2:	MVI	A,1
	CALL	RCB		;GET OPER CMD
	CPI	'S'	
	JZ	STST		;JIF START TESTING
	CPI	'C'
	JZ	TLP		;JIF CON'T TESTING
	CPI	'X'
	JZ	XIT		;JIF EXIT TO CPM
; INVALID INPUT
INV:	PRNT	MSG19		; 'INVALID INPUT'
	JMP	LOOP		;RETRY COMMAND
;
;   EXIT TO CPM
;
XIT:	CALL	CRLF
	CALL	COFF		;TURN DRIVE/CTLR OFF
	IF	TEST
	JMP	DEBUG
	ELSE
	JMP	WBOOT		;BYE!
	ENDIF
;
;   START TESTING
;
STST:	CALL	INZVAR		;INIT VARIABLES
	CALL	TKZ		;RESTORE
	CALL	WAS		;WRITE CYL 0, HEAD 0, SECTOR 0
;
;   TEST LOOP
;
TLP:	PRNT	MSG25		; 'TESTING'
;
TLP1:	CALL	RNG		;GET RANDOM NUM
	LXI	D,MAXCYL
	CALL	DIVIDE		;DIVIDE TO FORM RANDOM CYLINDER
	XCHG
	SHLD	CYL		;PICK RANDOM CYL
	CALL	RNG		;GET RANDOM NUM
	LXI	D,MAXHD
	CALL	DIVIDE		;DIVIDE TO FORM RANDOM HEAD
	MOV	A,E
	STA	HEAD		;PICK RANDOM HEAD
	CALL	RNG		;GET RANDOM NUM
	LXI	D,MAXSEC
	CALL	DIVIDE		;FORM SECT ADR
	MOV	A,E
	STA	SECTOR		;PICK RANDOM SECTOR
	CALL	WAS		;WRITE A SECTOR
	CALL	RAS		;READ PREVIOS SECTOR
	LXI	H,PASS
	CALL	BUMP		;BUMP PASS COUNT
	LXI	H,CYL
	LXI	D,OLDCYL
	MVI	B,4
	CALL	MOVE		;MAKE CURRENT RANDOM ADR INTO OLD ADR
	CALL	STOP		;STOP TESTING ?  (RIF NO)
	JMP	TLP1		;KEEP GOING

	PAGE




;	=============================
;	** FORMAT, WRITE, & VERIFY **
;	=============================




;	---WRITE A SECTOR---
;
;
WAS:	MVI	A,WRTCNT
	STA	RETRY		;SET WRITE RETRY COUNT
	LXI	H,CYL
	LXI	D,TCYL
	MVI	B,4
	CALL	MOVE		;MAKE CURRENT DISK ADR INTO TARGET
	CALL	PUTHDR		;PUT HDR INFO INTO DATA BFR
	CALL	SEEK		;SEEK TO NEW CYL
;
WAS1:	LXI	H,WTBL
	CALL	DORW		;WRITE A SECTOR
	JZ	WAS2		;JIF WRITE OK
	LXI	H,RETRY
	DCR	M		;DECR RETRY COUNT
	JNZ	WAS1		;JIF RETRY WRITE
	LXI	D,MSG10		; 'WRITE ERROR'
	CALL	PEM		;PRINT ERROR MSG
	LXI	H,HWERR
	JMP	BUMP		;BUMP HARD WRITE ERROR COUNT, EXIT
;
WAS2:	LDA	RETRY
	SUI	WRTCNT
	RZ			;RIF NO SOFT ERRORS
	LXI	H,SWERR
	JMP	BUMP		;BUMP SOFT ERR CNT, EXIT






;	---READ A SECTOR---
;
;
RAS:	MVI	A,RDCNT
	STA	RETRY		;SET RETRY COUNT
	LXI	H,OLDCYL
	LXI	D,TCYL
	MVI	B,4
	CALL	MOVE		;MAKE OLD DISK ADR INTO TARGET
	CALL	SEEK		;SEEK TO NEW CYL
;
RAS1:	LXI	H,RTBL
	CALL	DORW		;READ THE SECTOR
	JZ	RAS2		;JIF READ OK
	LXI	H,RETRY
	DCR	M		;DECR RETRY COUNT
	JNZ	RAS1		;JIF RETRY READ
	LXI	D,MSG11
	CALL	PEM		;PRINT ERR MSG
	LXI	H,HRERR
	JMP	BUMP		;BUMP HARD READ ERROR CNT, EXIT
;
RAS2:	CALL	TSTHDR		;TEST HDR INFO (IN DATA BFR)
	JZ	RAS3		;JIF DATA OK
	LXI	D,MSG8		; 'VERIFY ERROR'
	CALL	PEM		;PRINT ERROR MSG
	LXI	H,DVERR
	CALL	BUMP		;BUMP VERIFY ERROR CNT
;
RAS3:	LDA	RETRY
	SUI	RDCNT
	RZ			;RIF NO SOFT ERRORS
	LXI	H,SRERR
	JMP	BUMP		;BUMP SOFT READ ERROR CNT, EWXIT

	PAGE


;	============================
;	** DISK RELATED FUNCTIONS **
;	============================




;	---EXECUTE READ/WRITE COMMANDS---
;
;
DORW:	CALL	RDY		;DRIVE READY ?  (RIF YES)
	MOV	A,M
	OUT	CTCSR		;ENB CMP BFR
	INX	H
	MOV	A,M
	OUT	CTBFR		;SET CMP BFR ADR
	INX	H
;
	XCHG			;D/E = TBL ADR
	LXI	H,TCYL		;TARGET DISK ADR
	MVI	B,4
DO1:	MOV	A,M
	OUT	CTDP		;PUT HDR INFO INTO CMP BFR
	INX	H
	DCR	B
	JNZ	DO1
	XCHG			;H/L = TBL ADR
;
	MOV	A,M		;A = CNTL BANK
	INX	H
	MOV	B,A
	OUT	CTCSR		;SLCT CNTL BANK
	MOV	A,M
	OUT	CTBFR		;SET START ADR
	INX	H
	MOV	A,B
	ORI	START
	OUT	CTCSR		;START R/W CMD
;
DO2:	CALL	WFD		;WAIT FOR CMD TO FINISH
	STA	CSTAT		;SAVE CTLR STATUS
	ANA	M		;TEST CTLR STATUS (0=OK)
	MOV	B,A
	IN	DRCSR		;DRIVE STATUS
	STA	DSTAT		;  SAVE IT
	ANI	WRTFLT		;WRITE FAULT ?
	CNZ	CLRDF		;CIF CLEAR DRIVE FAULT
	ORA	B		;SET/CLEAR ERROR FLAG (0=OK)
	RET




;	---WAIT FOR DONE---
;
;
WFD:	PUSH	H		;SAVE H/L
	LXI	H,0		;DELAY COUNT
;
WFD1:	IN	CTCSR		;CTLR STATUS
	RRC
	JC	WFD2		;JIF READ/WRITE DONE
	DCX	H		;DECR DELAY COUNT
	MOV	A,H
	ORA	L
	JNZ	WFD1		;JIF CON'T WAITING
	OUT	CTCSR		;STOP THE CTLR
	PRNT	MSG24		; 'READ/WRITE FAILED'
	CALL	REPORT		;DISPLAY COUNTS
	JMP	LOOP		;ABORT
;
WFD2:	POP	H		;RESTORE H/L
	IN	CTCSR		;GET NON-CHANGING STATUS
	MOV	B,A		;..SAVE IT
	XRA	A
	OUT	CTCSR		;STOP CTLR
	MOV	A,B
	RET




;	---RESTORE---
;
;
TKZ:	IN	DRCSR		;DRIVE STATUS
	ANI	TK00		;TRACK ZERO FLAG
	RNZ			;RIF ALREADY AT TRACK ZERO
;
	LXI	H,511		;# CYLS WE CAN SEEK
	CALL	RTZ		;SEEK OUT
	RNZ			;RIF AT TRACK 0
	LXI	H,LZONE+10-511
	CALL	RTZ		;TRY IT AGAIN
	RNZ			;RIF FOUND TRACK ZERO THIS TIME
; PRINT ERROR MSG
	PRNT	MSG6		; 'RESTORE FAILED'
	CALL	PDS		;PRINT DRIVE STATUS
	CALL	CRLF		;DO CR.LF
	JMP	LOOP		;ABORT
;
;   SEEK OUTWARD
;
RTZ:	CALL	RDY		;DRIVE READY ?  (RIF YES)
	MOV	A,L
	OUT	LOSC		;SET LSB OF SEEK COUNT
	MOV	A,H
	OUT	HISC		;SET MSB
	MVI	A,1
	OUT	EXTCMD		;SET SEEK DIRC = OUTWARD
	MVI	A,SKCMD
	OUT	DRCSR		;ISSUE SEEK CMD
	CALL	WSC		;WAIT FOR SEEK DONE
	IN	DRCSR		;DRIVE STATUS
	ANI	TK00		;TRACK ZERO INDICATOR
	RET




;	---SEEK TO TRACK ZERO---
;
;
STZ:	LXI	H,0
	SHLD	TCYL		;SET CYL ADR = 0
				;  * FALL THRU TO 'SEEK'




;	---SEEK---
;
;
SEEK:	MVI	A,3
	STA	SKRTC		;SET RETRY COUNT
;
SEK0:	LHLD	CCA		;CURR CYL ADR
	XCHG
	LHLD	TCYL		;TARGET CYL
	SHLD	CCA		;UPDATE CURR CYL ADR
	XCHG
	CALL	SUBT		; (H/L) - (D/E)
	JZ	HDSEL		;NO NEED TO SEEK, BUT SLCT NEW HEAD
	MVI	B,1		;DIRC = OUT
	RAL
	JNC	SEK1		;JIF SEEK OUTWARD
; SEEK INWARD
	MVI	B,3		;DIRC = INWARD
	MOV	A,L		;MAKE SEEK COUNT POSITIVE
	CMA			  ;
	MOV	L,A		  ;
	MOV	A,H		  ;
	CMA			  ;
	MOV	H,A		  ;
	INX	H		  ;
;
SEK1:	MOV	A,B		;A = SEEK DIRC
	STA	SKDIR		;..SAVE IT
	LXI	D,512
	CALL	TSUB
	JNC	SEK2		;JIF DOUBLE-PUMP (SEEK > 511 CYLS)
	CALL	PSK		;SEEK =< 511 CYLS
	JMP	SEK3
; DOUBLE PUMP SEEKS
SEK2:	DCX	D		;D/E = 511
	CALL	SUBT		; (H/L) - (D/E)
	SHLD	RSKNT		;SAVE RESIDUAL SEEK CNT
	XCHG			;H/L = 511
	CALL	PSK		;SEEK 511 CYLS
	LHLD	RSKNT		;RESIDUAL SEEK COUNT
	CALL	PSK		;SEEK REMAINDER
; SEEK VERIFY
SEK3:	MVI	A,3
	STA	VSRTC		;SET RETRY COUNT
	MVI	A,CBENB
	OUT	CTCSR		;ENABLE BANK ZERO CMP BFR
	MVI	A,VCA
	OUT	CTBFR		;SET CMP BFR ADR
	LHLD	CCA		;CURR CYL ADR
	MOV	A,L
	OUT	CTDP		;SET CYL ADR, LSB
	MOV	A,H
	OUT	CTDP		;SET CYL ADR, MSB
SEK4:	MVI	A,VSA
	OUT	CTBFR		;SET M/CODE START ADR
	MVI	A,START
	OUT	CTCSR		;START VERIFY
	CALL	WFD		;WAIT FOR DONE
	ANI	0CH		;TEST CTLR STATUS
	JZ	HDSEL		;VERIFY OK...SLCT HEAD, EXIT
	LXI	H,VSRTC
	DCR	M		;DECR RETRY COUNT
	JNZ	SEK4		;JIF RETRY SEEK VERIFY
; VERIFY FAILED
	LXI	H,SKERR
	CALL	BUMP		;INCR SEEK ERROR COUNT
	CALL	TKZ		;RESTORE
	LXI	H,0
	SHLD	CCA		;SET CURR CYL ADR = 0
	LXI	H,SKRTC
	DCR	M		;DECR RETRY COUNT
	JNZ	SEK0		;JIF RETRY SEEK
; UNRECOVERABLE SEEK ERROR
	PRNT	MSG21		; 'SEEK FAILED'
	LHLD	TCYL
	CALL	P3D		;PRINT CYL ADR
	CALL	CRLF		;DO CR.LF
	JMP	LOOP		;ABORT
;
;   PARTIAL SEEK
;
PSK:	CALL	RDY		;DRIVE READY ?  (RIF YES)
	MOV	A,L
	OUT	LOSC		;SET SEEK COUNT, LSB
	MOV	A,H
	OUT	HISC		;   * MSB
	LDA	SKDIR
	OUT	EXTCMD		;SET SEEK DIRECTION
	MVI	A,3
	OUT	DRCSR		;ISSUE SEEK CMD
	JMP	WSC		;WAIT FOR SEEK DONE, EXIT




;	---WAIT FOR SEEK COMPLETE---
;
;
WSC:	LXI	H,0		;DELAY COUNT
	MVI	B,3
;
WSC1:	IN	DRCSR		;DRIVE STATUS
	RAL
	JC	WSC2		;JIF SEEK DONE
	DCX	H		;DECR DELAY COUNT
	MOV	A,H
	ORA	L
	JNZ	WSC1		;JIF CON'T WAITING
	DCR	B
	JNZ	WSC1		;JIF CON'T WAITING
	PRNT	MSG23		; 'SEEK FAILED'
	CALL	REPORT		;DISPLAY COUNTS
	JMP	LOOP		;ABORT
;
WSC2:	RAR
	STA	DSTAT		;SAVE DRIVE STATUS
	RET




;	---HEAD SELECT---
;
;
HDSEL:	LDA	THEAD		;TARGET HEAD
	RLC			;SHIFT FOR H/W
	RLC
	ORI	1		;TO MAINTAIN DRIVE SLCT
	OUT	EXTCMD		;SELECT THE HEAD
	RET




;	---DRIVE READY TEST---
;
;
RDY:	IN	DRCSR		;DRIVE STATUS
	RRC
	RC			;RIF DRIVE READY
	PRNT	MSG12		;PRINT ERROR MSG
	JMP	LOOP		;ABORT




;	---CLEAR DRIVE FAULT---
;
;
CLRDF:	XRA	A
	OUT	EXTCMD		;DE-SELECT THE DRIVE
	JMP	HDSEL		;RE-SELECT DRIVE & CORRECT HEAD




;	---SET READ/WRITE MICROCODE---
;
;
SRW:	LXI	H,RDCODE
	CALL	SQUIRT		;SET READ M/CODE
	LXI	H,WRTCODE
	JMP	SQUIRT		;SET WRITE M/CODE, EXIT




;	---PUT HEADER INFO INTO DATA BUFFER---
;
;
PUTHDR:	CALL	STP		;SET TEST PAMS
;
PH1:	MOV	A,M
	OUT	CTDP		;OUTPUT HDR INFO
	INX	H
	DCR	B
	JNZ	PH1
; FILL REST OF BFR WITH WORST CASE DATA
	LXI	H,WORST		;WORST CASE DATA PATTERN
	MVI	B,(256-4)/2	;LOOP COUNT
PH2:	MOV	A,L
	OUT	CTDP
	MOV	A,H
	OUT	CTDP
	DCR	B
	JNZ	PH2
	RET




;	---TEST HEADER INFO (IN DATA BUFFER)---
;
;
TSTHDR:	CALL	STP		;SET TEST PAMS
	IN	CTDP		;PRIME INPUT DATA
	NOP			;TIME DELAY
;
TH1:	IN	CTDP		;GET DATA FROM BFR
	CMP	M
	RNZ			;RIF COMPARE ERROR
	INX	H
	DCR	B
	JNZ	TH1
	RET			;EXIT, DATA COMPARES




;	---SET TEST PARAMETERS----
;
;
STP:	MVI	A,DBENB
	OUT	CTCSR		;ENABLE DATA BUFFER
	XRA	A
	OUT	CTBFR		;INIT BFR ADR
	LXI	H,TCYL		;ADR OF HDR INFO
	MVI	B,4		;NUM BYTES IN HDR
	RET




;	---FILL CONTROLLER COMMAND BUFFER---
;
;
SQUIRT:	MOV	A,M
	OUT	CTCSR		;SLCT BFR
	INX	H
	MOV	A,M
	OUT	CTBFR		;SET CTLR BFR ADR
	INX	H
;
SQ1:	MOV	A,M
	ORA	A
	RZ			;RIF DONE
	INX	H		;INCR TBL ADR
	MOV	B,A		;B = COUNT
	RAL
	JC	SQ3		;JIF REPEAT
;
SQ2:	MOV	A,M
	OUT	CTDP		;OUTPUT CONTROL INFO
	INX	H
	DCR	B
	JNZ	SQ2
	JMP	SQ1		;DO NEXT STRING
;
SQ3:	ORA	A		;CLEAR CY
	RAR
	MOV	B,A		;B = COUNT
	MOV	A,M
	INX	H
SQ4:	OUT	CTDP		;OUTPUT REPEAT CONTROL INFO
	DCR	B
	JNZ	SQ4
	JMP	SQ1		;DO NEXT STRING




;	---COMMAND TABLES---
;
;
;
;   WRITE
;
WTBL:	DB	5		;CMP BFR ENB
	DB	0E6H		;CMP BFR ADR
	DB	BANK1		;CNTL BANK
	DB	0D3H		;START ADR
	DB	0EH		;STATUS MASK
;
;   READ
;
RTBL:	DB	4		;CMP BFR ENB
	DB	0EAH		;CMP BFR ADR
	DB	BANK0		;CNTL BANK
	DB	0D7H		;START ADDRESS
	DB	0EH		;STATUS MASK





;	---WRITE MICROCODE---
;
;
WRTCODE	EQU	$		;  MINISCRIBE     06-09-81
;
	DB	1		
	DB	0D3H		
	DB	2,40H		
	DB	41H		
	DB	RPT+3,40H	
	DB	RPT+12,6DH	
	DB	1,63H		
	DB	RPT+4,65H	
	DB	RPT+2,67H	
	DB	2,49H		
	DB	40H		
	DB	RPT+17,50H	
	DB	5,5BH		
	DB	0DDH		
	DB	5DH		
	DB	50H		
	DB	0FH		
	DB	0		




;	---READ MICROCODE---
;
;
RDCODE	EQU	$		;  MINISCRIBE     06-09-81
;
	DB	0		
	DB	0D7H		
	DB	2,40H		
	DB	41H		
	DB	RPT+3,40H	
	DB	RPT+12,6DH	
	DB	1,63H		
	DB	RPT+4,65H	
	DB	RPT+2,67H	
	DB	1,49H		
	DB	RPT+2,40H	
	DB	RPT+12,6DH	
	DB	3,63H		
	DB	0E7H		
	DB	67H		
	DB	RPT+7,0FH	
;
	DB	2,40H		
	DB	41H		
	DB	RPT+3,40H	
	DB	RPT+12,6DH	
	DB	1,63H		
	DB	RPT+2,65H	
	DB	RPT+2,60H	
	DB	RPT+2,67H	
	DB	1,0FH		
	DB	0		

	PAGE


;	======================================
;	** SUBROUTINES, MESSAGES, & STORAGE **
;	======================================




;	---CALL CPM---
;
;   USED TO GO TO CPM (VIA BDOS) TO PERFORM NON-DISK
;   I/O. ALL REGISTERS (EXCEPT 'A') ARE SAVED & THEN
;   RESTORED AFTER THE FUNCTION IS COMPLETE. THE
;   CALLING SEQUENCE IS ---
;
;	CALL	SYSTEM		;ENTRY POINT
;	DB	FC		;FUNCTION CODE
;
;
SYSTEM:	XTHL			;H/L = ADR OF F.C.
	PUSH	D		;SAVE REGS
	PUSH	B
	MOV	C,M		;C = FUNCTION CODE
	INX	H		;H/L = RETURN ADR
	PUSH	H		;  SAVE IT
	CALL	BDOS		;DO CPM FUNCTION
	POP	H		;RESTORE RETURN ADR
	POP	B		;RESTORE REGS
	POP	D
	XTHL			;STACK = RET ADR
	ORA	A		;SET 8080 FLAGS
	RET



;	---INITIALIZE VARIABLES---
;
;
INZVAR:	LXI	H,VARBGN	;START OF VARIABLES
	MVI	B,VARSIZ	;NUMBER OF VARIABLES
	XRA	A
INZ1:	MOV	M,A
	INX	H
	DCR	B
	JNZ	INZ1
	RET




;	---READ CONSOLE BUFFER---
;
;   READS A STRING OF ASCII CHARACTERS. LOWER CASE CHARACTERS ARE
;   TRANSLATED TO UPPER CASE. USES THE CPM 'READ BUFFER' FUNCTION.
;
;
;	ENTRY:	A = INPUT CHARACTER COUNT
;
;	EXIT:	A = FIRST CHARACTER IN THE STRING
;	      D/E = BUFFER ADDRESS OF THE FIRST CHARACTER
;
;
;   ERRORS - - IF AN ERROR OCCURES, 'RCB' IS ABORTED & THE
;	       PROGRAM RESUMES RUNNING AT 'LOOP'.
;
;
RCB:	LXI	D,RDBFR		;READ BFR
	INR	A		;FOR N+1 CHARS
	STAX	D		;SET CHAR COUNT
	MOV	B,A		;..SAVE THE COUNT
	CPM	F@RCB		;READ THE STRING
	CALL	CRLF		;DO CR.LF
	LXI	H,RDBFR+1
	MOV	A,M		;A = INPUT CHAR COUNT
	ORA	A
	JZ	INV		;ERROR, NO CHARS INPUT
	CMP	B		;B = MAX CHAR CNT +1
	JNC	INV		;ERROR, TOO MANY INPUT CHARS
; CONVERT L/C TO U/C
	MOV	B,A		;B = # CHARS TO CONVERT
RCB1:	INX	H		;INCR BFR ADR
	MOV	A,M
	CALL	UCT		;CONVERT
	MOV	M,A
	DCR	B
	JNZ	RCB1
	LXI	D,RDBFR+2	;D/E = ADR OF 1ST CHAR IN STRING
	LDAX	D		;..GET IT
	RET




;	---UPPER CASE TRANSLATION---
;
;   CONVERTS LOWER CASE CHARACTERS TO UPPER CASE.
;
;
UCT:	ANI	7FH		;KILL PARITY BIT
	CPI	61H
	RC			;RIF NOT L/C CHAR
	CPI	7BH	
	RNC			;RIF NOT L/C CHAR
	SUI	20H		;DO CONVERSION
	RET




;	---RANDOM NUMBER GENERATOR---
;
;
RNG:	PUSH	D
	PUSH	B		;SAVE REGS
	LHLD	SEED		;CURRENT RANDOM NUMBER
	MOV	D,H
	MOV	E,L		;COPY TO D/E
	MVI	B,11
RNG1:	DAD	H		;SHIFT H/L LEFT 11 TIMES
	DCR	B
	JNZ	RNG1
	DAD	D		;ADD ORIG #
	DAD	H
	DAD	H		;SHIFT LEFT TWICE
	DAD	D		;ADD ORIG #
	LXI	D,RNCON
	DAD	D		;ADD CONSTANT
	SHLD	SEED		;SAVE NEW RANDOM NUMBER
	POP	B
	POP	D		;RESTORE REGS
	MOV	A,H		;MSB MOST RANDOM
	RET





;	---DIVIDE---
;
;   ENTRY:	H/L = DIVIDEND
;		D/E = DIVISOR
;
;   EXIT:	H/L = QUOTIENT
;		D/E = REMAINDER
;
;
DIVIDE:	XRA	A		;CLEAR SHIFT COUNT
	XCHG			;H/L = DIVISOR
;
DIV1:	INR	A		;INCR SHIFT COUNT
	DAD	H		;SHIFT H/L LEFT UNTIL A ONE
	JNC	DIV1		;  BIT DROPS OFF THE END
	XCHG
	MOV	C,E
	MOV	B,D		;B/C = DIVISOR
	LXI	D,0		;CLEAR QUOTIENT
;
DIV2:	PUSH	PSW		;SAVE SHIFT COUNT
	MOV	A,B		;  (CY SET 1ST TIME ONLY)
	RAR
	MOV	B,A
	MOV	A,C
	RAR
	MOV	C,A		;SHIFT DIVISOR RIGHT
	MOV	A,E
	STC
	RAL
	MOV	E,A
	MOV	A,D
	RAL
	MOV	D,A		;SHIFT QUOTIENT LEFT, SET LSB
	MOV	A,L
	SUB	C
	MOV	L,A
	MOV	A,H
	SBB	B
	MOV	H,A		;DIVIDEND = DIVIDEND - DIVISOR
	JNC	DIV3		;JIF NO BORROW
	DAD	B		;RESTORE DIVIDEND
	DCR	E		;CLEAR LSB OF QUOTIEENT
;
DIV3:	POP	PSW		;A = SHIFT COUNT
	ORA	A		;CLEAR CY
	DCR	A		;DECR SHIFT COUNT
	JNZ	DIV2		;JIF MORE TO DO
	XCHG
	RET




;	---16 BIT SUBTRACTION---
;
;   (H/L) - (D/E)
;
SUBT:	MOV	A,L
	SUB	E		;LSB
	MOV	L,A
	MOV	A,H
	SBB	D		;MSB
	MOV	H,A
	ORA	L		;TEST RESULTS FOR ZERO
	MOV	A,H		;FOR EXTERNAL USE
	RET




;	---16 BIT SUBTRACTION TEST ---
;
;
TSUB:	MOV	A,L		; (H/L) - (D/E)
	SUB	E
	MOV	A,H
	SBB	D
	RET




;	---BUMP A 3 BYTE COUNTER---
;
;
BUMP:	MVI	B,3
;
BMP1:	INR	M
	RNZ
	INX	H
	DCR	B
	JNZ	BMP1
	RET




;	---STOP TEST---
;
;   TEST FOR THE "ESC" KEY. IF FOUND, ABORT FUNCTION.
;
;
STOP:	CPM	F@REQ		;GET CONSOLE STATUS
	RZ			;RIF DATA NOT AVAIL
	CPM	F@CI		;GET INPUT CHAR
	ANI	7FH		;KILL PARITY BIT
	CPI	ESC		;ESC KEY ?
	JNZ	REPORT		;NO, REPORT CURRENT STATUS
	PRNT	MSG18		; 'PROGRAM STOPPED'
	CALL	REPORT		;REPORT CURRENT STATUS
	CALL	STZ		;MOVE HEADS BACK TO TRACK ZERO
	JMP	LOOP		;ABORT




;	---REPORT CURRENT STATUS---
;
;
REPORT:	CALL	CR2X		;CR.LF 2 TIMES
;
	PRNT	MSG2		; 'PASS'
	LXI	H,PASS+2
	CALL	PCNT		;PRINT PASS COUNT
;
	PRNT	MSG3		; 'SOFT WRITE ERRORS'
	LXI	H,SWERR+2
	CALL	PCNT		;PRINT SOFT WRITE ERROR COUNT
;
	PRNT	MSG4		; 'HARD WRITE ERRORS'
	LXI	H,HWERR+2
	CALL	PCNT		;PRINT HARD WRITE ERROR COUNT
;
	PRNT	MSG5		; 'SOFT READ ERRORS'
	LXI	H,SRERR+2
	CALL	PCNT		;PRINT SOFT READ ERROR COUNT
;
	PRNT	MSG7		; 'HARD READ ERRORS'
	LXI	H,HRERR+2
	CALL	PCNT		;PRINT HARD READ ERROR COUNT
;
	PRNT	MSG9		; 'DATA ERRORS'
	LXI	H,DVERR+2
	CALL	PCNT		;PRINT VERIFY ERROR COUNT
;
	PRNT	MSG20		; 'SEEK ERRORS'
	LXI	H,SKERR+2
	CALL	PCNT		;PRINT SEEK ERROR COUNT
;
	JMP	CR2X		;DO 2 CR.LF, EXIT




;	---PRINT COUNTS---
;
;
PCNT:	MOV	A,M
	ORA	A
	JZ	PCNT1		;PRINT IF COUNT < 64K
;
	MVI	B,3		;LOOP COUNT
PCNT0:	MOV	A,M
	CALL	BTH		;PRINT COUNT IN HEX
	DCX	H
	DCR	B
	JNZ	PCNT0
	RET
;
PCNT1:	DCX	H
	MOV	D,M		;COUNT, MSB
	DCX	H
	MOV	E,M		;COUNT, LSB
	XCHG			;MOVE COUNT TO H/L
	JMP	DEC		;PRINT COUNT IN DECIMAL, EXIT




;	---CR.LF TWICE---
;
;
CR2X:	CALL	CRLF
	JMP	CRLF




;	---PRINT DECIMAL---
;
;   PRINT THE CONTENTS OF 'H/L' IN DECIMAL. LEADING
;   ZEROS ARE NOT PRINTED.
;
;
DEC:	MOV	A,L
	ORA	H
	JNZ	DEC1		;JIF VALUE <> 0
	MVI	E,'0'
	CPM	F@CO		;PRINT 1 ZERO
	RET
;
DEC1:	MVI	B,0		;RESET NON-ZERO FLAG
	LXI	D,10000
	CALL	DX		;PRINT 10K'S
	LXI	D,1000
	CALL	DX		;PRINT 1K'S
	LXI	D,100
	CALL	DX		;PRINT 100'S
	LXI	D,10
	CALL	DX		;PRINT 10'S
	MOV	A,L
	ADI	'0'		;MAKE NUM ASCII
	MOV	E,A
	JMP	DX4		;PRINT 1'S, THEN EXIT
;
;
;
DX:	MVI	C,0		;NUMBER OF "N"
;
DX1:	MOV	A,L		; (H/L) = (H/L)-(D/E)
	SUB	E
	MOV	L,A
	MOV	A,H
	SBB	D
	MOV	H,A
	JC	DX2
	INR	C		;INCR NUM OF "N"
	JMP	DX1
;
DX2:	DAD	D		;RESTORE #
	MOV	A,C
	ADI	'0'		;CONVERT NUM TO ASCII
	MOV	E,A
	CPI	'0'
	JZ	DX3		;JIF NUM WAS ZERO
	INR	B		;SET FLAG = PRINT ZEROS
	JMP	DX4
;
DX3:	MOV	A,B
	ORA	A
	RZ			;DON'T PRINT LEADING ZEROS
;
DX4:	CPM	F@CO		;PRINT THE NUM
	RET




;	---BINARY/HEX CONVERSION---
;
;   CONVERTS BINARY DATA (IN 'A') TO 2 ASCII
;   HEX CHARACTERS & PRINTS THOSE CHARACTERS.
;
;
BTH:	PUSH	D		;SAVE D/E
	PUSH	PSW
	RRC			;MOVE MSD TO LSD
	RRC
	RRC
	RRC
	CALL	BTH1		;CONVERT MSD & PRINT IT
	POP	PSW
	CALL	BTH1		;  * LSD
	POP	D		;RESTORE D/E
	RET		
;
BTH1:	ANI	0FH		;KILL MSD
	ADI	'0'
	CPI	'9'+1
	JC	BTH2		;JIF CHAR 0-9
	ADI	7		;CHAR A-F
BTH2:	MOV	E,A
	CPM	F@CO		;PRINT THE DIGIT
	RET




;	---PRINT ERROR MESSAGE---
;
;
PEM:	CPM	F@PRNT		;PRINT ERROR MSG
	CALL	PDS		;  DRIVE STATUS
	CALL	PCS		;  CTLR STATUS
	CALL	PCA		;  CYL ADR
	CALL	PHN		;  HEAD #
	CALL	PSA		;  SECT ADR
	CALL	CRLF		;DO CR.LF
	JMP	STOP		;ABORT TESTING ?




;	---PRINT DRIVE STATUS---
;
;
PDS:	PRNT	MSG13		; 'DST='
	LDA	DSTAT
	JMP	BTH		;PRINT STATUS, EXIT




;	---PRINT CONTROLLER STATUS---
;
;
PCS:	PRNT	MSG14		; 'CST='
	LDA	CSTAT
	JMP	BTH		;PRINT STATUS, EXIT




;	---PRINT CYLINDER ADDRESS---
;
;
PCA:	PRNT	MSG15		; 'CYL='
	LHLD	TCYL		;TARGET CYL
; PRINT 3 DIGITS
P3D:	MOV	A,H
	CALL	BTH1		;PRINT MS DIGIT OF CYL ADR
	MOV	A,L
	JMP	BTH		;LAST 2 DIGITS, EXIT




;	---PRINT HEAD NUMBER---
;
;
PHN:	PRNT	MSG16		; 'HD='
	LDA	THEAD		;TARGET HEAD
	JMP	BTH1		;PRINT HEAD #, EXIT




;	---PRINT SECTOR ADDRESS---
;
;
PSA:	PRNT	MSG17		; 'SECT='
	LDA	TSEC		;TARGET SECTOR
	JMP	BTH		;PRINT ADR, EXIT




;	---MULTI BYTE MOVER---
;
;
MOVE:	MOV	A,M		;GET DATA
	STAX	D		;PUT DATA
	INX	H
	INX	D
	DCR	B		;DECR COUNT
	JNZ	MOVE
	RET




;	---CARRIAGE RETURN, LINE FEED---
;
;
CRLF:	PUSH	D		;SAVE D/E
	MVI	E,CR
	CPM	F@CO
	MVI	E,LF
	CPM	F@CO
	POP	D		;RESTORE D/E
	RET




;	---TURN DRIVE CONTROLLER OFF---
;
;
COFF:	XRA	A
	OUT	CTCSR		;STOP CTLR
	OUT	DRCSR		;TURN OFF INTERFACE
	OUT	EXTCMD		;DE-SELECT THE DRIVE
	RET



;	---MESSAGES---
;
;
MSG1:	DB	CR,LF,LF,LF
	DB	'MHRDT -- Vers 1.0',CR,LF
	DB	'12-Apr-83',CR,LF,'$'
;
MSG1A:	DB	CR,LF,LF
	DB	'Select one of the following options ---',CR,LF,LF
	DB	'  S - Start Testing',CR,LF
	DB	'  C - Continue Testing',CR,LF
	DB	'  X - Exit to CPM',CR,LF,LF
	DB	'Cmd? $'
;
MSG2:	DB	CR,LF,'             Pass = $'
;
MSG3:	DB	CR,LF,'Soft Write Errors = $'
;
MSG4:	DB	CR,LF,'Hard Write Errors = $'
;
MSG5:	DB	CR,LF,' Soft Read Errors = $'
;
MSG6:	DB	BELL,CR,LF,LF,'* Restore Failed  $'
;
MSG7:	DB	CR,LF,' Hard Read Errors = $'
;
MSG8:	DB	CR,LF,'* Verify Error  $'
;
MSG9:	DB	CR,LF,'      Data Errors = $'
;
MSG10:	DB	CR,LF,'* Write Error  $'
;
MSG11:	DB	CR,LF,'* Read Error  $'
;
MSG12:	DB	BELL,CR,LF,LF,'* Drive not Ready',CR,LF,'$'
;
MSG13:	DB	'  Dst=$'
;
MSG14:	DB	'  Cst=$'
;
MSG15:	DB	'  Cyl=$'
;
MSG16:	DB	'  Hd=$'
;
MSG17:	DB	'  Sect=$'
;
MSG18:	DB	BELL,CR,LF,LF,'* Program Stopped  $'
;
MSG19:	DB	BELL,CR,LF,LF,'* Invalid Input$'
;
MSG20:	DB	CR,LF,'      Seek Errors = $'
;
MSG21:	DB	BELL,CR,LF,LF
	DB	'* Unrecoverable Seek Error...Target Cylinder = $' 
;
MSG22:	DB	'> 64K$'
;
MSG23:	DB	BELL,CR,LF,LF
	DB	'* Seek Failed to Finish (Time-Out Error)'
	DB	CR,LF,'$'
;
MSG24:	DB	BELL,CR,LF,LF
	DB	'* Read/Write Failed to Finish (Time-Out Error)'
	DB	CR,LF,'$'
;
MSG25:	DB	CR,LF,LF,'Testing.....',CR,LF,LF,'$'





;	---STORAGE---
;
;
VARBGN	EQU	$		;START OF VARIABLES
;
PASS:	DS	3		;PASS COUNT
SWERR:	DS	3		;SOFT WRITE ERROR COUNT
HWERR:	DS	3		;HARD WRITE ERROR COUNT
SRERR:	DS	3		;SOFT READ ERROR COUNT
HRERR:	DS	3		;HARD READ ERROR COUNT
DVERR:	DS	3		;DATA VERIFY ERROR COUNT
SKERR:	DS	3		;SEEK VERIFY ERROR COUNT
;
CYL:	DS	2		;CYLINDER ADDRESS
HEAD:	DS	1		;HEAD NUMBER
SECTOR:	DS	1		;SECTOR ADDRESS
;
OLDCYL:	DS	2		;PREVIOUS CYLINDER ADDRESS
OLDHD:	DS	1		;   HEAD
OLDSEC:	DS	1		;   SECTOR
;
TCYL:	DS	2		;TARGET CYLINDER ADDRESS
THEAD:	DS	1		;   HEAD
TSEC:	DS	1		;   SECTOR
;
CSTAT:	DS	1		;CONTROLLER STATUS
DSTAT:	DS	1		;DRIVE STATUS
CCA:	DS	2		;CURRENT CYLINDER ADDRESS
RETRY:	DS	1		;RETRY COUNT
SKRTC:	DS	1		;SEEK RETRY COUNT
VSRTC:	DS	1		;SEEK VERIFY RETRY COUNT
SKDIR:	DS	1		;SEEK DIRECTION
RSKNT:	DS	2		;RESIDUAL SEEK COUNT
;
VARSIZ	EQU	$-VARBGN	;SIZE OF VARIABLE STORAGE
;
SEED:	DS	2		;RANDOM NUMBER
;
RDBFR:	DS	10		;READ BUFFER



LAST	EQU	$		;DEFINES END OF PROGRAM



	END
