	TITLE	'MHROM -- BOOT CPM FROM A MINISCRIBE MS-3012 DRIVE'

;	04-15-83	** A **



;   MHROM is an example of a ROM resident program which is used to boot
;   a CPM system from a MiniScribe MS-3012 Hard Disk Drive. The first
;   sector of the disk is read into memory at location ZERO. This program
;   uses an XCOMP STS/STR Hard Disk Controller.





;	---PROGRAM CONFIGURATION---
;
;
;   THE FOLLOWING STATEMENTS ARE USED TO CONFIGURE THE PROGRAM.
;
;	1. TEST -- SET THIS "FALSE" BEFORE USING THIS PROGRAM.
;
;	2. S100 -- SET THIS "TRUE" IF YOU ARE USING A CONTROLLER WITH
;	   AN S-100 BUS COMPUTER (STS). SET THIS "FALSE" IF YOU ARE
;	   USING A CONTROLLER WITH A RIBBON CABLE INTERFACE (STR).
;
;
;
FALSE	EQU	0		;DEFINE FALSE
TRUE	EQU	NOT FALSE	;DEFINE TRUE
;
TEST	EQU	FALSE		;NOT TESTING
;
S100	EQU	TRUE		;S-100 CONTROLLER






;	---EQUATES---
;
;
	IF	TEST	;  ** IF WE ARE TESTING **
STRT	EQU	100H		;PROGRAM START ADDRESS
	ELSE		;  ** IF NORMAL **
STRT	EQU	0F800H		;PROGRAM START ADDRESS
	ENDIF
;
STACK	EQU	100H		;STACK ADDRESS
LDADR	EQU	0		;LOAD ADDRESS
LMSK	EQU	0FFH		;LSB MASK
LZONE	EQU	656		;LANDING ZONE
;
RCA	EQU	0EAH		;COMPARE BUFFER ADDRESS
RSA	EQU	0D7H		;MICROCODE START ADDRESS
RPT	EQU	80H		;REPEAT
INDEX	EQU	10H		;INDEX BIT
;
ROM	EQU	0F000H		;BASE ADR OF ROM
DEBUG	EQU	ROM		;DEBUG ENTRY POINT
TRAP	EQU	ROM+1EH		;DEBUG TRAP ADDRESS
;
;   ASCII CHARACTERS
;
CR	EQU	0DH		;CARRIAGE RETURN
LF	EQU	0AH		;LINE FEED
BELL	EQU	7		;DING!
ESC	EQU	1BH		;ESCAPE
;
;   CRT I/O
;
CRTS	EQU	9		;STATUS
CRTD	EQU	8		;DATA
CRTBE	EQU	1		;BUFFER EMPTY
CRTDA	EQU	2		;DATA AVAILABLE
;
;   DRIVE COMMANDS
;
SKCMD	EQU	3		;SEEK
SKOUT	EQU	1		;SEEK DIRECTION = OUTWARD
NOPC	EQU	40H		;NO PRE-COMPENSATION
LOWRT	EQU	80H		;LOW WRITE CURRENT
;
;   DRIVE STATUS
;
READY	EQU	1		;DRIVE READY
WRTFLT	EQU	2		;WRITE FAULT
TK00	EQU	4		;TRACK ZERO
RAWINDX	EQU	20H		;RAW INDEX
;
;   CONTROLLER COMMANDS
;
BANK0	EQU	0		;BANK 0 SELECT
BANK1	EQU	1		;BANK 1 SELECT
DBENB	EQU	2		;DATA BUFFER ENABLE
CBENB	EQU	4		;COMPARE BUFFER ENABLE
START	EQU	8		;START COMMAND
;
;   DRIVE/CONTROLLER I/O
;
	IF	S100	;IF S-100 COMPUTER
CBASE	EQU	70H		;BASE ADR OF THE CONTROLLER
DRCSR	EQU	CBASE		;DRIVE COMMAND/STATUS
EXTCMD	EQU	CBASE+1		;EXTENDED COMMNAND REGISTER
LOSC	EQU	CBASE+2		;SEEK COUNT, LSB
HISC	EQU	CBASE+3		;  * MSB
CTCSR	EQU	CBASE+4		;CONTROLLER COMMAND/STATUS
CTBFR	EQU	CBASE+5		;CONTROLLER BUFFER ADDRESS
CTDP	EQU	CBASE+6		;CONTROLLER DATA PORT
	ELSE		;IF RIBBON CABLE INTERFACE
CBASE	EQU	80H		;BASE ADR OF THE CONTROLLER
DRCSR	EQU	CBASE+7		;DRIVE COMMAND/STATUS
EXTCMD	EQU	CBASE+6		;EXTENDED COMMNAND REGISTER
LOSC	EQU	CBASE+5		;SEEK COUNT, LSB
HISC	EQU	CBASE+4		;  * MSB
CTCSR	EQU	CBASE+3		;CONTROLLER COMMAND/STATUS
CTBFR	EQU	CBASE+2		;CONTROLLER BUFFER ADDRESS
CTDP	EQU	CBASE+1		;CONTROLLER DATA PORT
	ENDIF

	PAGE

	ORG	STRT



;	---PROGRAM ENTRY POINT---
;
;
BOOT:	LXI	SP,STACK	;SET SP
	XRA	A
	OUT	CTCSR
	OUT	DRCSR
	OUT	EXTCMD		;STOP ANY PRIOR ACTIVITY
	LXI	H,RDCODE
	CALL	SQUIRT		;SET READ M/CODE
	LXI	H,WRTCODE
	CALL	SQUIRT		;SET WRITE M/CODE
	MVI	A,1
	OUT	EXTCMD		;SLCT HEAD ZERO
	INR	A
	OUT	DRCSR		;ENB SECTOR/INDEX TO DATA BOARD
	CALL	TKZ		;RESTORE
;
;   READ LOOP
;
LOOP:	CALL	READ		;READ SECTOR ZERO
	MVI	A,DBENB
	OUT	CTCSR		;ENB DATA BFR
	XRA	A
	OUT	CTBFR		;SET BFR ADR = 0
	MOV	B,A		;B = COUNT (256)
	LXI	H,LDADR		;LOAD ADDRESS
	IN	CTDP		;PRIME INPUT DATA
	NOP			;TIME DELAY
L1:	IN	CTDP		;GET DATA FROM BFR
	MOV	M,A		;PUT DATA INTO RAM
	INX	H
	DCR	B
	JNZ	L1
;
	IF	TEST
	JMP	DEBUG		;IF WE ARE TESTING
	ELSE
	JMP	LDADR		;BOOT SYSTEM
	ENDIF





;	---READ A SECTOR---
;
;
READ:	MVI	D,10		;D = RETRY COUNT
	MVI	A,CBENB
	OUT	CTCSR		;ENABLE COMPARE BUFFER
	MVI	A,RCA
	OUT	CTBFR		;SET BFR ADR
	MVI	B,4
	XRA	A
DO1:	OUT	CTDP		;SET HEADER = CYL 0, HEAD 0, SECT 0 
	DCR	B
	JNZ	DO1
;
DO2:	CALL	RDY		;DRIVE READY ?  (RIF YES)
	XRA	A
	OUT	CTCSR		;SLCT BANK ZERO
	MVI	A,RSA
	OUT	CTBFR		;SET BFR ADR
	MVI	A,START
	OUT	CTCSR		;START READ CMD
	LXI	H,0		;TIME-OUT DELAY COUNT
;
DO3:	IN	CTCSR		;CTLR STATUS
	RRC
	JC	DO4		;JIF READ DONE
	DCX	H		;DECR DELAY COUNT
	MOV	A,H
	ORA	L
	JNZ	DO3		;JIF CON'T WAITING
	OUT	CTCSR		;STOP THE CTLR
	JMP	DO5		;ABORT, TIME-OUT ERROR
;
DO4:	IN	CTCSR		;GET NON-CHANGING STATUS
	ANI	0EH		;TEST CTLR STATUS
	MOV	B,A		;  SAVE RESULTS
	XRA	A
	OUT	CTCSR		;STOP CTLR
	IN	DRCSR		;DRIVE STATUS
	ANI	WRTFLT
	CNZ	CLRDF		;CIF CLEAR DRIVE FAULT
	ORA	B		;SET/CLEAR ERROR FLAG (0=OK)
	RZ			;RIF READ/WRITE OK
	DCR	D		;DECR RETRY CNT
	JNZ	DO2		;JIF RETRY READ/WRITE
; READ ERROR
DO5:	LXI	H,MSG3		;ERROR MSG
	JMP	PEM		;ABORT, READ FAILED




;	---RESTORE--
;
;
TKZ:	CALL	TZT		;TRACK ZERO TEST
	RZ			;RIF ALREADY AT TRACK ZERO
;
	LXI	H,511		;# CYLS WE CAN SEEK
	CALL	RTZ		;SEEK OUT
	RZ			;RIF AT TRACK 0
	LXI	H,LZONE+10-511
	CALL	RTZ		;TRY IT AGAIN
	CALL	TZT
	RZ			;RIF FOUND TRACK ZERO THIS TIME
; RESTORE FAILED
	LXI	H,MSG2		;H/L = ADR OF ERR MSG
	JMP	PEM		;ABORT, RESTORE FAILED
;
;   SEEK OUTWARD
;
RTZ:	CALL	RDY		;DRIVE READY ?  (RIF YES)
	MOV	A,L
	OUT	LOSC		;SET LSB OF SEEK COUNT
	MOV	A,H
	OUT	HISC		;SET MSB
	MVI	A,SKOUT
	OUT	EXTCMD		;SET SEEK DIRC = OUTWARD
	MVI	A,SKCMD
	OUT	DRCSR		;ISSUE SEEK CMD
	CALL	WSC		;WAIT FOR SEEK DONE
;
TZT:	IN	DRCSR		;DRIVE STATUS
	ANI	TK00		;TRACK ZERO INDICATOR
	XRI	TK00		;..MAKE IT LO-TRUE
	RET




;	---WAIT FOR SEEK COMPLETE---
;
;
WSC:	LXI	H,0		;TIME-OUT DELAY COUNT
	MVI	D,3
;
WSC1:	IN	DRCSR		;DRIVE STATUS
	RAL
	RC			;RIF SEEK DONE
	DCX	H		;DECR DELAY COUNT
	MOV	A,H
	ORA	L
	JNZ	WSC1		;JIF CON'T WAITING
	DCR	D
	JNZ	WSC1		;..DITTO WAIT
	LXI	H,MSG2
	JMP	PEM		;ABORT, RESTORE FAILED TO FINISH



;	---DRIVE READY TEST---
;
;
RDY:	IN	DRCSR		;DRIVE STATUS
	ANI	READY		;DRIVE RDY BIT
	XRI	READY		;..MAKE IT LO-TRUE
	RZ			;RIF DRIVE READY
	LXI	H,MSG1		;ERROR MSG
	JMP	PEM		;ABORT, DRIVE NOT READY




;	---CLEAR DRIVE FAULT---
;
;
CLRDF:	XRA	A
	OUT	EXTCMD		;DE-SELECT THE DRIVE
	MVI	A,1
	OUT	EXTCMD		;RE-SELECT THE DRIVE
	RET




;	---SQUIRT MICROCODE---
;
;
SQUIRT:	MOV	A,M
	OUT	CTCSR		;SLCT BANK
	INX	H
	MOV	A,M
	OUT	CTBFR		;SET CTLR BFR ADR
	INX	H
;
SQ1:	MOV	A,M
	ORA	A
	RZ			;RIF DONE
	INX	H		;INCR TBL ADR
	MOV	B,A		;B = COUNT
	RAL
	JC	SQ3		;JIF REPEAT
;
SQ2:	MOV	A,M
	OUT	CTDP		;OUTPUT CONTROL INFO
	INX	H
	DCR	B
	JNZ	SQ2
	JMP	SQ1		;DO NEXT STRING
;
SQ3:	ORA	A		;CLEAR CY
	RAR
	MOV	B,A		;B = COUNT
	MOV	A,M
	INX	H
SQ4:	OUT	CTDP		;OUTPUT REPEAT CONTROL INFO
	DCR	B
	JNZ	SQ4
	JMP	SQ1		;DO NEXT STRING





;	---WRITE MICROCODE---
;
;
WRTCODE	EQU	$		;  MINISCRIBE	06-09-81
;
	DB	1		
	DB	0D3H		
	DB	2,40H		
	DB	41H		
	DB	RPT+3,40H	
	DB	RPT+12,6DH	
	DB	1,63H		
	DB	RPT+4,65H	
	DB	RPT+2,67H	
	DB	2,49H		
	DB	40H		
	DB	RPT+17,50H	
	DB	5,5BH		
	DB	0DDH		
	DB	5DH		
	DB	50H		
	DB	0FH		
	DB	0		




;	---READ MICROCODE---
;
;
RDCODE	EQU	$		;  MINISCRIBE	06-09-81
;
	DB	0		
	DB	0D7H		
	DB	2,40H		
	DB	41H		
	DB	RPT+3,40H	
	DB	RPT+12,6DH	
	DB	1,63H		
	DB	RPT+4,65H	
	DB	RPT+2,67H	
	DB	1,49H		
	DB	RPT+2,40H	
	DB	RPT+12,6DH	
	DB	3,63H		
	DB	0E7H		
	DB	67H		
	DB	RPT+7,0FH	
;
	DB	2,40H		
	DB	41H		
	DB	RPT+3,40H	
	DB	RPT+12,6DH	
	DB	1,63H		
	DB	RPT+2,65H	
	DB	RPT+2,60H	
	DB	RPT+2,67H	
	DB	1,0FH		
	DB	0		





;	---PRINT ERROR MESSAGES---
;
;   UNRECOVERABLE ERRORS COME HERE WITH THE ADDRESS OF
;   AN ERROR MESSAGE IN "H/L".
;
;
PEM:	LXI	SP,STACK	;RESET STACK ADR
	CALL	PRINT		;PRINT ERROR MSG
	CALL	CI		;WAIT FOR THE OPER
	IF	TEST		;IF WE ARE TESTING
	CPI	ESC		;ESCAPE KEY ?
	CZ	TRAP		;YES, TRAP TO DEBUG
	ENDIF
	LXI	H,CRLF
	CALL	PRINT		;DO CR.LF
	JMP	LOOP		;TRY TO BOOT AGAIN




;	---PRINT MESSAGES---
;
;
PRINT:	MOV	A,M		;A = PRINT DATA
	INX	H		;POINT TO NEXT CHAR
	ORA	A
	RZ			;ZERO ENDS MSG
	MOV	C,A
	CALL	CO		;PRINT A CHAR
	JMP	PRINT




;	---CONSOLE INPUT---
;
;
CI:	IN	CRTS		;CRT STATUS
	ANI	CRTDA
	JZ	CI		;WAIT FOR INPUT CHAR
	IN	CRTD		;INPUT DATA
	ANI	7FH		;KILL PARITY BIT
	RET




;	---CONSOLE OUTPUT---
;
;
CO:	IN	CRTS		;CRT STATUS
	ANI	CRTBE
	JZ	CO		;WAIT FOR BFR EMPTY
	MOV	A,C
	OUT	CRTD		;OUTPUT DATA
	RET




;	---MESSAGES---
;
;
MSG1:	DB	BELL,CR,LF,'* Drive not Ready ',0
;
MSG2:	DB	BELL,CR,LF,'* Restore Failed ',0
;
MSG3:	DB	BELL,CR,LF,'* Read Failed ',0
;
CRLF:	DB	CR,LF,0




	END
