	TITLE	'MHVER -- MINISCRIBE MS-3012 VERIFY'

;	04-13-83


;   MHVER is used to Read (Verify) a MiniScribe MS-3012 hard disk drive.
;   Both SOFT & HARD errors are reported. SOFT errors are defined as the
;   ability to read a sector with less than 10 retries. HARD errors are
;   defined as the inability to read a sector within 10 retries. Errors
;   will be reported as follows:
;
;
;	1)   * Soft Read Error  Cyl=xx  Hd=xx  Sect=xx  Retries=xx
;
;	2)   * Hard Read Error  Cyl=xx  Hd=xx  Sect=xx  Error=[RNF/CRC]
;
;	     WHERE --	Cyl = Cylinder Address	:
;			 Hd = Head Number	:  Numbers are in Hex
;		       Sect = Sector Address	:
;		        ----------------------
;			RNF = Record not Found
;			CRC = Data CRC Error
;
;
;   NOTE -- Hitting the ESC key while running will ABORT verification.





;	---PROGRAM CONFIGURATION---
;
;
;   THE FOLLOWING STATEMENTS ARE USED TO CONFIGURE THE PROGRAM.
;
;	1. TEST -- SET THIS 'FALSE' BEFORE USING THE PROGRAM. 
;
;	2. S100 -- SET THIS 'TRUE' IF YOU ARE USING A CONTROLLER WITH
;	   AN S-100 BUS INTERFACE (STS). SET THIS 'FALSE' IF YOU ARE
;	   USING A CONTROLLER WITH THE RIBBON CABLE INTERFACE (STR).
;
;	3. FASTCPU -- SET THIS 'TRUE' IF YOUR PROCESSOR IS RUNNING AT
;	   4 MHZ. SET THIS 'FALSE' IF YOUR PROCESSOR IS RUNNING AT
;	   2 MHZ.
;
;
;
FALSE	EQU	0		;DEFINE FALSE
TRUE	EQU	NOT FALSE	;..TRUE
;
TEST	EQU	FALSE		;NOT TESTING
;
S100	EQU	TRUE		;S-100 CONTROLLER
;
FASTCPU	EQU	TRUE		;RUNNING AT 4 MHZ






;	---EQUATES---
;
;
STRT	EQU	100H		;PROGRAM START ADDRESS
STACK	EQU	100H		;STACK ADDRESS
LMSK	EQU	0FFH		;LSB MASK
WBOOT	EQU	0		;CPM WARM BOOT
BDOS	EQU	5		;BDOS ENTRY POINT
DEBUG	EQU	0F000H		;DEBUG ENTRY POINT
RPT	EQU	80H		;REPEAT FLAG
;
VSA	EQU	8		;VERIFY START ADDRESS
VCA	EQU	1BH		;VERIFY COMPARE ADDRESS
VCNT	EQU	10		;VERIFY RETRY COUNT
;
	IF	FASTCPU
SKEW	EQU	10		;FORMAT INTERLACE FACTOR
	ELSE
SKEW	EQU	16		;FORMAT INTERLACE FACTOR
	ENDIF
;
;
MAXHD	EQU	2		;NUMBER OF HEAD
MAXCYL	EQU	612		;NUMBER OF CYLINDERS
LZONE	EQU	656		;LANDING ZONE
MAXSEC	EQU	32		;NUMBER OF SECTORS PER TRACK
;
;   DRIVE COMMANDS
;
SKCMD	EQU	3		;SEEK COMMAND
SKOUT	EQU	1		;SEEK DIRECTION = OUTWARD
SKIN	EQU	2		;..INWARD
NOPC	EQU	40H		;NO PRE-COMPENSATION
LOWRT	EQU	80H		;LOW WRITE CURRENT
;
;   DRIVE STATUS
;
READY	EQU	1		;DRIVE READY
WRTFLT	EQU	2		;WRITE FAULT
TK00	EQU	4		;TRACK ZERO
RAWINDX	EQU	20H		;RAW INDEX
;
;   CONTROLLER COMMANDS
;
BANK0	EQU	0		;BANK 0 SELECT
BANK1	EQU	1		;BANK 1 SELECT
DBENB	EQU	2		;DATA BUFFER ENABLE
CBENB	EQU	4		;COMPARE BUFFER ENABLE
START	EQU	8		;START COMMAND
;
;   CONTROLLER STATUS
;
INDEX	EQU	10H		;DELAYED INDEX
SECTOR	EQU	20H		;SECTOR
;
;   DRIVE/CONTROLLER I/O
;
	IF	S100	; IF S-100 BUS COMPUTER
CBASE	EQU	70H		;BASE ADR OF THE CONTROLLER
DRCSR	EQU	CBASE		;DRIVE COMMAND/STATUS
EXTCMD	EQU	CBASE+1		;EXTENDED COMMNAND REGISTER
LOSC	EQU	CBASE+2		;SEEK COUNT, LSB
HISC	EQU	CBASE+3		;  * MSB
CTCSR	EQU	CBASE+4		;CONTROLLER COMMAND/STATUS
CTBFR	EQU	CBASE+5		;CONTROLLER BUFFER ADDRESS
CTDP	EQU	CBASE+6		;CONTROLLER DATA PORT
	ELSE		; IF RIBBON CABLE INTERFACE
CBASE	EQU	80H		;BASE ADR OF THE CONTROLLER
DRCSR	EQU	CBASE+7		;DRIVE COMMAND/STATUS
EXTCMD	EQU	CBASE+6		;EXTENDED COMMNAND REGISTER
LOSC	EQU	CBASE+5		;SEEK COUNT, LSB
HISC	EQU	CBASE+4		;  * MSB
CTCSR	EQU	CBASE+3		;CONTROLLER COMMAND/STATUS
CTBFR	EQU	CBASE+2		;CONTROLLER BUFFER ADDRESS
CTDP	EQU	CBASE+1		;CONTROLLER DATA PORT
	ENDIF
;
;   ASCII CHARACTERS 
;
CR	EQU	0DH		;CARRIAGE RETURN
LF	EQU	0AH		;LINE FEED
BELL	EQU	7		;DING!
BKSP	EQU	8		;BACKSPACE
ESC	EQU	1BH		;ESCAPE KEY
;
;   CPM FUNCTION CODES
;
F@CI	EQU	1		;CONSOLE INPUT
F@CO	EQU	2		;CONSOLE OUTPUT
F@PRNT	EQU	9		;PRINT MESSAGES
F@RCB	EQU	10		;READ CONSOLE BUFFER
F@REQ	EQU	11		;GET CONSOLE STATUS

	PAGE



;	============
;	** MACROS **
;	============



;	---CALL BDOS---
;
;
CPM	MACRO	FC
	CALL	SYSTEM		;; SYSTEM CALL ROUTINE
	DB	FC		;; CPM FUNCTION CODE
	ENDM




;	---PRINT MESSAGES---
;
;
PRNT	MACRO	MSG
	LXI	D,MSG		;; ADR OF THE MSG
	CPM	F@PRNT		;; PRINT THE MSG
	ENDM

	PAGE

	ORG	STRT	


;	==========================
;	** PROGRAM ENTRY POINT **
;	==========================
;
;
;
BGN:	LXI	SP,STACK	;USE INTERNAL STACK
	DI			;INHIBIT INTERRUPTS
	CALL	COFF		;STOP ANY PRIOR ACTIVITY
	PRNT	MSG1		;SAY HELLO
	MVI	A,1
	OUT	EXTCMD		;SLCT DRIVE ZERO
	INR	A
	OUT	DRCSR		;ENABLE INTERFACE BOARD
	CALL	BST		;BUILD SECTOR TABLE
;
;   COMMAND LOOP
;
LOOP:	LXI	SP,STACK	;RESET SP
	PRNT	MSG2		;PRINT MENU
	LDA	BGNHED
	CALL	BTH1		;PRINT START HEAD
	MVI	E,'-'
	CPM	F@CO		;PRINT SEPERATOR
	LDA	ENDHED		;END HEAD NUM
	DCR	A		;..ADJUST IT
	CALL	BTH1		;..PRINT IT
	PRNT	MSG2A		; 'CYLS = '
	LHLD	BGNCYL
	CALL	P3D		;PRINT START CYL
	MVI	E,'-'
	CPM	F@CO		;PRINT SEPERATOR
	LHLD	ENDCYL
	DCX	H
	CALL	P3D		;PRINT END CYL
	PRNT	MSG2B		; 'CMD?'
;
	MVI	A,1
	CALL	RCB		;GET OPER CMD
	CPI	'C'
	JZ	NEWCYL		;JIF GET NEW CYL LIMITS
	CPI	'H'
	JZ	NEWHED		;JIF GET NEW HEAD LIMITS
	CPI	'S'
	JZ	VERIFY		;JIF VERIFY THE DISK
	CPI	'X'
	JZ	XIT		;JIF EXIT TO CPM
;
;   INVALID INPUT
;
INV:	PRNT	MSG3		;PRINT 'INVALID INPUT'
	JMP	LOOP		;DO NEXT CMD
;
;   EXIT TO CPM
;
XIT:	CALL	CRLF		;DO CR.LF
	CALL	COFF		;TURN DRIVE/CTLR OFF
 	IF	TEST
	JMP	DEBUG		;IF WE ARE TESTING
 	ELSE
 	JMP	WBOOT		;WARM BOOT CPM
 	ENDIF




;	---GET NEW HEAD LIMITS---
;
;
NEWHED:	PRNT	MSG11		; 'START HEAD? '
	CALL	NHX		;GET HEAD NUM
	JNC	INV		;JIF INVALID HEAD NUM
	STA	TEMP		;SAVE NEW START HEAD
	PRNT	MSG12		; 'END HEAD? '
	CALL	NHX		;GET HEAD NUM
	JNC	INV		;JIF INVALID HEAD NUM
	INR	A		;FOR PROPER TESTING
	MOV	B,A		;B = END HEAD (+1)
	LDA	TEMP		;A = START HEAD
	CMP	B
	JNC	NH1		;JIF SH > EH
	MOV	L,A
	MOV	H,B
	SHLD	BGNHED		;SAVE NEW HEAD LIMITS
	JMP	LOOP		;DO NEXT CMD
; ERROR, START HEAD > END HEAD
NH1:	PRNT	MSG13		;PRINT ERROR MSG
	JMP	LOOP		;DO NEXT CMD
;
;   GET HEAD NUMBER
;
NHX:	MVI	A,2
	CALL	RCB		;INPUT HEAD NUM FROM OPER
	CALL	CONV		;CONVERT INPUT TO BINARY
	MOV	A,L
	CPI	MAXHD		;TEST HEAD NUM
	RET




;	---GET NEW CYLINDER LIMITS---
;
;
NEWCYL:	PRNT	MSG14		; 'START CYL? '
	CALL	NCX		;GET START CYL ADR
	JNC	INV		;JIF INVALID CYL ADR
	SHLD	TEMP		;SAVE CYL ADR
	PRNT	MSG15		; 'END CYL? '
	CALL	NCX		;GET END CYL ADR
	JNC	INV		;JIF INVALID CYL ADR
	INX	H		;FOR PROPER TESTING
	XCHG			;D/E = END CYL ADR
	LHLD	TEMP		;H/L = START CYL ADR
	CALL	TSUB		; (H/L) - (D/E)
	JNC	NC1		;JIF SC > EC
	SHLD	BGNCYL		;SAVE NEW START CYL ADR
	XCHG
	SHLD	ENDCYL		;SAVE NEW END CYL ADR
	JMP	LOOP		;DO NEXT CMD
; ERROR, START CYL > END CYL
NC1:	PRNT	MSG16		;PRINT ERROR MSG
	JMP	LOOP		;DO NEXT CMD
;
;   GET CYLINDER ADDRESS
;
NCX:	MVI	A,4
	CALL	RCB		;GET CYL ADR FROM OPER
	CALL	CONV		;CONVERT INPUT TO BINARY
	LXI	D,MAXCYL
	JMP	TSUB		;TEST CYL ADR, EXIT

	PAGE


;	=====================
;	** VERIFY THE DISK **
;	=====================




;	---VERIFY LOOP---
;
;
VERIFY:	CALL	CRLF		;DO CR.LF
	LXI	H,RDCODE
	CALL	SQUIRT		;LOAD READ M/CODE
	CALL	TKZ		;RESTORE
	CALL	SETUP		;SET START CYL/HEAD
	CALL	SEEK		;SEEK TO 1ST CYL
	CALL	NXP		;PRINT 1ST CYL ADR
;
VER1:	CALL	VAT		;VERIFY A TRACK
	CALL	STOP		;STOP VERIFY ?  (RIF NO)
	CALL	NXTK		;GOTO NEXT TRACK
	JC	VER1		;JIF MORE TO DO
	CALL	STZ		;SEEK TO TRACK ZERO
	PRNT	MSG9		; 'VERIFY DONE'
	JMP	LOOP		;DO NEXT CMD




;	---VERIFY A TRACK---
;
;
VAT:	LXI	H,SECTBL
	CALL	VHT		;READ EVEN SECTORS
	LXI	H,SECTBL+1	;..DO ODD SECTORS
;
;   READ HALF A TRACK
;
VHT:	SHLD	TBLADR		;SAVE SECTOR TABLE ADR
	MOV	A,M		;GET TABLE ENTRY
	STA	SECADR		;..SAVE IT
	INR	A		;TEST FOR DONE
	RZ			;RIF DONE
	MVI	A,VCNT
	STA	RTC		;SET RETRY COUNT
;
VHT1:	LXI	H,RTBL
	CALL	DORW		;READ A SECTOR
	LXI	H,RTC
	JZ	VHT2		;JIF READ OK
	DCR	M		;DECR RETRY COUNT
	JNZ	VHT1		;JIF RETRY READ
	CALL	HARD		;HARD ERROR, PRINT MSG
	JMP	VHT3
;
VHT2:	MOV	A,M		;A = RETRY COUNT
	SUI	VCNT		;TEST IT
	JZ	VHT3		;JIF HAD NO ERRORS
	CMA
	INR	A		;MAKE COUNT POSITIVE
	STA	RTC		;SAVE # RETRIES
	CALL	SOFT		;PRINT SOFT ERROR MSG
;
VHT3:	LHLD	TBLADR		;SECTOR TABLE ADR
	INX	H
	INX	H		;SKIP AN ENTRY
	JMP	VHT		;READ NEXT SECTOR

	PAGE


;	============================
;	** DISK RELATED FUNCTIONS **
;	============================




;	---EXECUTE READ/WRITE COMMANDS---
;
;
DORW:	CALL	RDY		;DRIVE READY ?  (RIF YES)
	MOV	A,M
	OUT	CTCSR		;ENB CMP BFR
	INX	H
	MOV	A,M
	OUT	CTBFR		;SET CMP BFR ADR
	INX	H
;
	XCHG			;D/E = TBL ADR
	LXI	H,CYL
	MVI	B,4
DO1:	MOV	A,M
	OUT	CTDP		;PUT HDR INFO INTO CMP BFR
	INX	H
	DCR	B
	JNZ	DO1
	XCHG			;H/L = TBL ADR
;
	MOV	A,M		;A = CNTL BANK
	INX	H
	MOV	B,A
	OUT	CTCSR		;SLCT CNTL BANK
	MOV	A,M
	OUT	CTBFR		;SET START ADR
	INX	H
	MOV	A,B
	ORI	START
	OUT	CTCSR		;START R/W CMD
;
DO3:	CALL	WFD		;WAIT FOR DONE
	STA	CSTAT		;SAVE CTLR STATUS
	ANI	0EH		;TEST FOR ERRORS
	MOV	B,A
	IN	DRCSR		;DRIVE STATUS
	STA	DSTAT		;  SAVE IT
	ANI	WRTFLT		;WRITE FAULT ?
	CNZ	CLRDF		;CIF CLEAR DRIVE FAULT
	ORA	B		;SET/CLEAR ERROR FLAG (0=OK)
	RET




;	---WAIT FOR DONE---
;
;
WFD:	LXI	H,0		;TIME-OUT DELAY COUNT
;
WFD1:	IN	CTCSR		;CTLR STATUS
	RRC
	JC	WFD2		;JIF DONE
	DCX	H		;DECR DELAY COUNT
	MOV	A,H
	ORA	L
	JNZ	WFD1		;JIF CON'T WAITING
	OUT	CTCSR		;STOP THE CTLR
	PRNT	MSG19		;PRINT ERROR MSG
	JMP	LOOP		;ABORT, TIME-OUT ERROR
;
WFD2:	IN	CTCSR		;GET CTLR STATUS
	MOV	B,A
	XRA	A
	OUT	CTCSR		;STOP THE CTLR
	MOV	A,B
	RET




;	---GOTO NEXT TRACK---
;
;
NXTK:	LHLD	BGNHED
	XCHG			;D = END HEAD, E = START HEAD
	LXI	H,HEAD
	INR	M		;INCR HEAD NUM
	MOV	A,M
	CMP	D
	JNC	NXT1		;JIF GOTO NEXT CYL
	CALL	HDSEL		;SLCT NEW HEAD
	STC			;SET FLAG = CON'T
	RET
;
NXT1:	MOV	M,E		;RESET HEAD NUM
	CALL	HDSEL		;SLCT NEW HEAD
	LHLD	ENDCYL
	XCHG			;D/E = END CYL ADR
	LHLD	CYL		;CURR CYL ADR
	INX	H
	SHLD	CYL		;SAVE NEW CYL ADR
	CALL	TSUB		; (H/L) - (D/E)
	RNC			;RIF TESTING DONE
	CALL	SEEK		;SEEK TO NEXT CYL
;
;   PRINT CURRENT CYLINDER/HEAD
;
NXP:	PRNT	MSG17		; 'CYL='
	LHLD	CYL
	CALL	P3D		;PRINT CYL ADR
	STC			;SET FLAG = CON'T TESTING
	RET




;	---REZERO---
;
;
TKZ:	LXI	H,0
	SHLD	CCA		;INIT CURR CYL ADR
	IN	DRCSR		;DRIVE STATUS
	ANI	TK00		;TRACK ZERO FLAG
	RNZ			;RIF ALREADY AT TRACK ZERO
;
	LXI	H,511		;# CYLS WE CAN SEEK
	CALL	RTZ		;SEEK OUT
	RNZ			;RIF AT TRACK 0
	LXI	H,LZONE+10-511
	CALL	RTZ		;TRY IT AGAIN
	RNZ			;RIF FOUND TRACK ZERO THIS TIME
; PRINT ERROR MSG
	PRNT	MSG4		; 'RESTORE FAILED'
	CALL	CRLF		;DO CR.LF
	JMP	LOOP		;ABORT
;
;   SEEK OUTWARD
;
RTZ:	CALL	RDY		;DRIVE READY ?  (RIF YES)
	MOV	A,L
	OUT	LOSC		;SET LSB OF SEEK COUNT
	MOV	A,H
	OUT	HISC		;SET MSB
	MVI	A,SKOUT
	OUT	EXTCMD		;SET SEEK DIRC = OUTWARD
	MVI	A,SKCMD
	OUT	DRCSR		;ISSUE SEEK CMD
	CALL	WSC		;WAIT FOR SEEK DONE
	IN	DRCSR		;DRIVE STATUS
	ANI	TK00		;TRACK ZERO INDICATOR
	RET




;	---RETURN TO TRACK ZERO---
;
;
STZ:	LXI	H,0
	SHLD	CYL		;SET TARGET CYL = ZERO
				;   * FALL THRU TO 'SEEK' *




;	---SEEK---
;
;
SEEK:	MVI	A,3
	STA	SKRTC		;SET RETRY COUNT
;
SEK0:	LHLD	CCA		;CURR CYL ADR
	XCHG
	LHLD	CYL		;TARGET CYL
	SHLD	CCA		;UPDATE CURR CYL ADR
	XCHG
	CALL	SUBT		; (H/L) - (D/E)
	RZ			;RIF SAME CYL
	MVI	B,1		;DIRC = OUT
	RAL
	JNC	SEK1		;JIF SEEK OUTWARD
; SEEK INWARD
	MVI	B,3		;DIRC = INWARD
	MOV	A,L		;MAKE SEEK COUNT POSITIVE
	CMA			  ;
	MOV	L,A		  ;
	MOV	A,H		  ;
	CMA			  ;
	MOV	H,A		  ;
	INX	H		  ;
;
SEK1:	LDA	CHA		;CURR HEAD ADR
	ORA	B		;B = SEEK DIRC
	STA	SKDIR		;..SAVE RESULTS
	LXI	D,512
	CALL	TSUB
	JNC	SEK2		;JIF DOUBLE-PUMP (SEEK > 511 CYLS)
	CALL	PSK		;SEEK =< 511 CYLS
	JMP	SEK3
; DOUBLE PUMP SEEKS
SEK2:	DCX	D		;D/E = 511
	CALL	SUBT		; (H/L) - (D/E)
	SHLD	RSKNT		;SAVE RESIDUAL SEEK CNT
	XCHG			;H/L = 511
	CALL	PSK		;SEEK 511 CYLS
	LHLD	RSKNT		;RESIDUAL SEEK COUNT
	CALL	PSK		;SEEK REMAINDER
; SEEK VERIFY
SEK3:	MVI	A,3
	STA	VSRTC		;SET RETRY COUNT
	MVI	A,CBENB
	OUT	CTCSR		;ENABLE BANK ZERO CMP BFR
	MVI	A,VCA
	OUT	CTBFR		;SET CMP BFR ADR
	LHLD	CCA		;CURR CYL ADR
	MOV	A,L
	OUT	CTDP		;SET CYL ADR, LSB
	MOV	A,H
	OUT	CTDP		;SET CYL ADR, MSB
SEK4:	MVI	A,VSA
	OUT	CTBFR		;SET M/CODE START ADR
	MVI	A,START
	OUT	CTCSR		;START VERIFY
	CALL	WFD		;WAIT FOR DONE
	ANI	0CH		;TEST CTLR STATUS
	RZ			;RIF VERIFY OK
	LXI	H,VSRTC
	DCR	M		;DECR RETRY COUNT
	JNZ	SEK4		;JIF RETRY SEEK VERIFY
; VERIFY FAILED
	CALL	TKZ		;RESTORE
	LXI	H,SKRTC
	DCR	M		;DECR RETRY COUNT
	JNZ	SEK0		;JIF RETRY SEEK
; UNRECOVERABLE SEEK ERROR
	PRNT	MSG24		; 'SEEK FAILED'
	LHLD	CYL
	CALL	P3D		;PRINT CYL ADR
	CALL	CRLF		;DO CR.LF
	JMP	LOOP		;ABORT
;
;   PARTIAL SEEK
;
PSK:	CALL	RDY		;DRIVE READY ?  (RIF YES)
	MOV	A,L
	OUT	LOSC		;SET SEEK COUNT, LSB
	MOV	A,H
	OUT	HISC		;   * MSB
	LDA	SKDIR
	OUT	EXTCMD		;SET SEEK DIRECTION
	MVI	A,3
	OUT	DRCSR		;ISSUE SEEK CMD
	JMP	WSC		;WAIT FOR SEEK DONE, EXIT




;	---WAIT FOR SEEK COMPLETE---
;
;
WSC:	LXI	H,0		;TIME-OUT DELAY COUNT
	MVI	B,3
;
WSC1:	IN	DRCSR		;DRIVE STATUS
	RAL
	JC	WSC2		;JIF SEEK DONE
	DCX	H		;DECR RETRY COUNT
	MOV	A,H
	ORA	L
	JNZ	WSC1		;JIF CON'T WAITING
	DCR	B
	JNZ	WSC1		;JIF CON'T WAITING
	PRNT	MSG18		;PRINT ERROR MSG
	JMP	LOOP		;ABORT, TIME-OUT ERROR
;
WSC2:	RAR
	STA	DSTAT		;SAVE DRIVE STATUS
	RET




;	---HEAD SELECT---
;
;
HDSEL:	LDA	HEAD
	RLC			;SHIFT FOR H/W
	RLC
	ORI	1		;TO MAINTAIN DRIVE SLCT
	OUT	EXTCMD		;SELECT THE HEAD
	STA	CHA		;..SAVE FOR FUTURE USE
	RET




;	---DRIVE READY TEST---
;
;
RDY:	IN	DRCSR		;DRIVE STATUS
	RRC
	RC			;RIF DRIVE READY
	PRNT	MSG10		;PRINT ERROR MSG
	JMP	LOOP		;DO NEXT CMD




;	---CLEAR DRIVE FAULT---
;
;
CLRDF:	XRA	A
	OUT	EXTCMD		;DISABLE DRIVE 0
	JMP	HDSEL		;RE-SELECT DRIVE & CORRECT HEAD




;	---CYLINDER/HEAD SETUP---
;
;
SETUP:	LHLD	BGNCYL
	SHLD	CYL		;INIT CYL ADR
	LDA	BGNHED		;1ST HEAD TO USE
	STA	HEAD		;  MAKE IT CURRENT
	JMP	HDSEL		;SELECT 1ST HEAD, EXIT




;	---SQUIRT MICROCODE---
;
;
SQUIRT:	MOV	A,M
	OUT	CTCSR		;SLCT BANK
	INX	H
	MOV	A,M
	OUT	CTBFR		;SET CTLR BFR ADR
	INX	H
;
SQ1:	MOV	A,M
	ORA	A
	RZ			;RIF DONE
	INX	H		;INCR TBL ADR
	MOV	B,A		;B = COUNT
	RAL
	JC	SQ3		;JIF REPEAT
;
SQ2:	MOV	A,M
	OUT	CTDP		;OUTPUT CONTROL INFO
	INX	H
	DCR	B
	JNZ	SQ2
	JMP	SQ1		;DO NEXT STRING
;
SQ3:	ORA	A		;CLEAR CY
	RAR
	MOV	B,A		;B = COUNT
	MOV	A,M
	INX	H
SQ4:	OUT	CTDP		;OUTPUT REPEAT CONTROL INFO
	DCR	B
	JNZ	SQ4
	JMP	SQ1		;DO NEXT STRING




;	---COMMAND TABLES---
;
;
RTBL:	DB	4		;CMP BFR ENB
	DB	0EAH		;CMP BFR ADR
	DB	BANK0		;CNTL BANK
	DB	0D7H		;START ADDRESS




;	---READ MICROCODE---
;
;
RDCODE	EQU	$		;  MINISCRIBE     06-09-81
;
	DB	0		
	DB	0D7H		
	DB	2,40H		
	DB	41H		
	DB	RPT+3,40H	
	DB	RPT+12,6DH	
	DB	1,63H		
	DB	RPT+4,65H	
	DB	RPT+2,67H	
	DB	1,49H		
	DB	RPT+2,40H	
	DB	RPT+12,6DH	
	DB	3,63H		
	DB	0E7H		
	DB	67H		
	DB	RPT+7,0FH	
;
	DB	2,40H		
	DB	41H		
	DB	RPT+3,40H	
	DB	RPT+12,6DH	
	DB	1,63H		
	DB	RPT+2,65H	
	DB	RPT+2,60H	
	DB	RPT+2,67H	
	DB	1,0FH		
	DB	0		

	PAGE


;	======================================
;	** SUBROUTINES, MESSAGES, & STORAGE **
;	======================================




;	---CALL CPM---
;
;   USED TO GO TO CPM (VIA BDOS) TO PERFORM NON-DISK
;   I/O. ALL REGISTERS (EXCEPT 'A') ARE SAVED & THEN
;   RESTORED AFTER THE FUNCTION IS COMPLETE. THE
;   CALLING SEQUENCE IS ---
;
;	CALL	SYSTEM		;ENTRY POINT
;	DB	FC		;FUNCTION CODE
;
;
SYSTEM:	XTHL			;H/L = ADR OF F.C.
	PUSH	D		;SAVE REGS
	PUSH	B
	MOV	C,M		;C = FUNCTION CODE
	INX	H		;H/L = RETURN ADR
	PUSH	H		;  SAVE IT
	CALL	BDOS		;DO CPM FUNCTION
	POP	H		;RESTORE RETURN ADR
	POP	B		;RESTORE REGS
	POP	D
	XTHL			;STACK = RET ADR
	ORA	A		;SET 8080 FLAGS
	RET




;	---READ CONSOLE BUFFER---
;
;   READS A STRING OF ASCII CHARACTERS. LOWER CASE CHARACTERS ARE
;   TRANSLATED TO UPPER CASE. USES THE CPM 'READ BUFFER' FUNCTION.
;
;
;	ENTRY:	A = INPUT CHARACTER COUNT
;
;	EXIT:	A = FIRST CHARACTER IN THE STRING
;	      D/E = BUFFER ADDRESS OF THE FIRST CHARACTER
;
;
;   ERRORS - - IF AN ERROR OCCURES, 'RCB' IS ABORTED & THE
;	       PROGRAM RESUMES RUNNING AT 'LOOP'.
;
;
RCB:	LXI	D,RDBFR		;READ BFR
	INR	A		;FOR N+1 CHARS
	STAX	D		;SET CHAR COUNT
	MOV	B,A		;..SAVE THE COUNT
	CPM	F@RCB		;READ THE STRING
	CALL	CRLF		;DO CR.LF
	LXI	H,RDBFR+1
	MOV	A,M		;A = INPUT CHAR COUNT
	ORA	A
	JZ	INV		;ERROR, NO CHARS INPUT
	CMP	B		;B = MAX CHAR CNT +1
	JNC	INV		;ERROR, TOO MANY INPUT CHARS
; CONVERT L/C TO U/C
	MOV	B,A		;B = # CHARS TO CONVERT
RCB1:	INX	H		;INCR BFR ADR
	MOV	A,M
	CALL	UCT		;CONVERT
	MOV	M,A
	DCR	B
	JNZ	RCB1
	LXI	D,RDBFR+2	;D/E = ADR OF 1ST CHAR IN STRING
	LDAX	D		;..GET IT
	RET




;	---STOP TEST---
;
;   TEST FOR "ESC" KEY. IF FOUND, ABORT VERIFY.
;
;
STOP:	CPM	F@REQ		;GET CONSOLE STATUS
	RZ			;RIF DATA NOT AVAIL
	CPM	F@CI		;GET INPUT CHAR
	ANI	7FH		;KILL PARITY BIT
	CPI	ESC		;"ESC" KEY ?
	RNZ			;NO, EXIT
	PRNT	MSG8		; 'VERIFY ABORTED'
	CALL	PCA		;PRINT CYL ADR
	CALL	PHN		;PRINT HEAD #
	CALL	CRLF		;DO CR.LF
	CALL	STZ		;MOVE HEADS BACK TO TRACK 00
	JMP	LOOP		;ABORT, DO NEXT CMD




;	---BUILD SECTOR ADDRESS TABLE---
;
;
BST:	LXI	H,SECTBL	;SECTOR TABLE 
	MVI	B,MAXSEC+2
	MVI	A,-1		;A = SLOT EMPTY FLAG
BST1:	MOV	M,A		;PRE-FILL TABLE
	INX	H
	DCR	B
	JNZ	BST1
;
	LXI	D,0		;INIT TABLE INDEX
	MVI	B,0		;B = REAL SECTOR ADR
BST2:	LXI	H,SECTBL	;BASE ADR OF SECT TBL
	DAD	D		;INDEX INTO TABLE
	MOV	A,M		;A = TABLE ENTRY
	INR	A		;TEST FOR EMPTY SLOT
	JZ	BST3		;JIF STUFF TBL ENTRY
	INR	E		;INCR TBL INDEX
	MOV	A,E
	ANI	1FH		;KILL UNWANTED BITS
	MOV	E,A
	JMP	BST2		;TRY NEXT TABL ENTRY
;
BST3:	MOV	M,B		;PUT SECT ADR INTO TABLE SLOT
	INR	B		;INCR SECTOR ADR
	MOV	A,B
	CPI	MAXSEC		;ARE WE DONE ?
	RNC			;YES, EXIT
	MVI	A,SKEW
	ADD	E		;ADD SKEW FACTOR TO TBL INDEX
	ANI	1FH		;KILL UNWANTED BITS
	MOV	E,A
	JMP	BST2		;DO NEXT TBL ENTRY




;	---TURN DRIVE/CONTROLLER OFF---
;
;
COFF:	XRA	A
	OUT	CTCSR		;STOP CTLR
	OUT	DRCSR		;TURN OFF INTERFACE
	OUT	EXTCMD		;DE-SELECT THE DRIVE
	RET




;	---CONVERT---
;
;   CONVERTS AN ASCII CHARACTER STRING TO BINARY.
;
;   EXIT:	H/L = BINARY DATA
;
;
;   ERROR:	IF A CONVERSION ERROR IS DETECTED 'CONV' IS
;		ABORTED & THE PROGRAM RESUMES RUNNING AT
;		'LOOP'.
;
;
;
CONV:	LXI	H,RDBFR+1	;READ BFR
	MOV	B,M		;B = INPUT CHAR COUNT
	INX	H
	XCHG
	LXI	H,0
;
CNV1:	LDAX	D		;GET ASCII CHAR
	INX	D		;POINT TO NEXT CHAR IN STRING
	CALL	HTB		;CONVERT TO BINART
	JC	INV		;JIF FOUND NON-HEX CHAR
	DAD	H		;SHIFT H/L LEFT 4   (1 DIGIT)
	DAD	H
	DAD	H
	DAD	H
	ORA	L		; 'OR' IN NEW DIGIT
	MOV	L,A
	DCR	B		;DECR COUNT
	JNZ	CNV1		;JIF NOT DONE
	RET




;	---HEX/BINARY CONVERSION---
;
;   CONVERTS AN ASCII CHARACTER (IN 'A') TO A BINARY DIGIT
;   (RETURNED IN 'A'). THE CARRY IS SET (CY=1) IF A NON-HEX 
;   CHARACTER IS ENCOUNTERED.
;
;
HTB:	ANI	7FH		;KILL PARITY BIT
	SUI	'0'
	RC			;ERROR, CHAR < 0
	CPI	9+1
	CMC
	RNC			;RIF CHAR 0-9
	SUI	7		;MAKES CHARS A-F
	CPI	0AH
	RC			;RIF INVALID CHAR
	CPI	10H		;CHAR > 'F' ?
	CMC
	RET




;	---BINARY/HEX CONVERSION---
;
;   CONVERTS BINARY DATA (IN 'A') TO 2 ASCII
;   HEX CHARACTERS & PRINTS THOSE CHARACTERS.
;
;
BTH:	PUSH	D		;SAVE D/E
	PUSH	PSW
	RRC			;MOVE MSD TO LSD
	RRC
	RRC
	RRC
	CALL	BTH1		;CONVERT MSD & PRINT IT
	POP	PSW
	CALL	BTH1		;  * LSD
	POP	D		;RESTORE D/E
	RET
;
BTH1:	ANI	0FH		;KILL MSD
	ADI	'0'
	CPI	'9'+1
	JC	BTH2		;JIF CHAR 0-9
	ADI	7		;CHAR A-F
BTH2:	MOV	E,A
	CPM	F@CO		;PRINT THE DIGIT
	RET




;	---UPPER CASE TRANSLATION---
;
;   CONVERTS LOWER CASE CHARACTERS TO UPPER CASE.
;
;
UCT:	ANI	7FH		;KILL PARITY BIT
	CPI	61H
	RC			;RIF NOT L/C CHAR
	CPI	7BH	
	RNC			;RIF NOT L/C CHAR
	SUI	20H		;DO CONVERSION
	RET




;	---16 BIT SUBTRACTION---
;
;
SUBT:	MOV	A,L		; (H/L) - (D/E)
	SUB	E
	MOV	L,A
	MOV	A,H
	SBB	D
	MOV	H,A
	ORA	L		;TEST RESULTS FOR ZERO
	RET




;	---TEST 16 BIT VALUE---
;
;
TSUB:	MOV	A,L		; (H/L) - (D/E)
	SUB	E
	MOV	A,H
	SBB	D
	RET




;	---SOFT READ ERROR---
;
;
SOFT:	PRNT	MSG20		; 'SOFT ERROR'
	CALL	PDA		;PRINT DISK ADR
	PRNT	MSG22		; 'RETRIES='
	LDA	RTC
	CALL	BTH1		;PRINT RETRY COUNT
	CALL	CRLF		;DO CR.LF
	JMP	STOP		;STOP TEST, EXIT




;	---HARD READ ERROR---
;
;
HARD:	PRNT	MSG21		; 'HARD ERROR'
	CALL	PDA		;PRINT DISK ADDRESS
	PRNT	MSG23		; 'ERROR='
	LXI	D,MSG25		; 'RNF'
	LDA	CSTAT		;CONTROLLER STATUS
	ANI	2
	JNZ	HARD1		;JIF HAD RECORD NOT FOUND
	LXI	D,MSG26		;HAD CRC ERROR
HARD1:	CPM	F@PRNT		;PRINT TYPE OF ERROR (RNF/CRC)
	JMP	STOP		;STOP TEST, EXIT




;	---PRINT DISK ADDRESS---
;
;
PDA:	CALL	PCA		;PRINT CYL ADR
	CALL	PHN		;..HEAD #
	JMP	PSA		;..SECT ADR, EXIT





;	---PRINT CYLINDER ADDRESS---
;
;
PCA:	PRNT	MSG5		; 'CYL='
	LHLD	CYL
; PRINT 3 DIGITS
P3D:	MOV	A,H
	CALL	BTH1		;PRINT MS DIGIT OF CYL ADR
	MOV	A,L
	JMP	BTH		;LAST 2 DIGITS, EXIT




;	---PRINT HEAD NUMBER---
;
;
PHN:	PRNT	MSG6		; 'HD='
	LDA	HEAD
	JMP	BTH1		;PRINT HEAD #, EXIT




;	---PRINT SECTOR ADDRESS---
;
;
PSA:	PRNT	MSG7		; 'SECT='
	LDA	SECADR
	JMP	BTH		;PRINT ADR, EXIT




;	---CARRIAGE RETURN, LINE FEED---
;
;
CRLF:	PUSH	D		;SAVE D/E
	MVI	E,CR
	CPM	F@CO
	MVI	E,LF
	CPM	F@CO
	POP	D		;RESTORE D/E
	RET




;	---MESSAGES---
;
;
MSG1:	DB	CR,LF,LF,LF
	DB	'MHVER -- Vers 1.0',CR,LF
	DB	'13-Apr-83',CR,LF,'$'
;
MSG2:	DB	CR,LF,LF
	DB	'Select one of the following Options ---'
	DB	CR,LF,LF
	DB	'  C = Alter Cylinder Limits',CR,LF
	DB	'  H = Alter Head Limits',CR,LF
	DB	'  S = Start Verification',CR,LF
	DB	'  X = Exit to CPM',CR,LF,LF
	DB	'Heads =   $'
MSG2A:	DB	CR,LF,' Cyls = $'
MSG2B:	DB	CR,LF,LF,'Option? $'
;
MSG3:	DB	BELL,CR,LF
	DB	'* Invalid Input'
	DB	CR,LF,'$'
;
MSG4:	DB	BELL,CR,LF,LF,'* Restore Failed  $'
;
MSG5:	DB	'  Cyl=$'
;
MSG6:	DB	'  Hd=$'
;
MSG7:	DB	'  Sect=$'
;
MSG8:	DB	BELL,CR,LF,LF,'* Verify Aborted!  $'
;
MSG9:	DB	CR,LF,LF,'Verify Finished',CR,LF,'$'
;
MSG10:	DB	BELL,CR,LF,LF
	DB	'* Drive not Ready',CR,LF,'$'

MSG11:	DB	CR,LF,'Starting Head? $'
;
MSG12:	DB	CR,LF,'Ending Head? $'
;
MSG13:	DB	CR,LF,LF,BELL
	DB	'* Error -- Starting Head > Ending Head'
	DB	CR,LF,'$'
;
MSG14:	DB	CR,LF,'Starting Cylinder? $'
;
MSG15:	DB	CR,LF,'Ending Cylinder? $'
;
MSG16:	DB	CR,LF,LF,BELL
	DB	'* Error -- Starting Cylinder > Ending Cylinder'
	DB	CR,LF,'$'
;
MSG17:	DB	CR,'Cyl=$'
;
MSG18:	DB	BELL,CR,LF,LF
	DB	'* Seek Failed to Finish (Time-Out Error)'
	DB	CR,LF,'$'
;
MSG19:	DB	BELL,CR,LF,LF
	DB	'* Read Failed to Finish (Time-Out Error)'
	DB	CR,LF,'$'
;
MSG20:	DB	CR,'* Soft Read Error$'
;
MSG21:	DB	CR,'* Hard Read Error$'
;
MSG22:	DB	'  Retries=$'
;
MSG23:	DB	'  Error=$'
;
MSG24:	DB	BELL,CR,LF,LF,'* Seek Failed -- Target Cyl = '
;
MSG25:	DB	'RNF',CR,LF,'$'
;
MSG26:	DB	'CRC',CR,LF,'$'




;	---STORAGE---
;
;
CYL:	DS	2		;CYLINDER ADDRESS
HEAD:	DS	1		;HEAD NUMBER
SECADR:	DS	1		;SECTOR ADDRESS
ERRORS:	DS	1		;ERROR FLAG
RTC:	DS	1		;RETRY COUNT
SKRTC:	DS	1		;SEEK RETRY COUNT
VSRTC:	DS	1		;SEEK VERIFY RETRY COUNT
CSTAT:	DS	1		;CONTROLLER STATUS
DSTAT:	DS	1		;DRIVE STATUS
CHA:	DS	1		;COMPOSITE HEAD ADDRESS
CCA:	DS	2		;CURRENT CYLINDER ADDRESS
TBLADR:	DS	2		;SECTOR TABLE ADDRESS
SECTBL:	DS	MAXSEC+2	;SECTOR ADDRESS TABLE
SKDIR:	DS	1		;SEEK DIRECTION
RSKNT:	DS	2		;RESIDUAL SEEK COUNT
;
BGNHED:	DB	0		;STARTING HEAD
ENDHED:	DB	MAXHD		;ENDING HEAD
BGNCYL:	DW	0		;STARTING CYLINDER
ENDCYL:	DW	MAXCYL		;ENDING CYLINDER
TEMP:	DS	2		;TEMP STORAGE
;
RDBFR:	DS	10		;CONSOLE READ BUFFER


LAST	EQU	$		;DEFINES END OF PROGRAM



	END
