TITLE	'DIAGNOSTIC PROGRAM TO READ AND WRITE SECTORS TO HARD DISK'
;REMEMBER ON HARD DISK SECTORS ARE NUMBERED 0 TO 31
;
;
ESC	EQU	1BH
BDOS	EQU	5
REBOOT	EQU	0
TAB	EQU	9
BELL	EQU	07H
CR	EQU	0DH
LF	EQU	0AH
MAXCYL	EQU	500			;<---WILL ALLOW A MAX OF ONLY 8 MBYTES
MAXSEC	EQU	31			;0....31 OR 1....32
LZONE	EQU	656
;
;   DRIVE COMMANDS
;
SKCMD	EQU	3		;SEEK
SKOUT	EQU	1		;SEEK OUTWARD
NOPC	EQU	40H		;NO PRE-COMPENSATION
LOWRT	EQU	80H		;LOW WRITE CURRENT
VSA	EQU	8		;SEEK VERIFY START ADDRESS
VCA	EQU	1BH		;SEEK VERIFY COMMAND ADDRESS
;
;   DRIVE STATUS
;
READY	EQU	1		;DRIVE READY
WRTFLT	EQU	2		;WRITE FAULT
TK00	EQU	4		;TRACK ZERO
RAWINDX	EQU	20H		;RAW INDEX
;
;   CONTROLLER COMMANDS
;
BANK0	EQU	0		;BANK 0 SELECT
BANK1	EQU	1		;BANK 1 SELECT
DBENB	EQU	2		;DATA BUFFER ENABLE
CBENB	EQU	4		;COMPARE BUFFER ENABLE
STARTCMD EQU	8		;START COMMAND
;
;   DRIVE/CONTROLLER I/O
;
CBASE	EQU	70H		;BASE ADR OF THE CONTROLLER
DRCSR	EQU	CBASE		;DRIVE COMMAND/STATUS
EXTCMD	EQU	CBASE+1		;EXTENDED COMMNAND REGISTER
LOSC	EQU	CBASE+2		;SEEK COUNT, LSB
HISC	EQU	CBASE+3		;  * MSB
CTCSR	EQU	CBASE+4		;CONTROLLER COMMAND/STATUS
CTBFR	EQU	CBASE+5		;CONTROLLER BUFFER ADDRESS
CTDP	EQU	CBASE+6		;CONTROLLER DATA PORT


	; INCLUDE Z-80 MACRO LIBRARY:
	MACLIB	Z80

;
	ORG	100H
;
	LXI	SP,STACK
	LXI	H,SIGNON
	CALL	PRINT
;
START:	CALL	HDLOGIN			;SEND HEADS TO TRACK ZERO
START1:	LXI	H,RDORWR
	CALL	PRINT
	CALL	TI
	CPI	'R'
	JZ	GET
	CPI	'W'
	JZ	PUT
;
ERROR:	LXI	H,ERMSG
	CALL	PRINT
	JMP	START1
;
PUT:	XRA	A		;FLAG FOR READ/WRITE SECTORS
	STA	FLAG		;FLAG = 0 FOR WRITE TO DISK
	JMP	DOIT
;
GET	XRA	A
	DCR	A
	STA	FLAG		;FLAG = 0FF FOR READ FROM DISK
;
;====================== READ/WRITE  SECTORS TO/FROM DISK ====================
;
DOIT:	CALL	SETUP		
MORE:	LHLD	@TRK		;FIRST GET TRACK #
	LXI	D,1000
	CALL	TSUB		;HL - DE  (< 1000 TRACKS)
	RAL
	JC	GETX
	LXI	H,TRKERR
	CALL	PRINT
	JMP	START1
;
GETX:	LDA	@SECT		;NOW GET SECTOR #
	CPI	MAXSEC		;DISK NUMBERS 0 TO 31 LOGICAL 1 TO 32 
	JC	GET2
	LXI	H,SECERR
	CALL	PRINT
	JMP	START1
;
GET2:	LDA	FLAG		;READ OR WRITE
	ORA	A
	JZ	WRTS
	CALL	HREAD		;NOW READ ONE SECTOR
	JMP	DDD
;
WRTS:	CALL	HWRITE
DDD:	CALL	DISPLAY		;DISPLAY IT ON CRT
;
	LDA	COUNT		;SEE IF MORE SECTORS ARE REQUIRED
	DCR	A
	JZ	START1
	STA	COUNT
	LXI	H,WAITSTR
	CALL	PRINT
WAIT:	MVI	C,11
	CALL	BDOS
	ORA	A
	JZ	WAIT
	MVI	C,1
	CALL	BDOS
	LDA	@SECT
	INR	A
	CPI	MAXSEC
	JNZ	STSEC
	LHLD	@TRK		;IS LAST SECTOR ON TRACK STEP IN ONE TRACK
	INX	H
	SHLD	@TRK
	MVI	A,1		;UPDATE SECTOR VALUE
STSEC:  STA	@SECT
	LHLD	@DMA
	LXI	B,256
	DAD	B
	SHLD	@DMA
	JMP	MORE
;
;
;------ GENERAL ROUTINE TO SETUP DISK AND BIOS FOR SECTOR TRANSFERR -----
;
SETUP:	LXI	H,GETTK
	CALL	PRINT
	CALL	GETHEX		;RETURNS WITH TRACK IN [HL]
	SHLD	@TRK		;STORE REQUESTED TRACK
	LXI	H,GETSEC
	CALL	PRINT
	CALL	GETHEX		;ACTUALLY ONLY [L] WILL CONTAIN DATA
	MOV	A,L
	STA	@SECT		;STORE REQUESTED SECTOR
	LXI	H,GTDMA
	CALL	PRINT
	CALL	GETHEX		;RETURNS WITH DMA ADDRESS IN [HL]
	SHLD	@DMA		;STORE DMA 
	LXI	H,GETCNT
	CALL	PRINT
	CALL	GETHEX		;ONLY [L] WILL CONTAIN DATA
	MOV	A,L
	STA	COUNT
	CALL	CRLF
	RET
;
;	---- SUPPORT ROUTINES ------
;
PRINT:	MOV	A,M
	CPI	'$'
	RZ
	MOV	C,A
	CALL	CO
	INX	H
	JMP	PRINT
;
CRLF:	MVI	C,CR
	CALL	CO
	MVI	C,LF
	CALL	CO
	RET
;
TI:	CALL	CI		;GET UPPERCASE CHARACTER FROM CONSOL
	CPI	CR
	RZ
	CPI	ESC
	JZ	FINISH
	CPI	'C'-40H
	JZ	FINISH
	CPI	'A'-1
	RC
	CPI	'z'+1
	RNC
	ANI	5FH
	RET
;
CI:	PUSH	BC
	PUSH	DE
	PUSH	HL
	MVI	C,1
	CALL	BDOS
	POP	HL
	POP	DE
	POP	BC
	RET
;
CO:	PUSH	BC
	PUSH	DE
	PUSH	HL
	MOV	E,C
	MVI	C,2
	CALL	BDOS
	POP	HL
	POP	DE
	POP	BC
	RET
;
;
FINISH:	LXI	H,FINMSG
	CALL	PRINT
	JMP	REBOOT
;
GETHEX:	LXI	H,0		;GET ONE OR TWO BYTES IN HL
EXO:	CALL	TI		;GET A CHARACTER
EXO1:	MOV	B,A		;SAVE IT		
	CALL	NIBBLE
	JC	NOTOK
	DAD	H		;SHIFT ACROSS WHAT IS THERE * 16
	DAD	H
	DAD	H
	DAD	H
	ORA	L
	MOV	L,A		;BRING IN NEXT CHARACTER
	JMP	EXO		
NOTOK:	MVI	A,CR
	CMP	B
	RZ			;TERMINATE EACH STRING OF CHARS WITH CR
	JMP	ERROR
;
NIBBLE:	SUI	'0'		;CONVERT ASCII TO HEX
	RC
	CPI	'G'-'0'		;> F
	CMC
	RC
	CPI	10
	CMC
	RNC
	SUI	'A'-'9'-1	;ADJUST
	CPI	0AH		;FILTER ":" THROUGH "@"
	RET
;
;
DISPLAY:LXI	H,L0		;CRLF TRACK=
	CALL	PRINT
	LHLD	@TRK
	MOV	A,H
	CALL	HEXOUT
	MOV	A,L
	CALL	HEXOUT
	LXI	H,L1
	CALL	PRINT		;SECTOR=
	LDA	@SECT
	CALL	HEXOUT	
	LXI	H,L2
	CALL	PRINT		;DMA=
	LHLD	@DMA
	MOV	A,H
	CALL	HEXOUT
	MOV	A,L
	CALL	HEXOUT
	CALL	CRLF
	CALL	CRLF
	MVI	B,0		;FOR BYTE COUNT
	INX	H		;SEEMS TO REQ THIS

;
MORED:	MOV	A,M
	CALL	HEXOUT
	MVI	C,' '
	CALL	CO
	INX	H
	DCR	B
	JZ	ASCII		;A JUMP TO ASCII WILL RETURN TO MAIN PROGRAM
	MOV	A,B
	ANI	0FH
	CZ	ASCII		;CRLF AFTER EVERY 16 CHARACTERS
	JMP	MORED
;
ASCII:	MVI	C,TAB		;NOW PRINT OUT THE ASCII CHARACTERS IN SECTOR
	CALL	CO
	XRA	A		;CLEAR CARRY
	LXI	D,16		;BACKUP 16 CHARACTERS
	DB	0EDH,52H	;SBC HL,DE
	MVI	C,16
ASCII1:	MOV	A,M
	ANI	7FH
	CPI	' '		;RANGE TEST
	JNC	ASCX
ASCY:	MVI	A,'.'		;REPLACE NON PRINTABLE CHARACTERS
ASCX:	CPI	7CH
	JNC	ASCY
	PUSH	B
	MOV	C,A
	CALL	CO
	POP	B
	INX	H
	DCR	C
	JNZ	ASCII1
	CALL	CRLF
	RET
;
HEXOUT:	PUSH	PSW
	RRC
	RRC
	RRC
	RRC
	CALL	HEX1
	POP	PSW
HEX1:	ANI	0FH
	ADI	90H
	DAA
	ACI	40H
	DAA
	MOV	C,A
	CALL	CO
	RET
;
;
;------	INITILIZE THE XCOMP HARD DISK CONTROLLER (MUST DOWNLOAD MICROCODE) ---
HDLOGIN:
	CALL	XTKZ	     	;RESTORE
	JNZ	RESERR
	XRA	A		;RETURN WITH NO ERROR
	RET

RESERR:	LXI	H,MSGH2		;RESTORE FAILED
	CALL	PRINT
	ORI	1
	RET
;
	;;;;; HDWRT
	; ROUTINE WRITES 1 SECTOR TO THE DISK:
;
HWRITE:	XRA	A
	STA	ERFLG
	CALL	GETDMA		;GET DMA ADR, SET POINTERS
	OUTIR
	CALL	XWRT
	LDA	ERFLG
	RET

XWRT:	LXI	H,WTBL		;GET COMMAND TABLE
	CALL	DORW		;EXICUTE WR/RD COMMANDS
	JMP	XR1

	;;;;; HDRD
	; ROUTINE READS 1 PHYSICAL SECTOR FROM THE DRIVE:
;
HREAD:	XRA	A
	STA	ERFLG	     	;CLEAR THE ERROR FLAG
	CALL	XREAD	     	;READ A BLOCK
	CALL	GETDMA	     	;GET DMA ADR, SET POINTERS
	NOP
	INIR		 	;BLOCK INPUT
	LDA	ERFLG	     	;ERROR FLAG
	RET
;
XREAD:	LXI	H,RTBL	     ;SEEK SECTOR COMMAND TABLE IS SENT TO CONTROLLER
	CALL	DORW	     ;SEND IT
XR1:	MVI	A,0
	RZ		     ;RIF READ/WRITE OK
	INR	A
	STA	ERFLG        ;SET ERROR FLAG
	RET	
;
DORW:	SHLD	CTA	     ;SAVE CMD TBL ADR
	CALL	XSEK	     ;SEEK TO NEW TRACK (IF REQUIRED)
	RNZ		     ;RIF SEEK FAILED
	CALL	XSEL	     ;HEAD SELECT
	LHLD	CTA
;
DO0:	MOV	A,M
	STA	SKRTC        ;SET RETRY COUNT
	INX	H
	MOV	A,M
	OUT	CTCSR	     ;ENB CMP BFR
	INX	H
	MOV	A,M
	OUT	CTBFR	     ;SET CMP BFR ADR
	INX	H
	SHLD	CTA	     ;SAVE CMD TBL ADR
;
	LXI	H,RCA	     ;REAL TK ADR
	MVI	B,3
DO1:	MOV	A,M	     
	OUT	CTDP	     ;PUT HDR INFO INTO CMP BFR
	INX	H
	DCR	B
	JNZ	DO1
	LDA	@SECT
	OUT	CTDP	     ;SET SECT ADR FOR COMPARE
;
DO2:	CALL	XRDY	     ;DRIVE READY ?
	RNZ		     ;  RIF NO
	LHLD	CTA          ;CMD TBL ADR
	MOV	A,M	     ;A = CNTL BANK
	INX	H
	MOV	B,A
	OUT	CTCSR	     ;SLCT CNTL BANK
	MOV	A,M
	OUT	CTBFR	     ;SET START ADR
	INX	H
	MOV	A,B
	ORI	STARTCMD
	OUT	CTCSR	     ;START R/W CMD
;
DO3:	CALL	WFD	     ;WAIT FOR READ/WRITE TO FINISH
	RC		     ;ABORT IF TIMEOUT
	ANA	M	     ;TEST CTLR STATUS (0=OK)
	MOV	B,A
	IN	DRCSR	     ;DRIVE STATUS
	ANI	WRTFLT
	CNZ	CLRDF        ;CIF CLEAR DRIVE FAULT
	ORA	B	     ;SET/CLEAR ERROR FLAG (0=OK)
	RZ		     ;RIF READ/WRITE OK
	LXI	H,SKRTC
	DCR	M	     ;DECR RETRY COUNT
	JNZ	DO2	     ;JIF RETRY READ/WRITE
;
;   SET ERROR FLAG
;
SEF:	MVI	A,1	     ;A = ERROR FLAG
	ORA	A	     ;SET 8080 FLAGS
	RET		     ;TAKE ERROR EXIT
;
;	---WAIT FOR DONE---
;
WFD:	PUSH	H
	LXI	H,0		;TIMEOUT DELAY COUNT
;
WFD1:	IN	CTCSR	     ;CTLR STATUS
	RRC	
	JC	WFD2	     ;WAIT FOR DONE
	DCX	H
	MOV	A,H
	ORA	L
	JNZ	WFD1
;
	OUT	CTCSR
	POP	H
	MVI	A,1
	ORA	A
	STC
	RET
;
WFD2:	POP	H
	IN	CTCSR	     ;GET NON-CHANGING STATUS
	MOV	B,A	
	XRA	A
	OUT	CTCSR	     ;STOP CTLR
	MOV	A,B
	RET	
;	
;   SETS THE CONTROLLER BUFFER ADDRESS TO THE CORRECT
;   STARTING POINT. ALSO SETS B=256 & H/L=@DMA
;
GETDMA:	LHLD	@DMA
	MVI	C,CTDP	     ;FOR BLOCK I/O
	MVI	B,0	     ;B = COUNT (256 BYTES)
	MVI	A,DBENB
	OUT	CTCSR	     ;ENB DATA BFR
	MVI	A,0
GET1:	OUT	CTBFR	     ;SET CTLR DATA BFR ADR
	RET	
;
;	---REZERO---
;
XTKZ:	LXI	H,0
	SHLD	RCA
	CALL	TZT		;TEST IF TRK 0
	RZ
;
	LXI	H,511		;#OF CYLINDERS WE CAN COUNT ON CONTROLLER
	CALL	RTZ		;SEEK OUT
	RC			;ABORT DRIVE NOT READY
	RZ			;IS AT 0
	LXI	H,LZONE+10-511
	CALL	RTZ		;TRY SECOND PUMP
	RC
	RZ
	JMP	SEF		;ABORT RESTORE FAILED
;
;	SEEK OUTWARD
;
RTZ:	CALL	XRDY
	STC
	RNZ
	MOV	A,L
	OUT	LOSC		;SET LSB OF SEEK COUNT
	MOV	A,H
	OUT	HISC		;SET MSB
	MVI	A,SKOUT
	OUT	EXTCMD		;SET SEEK DIRECTION OUTWARD
	MVI	A,SKCMD
	OUT	DRCSR		;ISSUE SEEK
	CALL	WSC
	RC
;
TZT:	IN	DRCSR		;GET DRIVE STATUS
	ANI	TK00
	XRI	TK00
	RET
;
;
;	---SEEK---
;
XSEK:	MVI	A,3
	STA	SKRTC	     ;SET SEEK RETRY COUNT
;
XSEK1:	LHLD	@TRK	     ;REQUESTED TRACK
	MVI	B,1
	CALL	DRS	     ;SHIFT H/L RIGHT ONCE 
	LXI	D,MAXCYL
	CALL	TSUB
	RAL
	JNC	SEF		;ABORT IF INVALID ADDRESS
XSEK2:	XCHG			;[DE] = NEW CYLINDER ADDRESS
	LHLD	RCA		;LOAD UP CURRENT REAL ADR
	XCHG
	SHLD	RCA		;SAVE NEW ADDRESS
	XCHG
	CALL	SUBT		;GET DIFFERENCE
	RZ			;RETURN IF SAME
	MVI	B,1		;DIR = OUT
	RAL
	JNC	XSEK3		;OK SEEK OUTWARD
;
	MVI	B,3		;SEEK INWARD
	MOV	A,L
	CMA			;MAKE SEEK POSITIVE
	MOV	L,A
	MOV	A,H
	CMA
	MOV	H,A
	INX	H
;
XSEK3:	MOV	A,B		;GO TO SEEKING INWARD
	STA	SKDIR
	LXI	D,512
	CALL	TSUB
	RAL
	JNC	XSEK4		;JIF DOUBLE PUMP IS REQ
	CALL	PSK		;DO PARTIAL SEEK
	RNZ
	JMP	XSEK5
;
XSEK4:	DCX	D		;[DE] =511
	CALL	SUBT		;[HL]-[DE]
	SHLD	RSKNT		;SAVE RESIDUAL COUNT
	XCHG
	CALL	PSK
	RNZ			;ABORT IF SEEK FAILED
	LHLD	RSKNT
	CALL	PSK		;SEND THE REST
	RNZ
;
;				;SEEK VERIFY
XSEK5:	MVI	A,3
	STA	VSRTC		;SET RETRY COUNT
	MVI	A,CBENB
	OUT	CTCSR		;ENABLE BANK ZERO CMP BFR
	MVI	A,VCA
	OUT	CTBFR		;SET CMP BFR ADR
	LHLD	RCA		;REAL (CURR) CYL ADR
	MOV	A,L
	OUT	CTDP		;SET CYL ADR, LSB
	MOV	A,H
	OUT	CTDP		;SET CYL ADR, MSB
XSEK6:	MVI	A,VSA
	OUT	CTBFR		;SET M/CODE START ADR
	MVI	A,STARTCMD
	OUT	CTCSR		;START VERIFY
	CALL	WFD		;WAIT FOR DONE
	ANI	0CH		;TEST CTLR STATUS
	RZ			;RIF VERIFY OK
	LXI	H,VSRTC
	DCR	M		;DECR RETRY COUNT
	JNZ	XSEK6		;JIF RETRY SEEK VERIFY
				; VERIFY FAILED
	CALL	XTKZ		;RESTORE
	LXI	H,SKRTC
	DCR	M		;DECR RETRY COUNT
	JNZ	XSEK1		;JIF RETRY SEEK
				; UNRECOVERABLE SEEK ERROR
	ORI	1		;SET ERROR FLAG
	RET			;ABORT
;
;   PARTIAL SEEK
;
PSK:	CALL	XRDY		;DRIVE READY ?
	RNZ			;ABORT, DRIVE NOT RDY
	MOV	A,L
	OUT	LOSC		;SET SEEK COUNT, LSB
	MOV	A,H
	OUT	HISC		;   * MSB
	LDA	SKDIR
	OUT	EXTCMD		;SET SEEK DIRECTION
	MVI	A,3
	OUT	DRCSR		;ISSUE SEEK CMD
;
;	 ---> FALL THRU TO 'WSC' <---
;
;	---WAIT FOR SEEK COMPLETE---
;
;
WSC:	PUSH	H		;SAVE REGS
	PUSH	B
	LXI	H,0		;TIME-OUT DELAY COUNT
	MVI	B,3		;..DITTO
;
WSC1:	IN	DRCSR		;DRIVE STATUS
	RAL
	JC	WSC2		;JIF SEEK DONE
	DCX	H		;DECR DELAY COUNT
	MOV	A,H
	ORA	L
	JNZ	WSC1		;JIF CON'T WAITING
	DCR	B
	JNZ	WSC1
				; TIME-OUT ERROR
	POP	B		;RESTORE REGS
	POP	H
	MVI	A,1
	ORA	A		;SET CPM ERROR FLAG
	STC			;SET INTERNAL ERROR FLAG
	RET
;
WSC2:	POP	B		;RESTORE REGS
	POP	H
	XRA	A		;SET FLAG = OK
	RET
;
;
;
;	---CLEAR DRIVE FAULT---
;
CLRDF:	XRA	A
	OUT	EXTCMD	     ;DE-SELECT THE DRIVE (FALL THRU TO 'XSEL' TO
			     ;RE-SELECT THE DRIVE)
;
;	---HEAD SELECT---
;
XSEL:	LDA	@TRK         ;REQUESTED TRACK
	ANI	1	     ;2 HEADS
	STA	RHD	     ;SAVE REAL HEAD #
	ADD	A	     ;SHIFT HEAD # LEFT TWICE FOR H/W
	ADD	A
	ORI	1	     ;TO MAINTAIN DRIVE SLCT
	OUT	EXTCMD	     ;SELECT HEAD 0 OR 1
	RET	
;
;	---DRIVE READY TEST---
;
XRDY:	IN	DRCSR	     ;DRIVE STATUS
	ANI	1	     ;DRIVE RDY BIT
	XRI	1	     ;  MAKE IT LO-TRUE
	RZ		     ;RIF DRIVE READY
	LXI	H,MSGH1	     ;ERROR MSG
	CALL	PRINT
	ORI	1	     ;SET ERROR FLAG
	RET	
;
;
;   SHIFTS THE CONTENTS OF H/L RIGHT 'N' TIMES.
;
DRS:	PUSH	PSW	     ;SAVE 'A'
DRS1:	MOV	A,H	     ;MSB
	ORA	A	     ;CLEAR CY
	RAR	
	MOV	H,A
	MOV	A,L	     ;LSB
	RAR	
	MOV	L,A
	DCR	B	     ;B = SHIFT COUNT
	JNZ	DRS1
	POP	PSW	     ;RESTORE 'A'
	RET	
;
SUBT:	MOV	A,L		;16 BIT SUBTRACTION
	SUB	E
	MOV	L,A
	MOV	A,H
	SBB	D
	MOV	H,A
	ORA	L
	MOV	A,H
	RET
;
TSUB:	PUSH	H		;SAVE [HL]
	CALL	SUBT
	POP	H
	RET
;
;------------------------- DATA STORAGE AREA -------------------------------
;
;	--- COMMAND TABLE TO SEEK SECTOR/TRACK ---
;
;
WTBL:	DB	5	     ;RETRY COUNT
	DB	5	     ;CMP BFR ENB
	DB	0E6H	     ;CMP BFR ADR
	DB	BANK1	     ;CNTL BANK
	DB	0D3H         ;START ADR
	DB	0EH	     ;STATUS MASK
;
RTBL:	DB	10	     ;RETRY COUNT
	DB	4	     ;CMP BFR ENB
	DB	0EAH	     ;CMP BFR ADR
	DB	BANK0	     ;CNTL BANK
	DB	0D7H	     ;START ADDRESS
	DB	0EH	     ;STATUS MASK
;
;
SIGNON		DB CR,LF,'DIAGNOSTIC PROGRAM (CPM3 TYPE) TO TEST HDISK',CR,LF
		DB 'Remember sectors (256 BYTES) are numbered 0 to 31 (1FH)'
		DB ' on a track)',CR,LF
		DB 'Also tracks are 0 to 1000 (3E8H)',CR,LF,'$'
RDORWR		DB CR,LF,LF,'TYPE "R" OR "W" TO READ OR WRITE TO DISK'
		DB CR,LF,'TYPE "ESC" OR ^C TO RETURN TO CP/M:-  $'
DRNRDY		DB CR,LF,'DRIVE NOT READY',CR,LF,7,'$'
DRFLT		DB CR,LF,'DRIVE HARDWARE FAULT',CR,LF,7,'$'
ERMSG		DB	'?',7H,CR,LF,'$'
TRKERR		DB	CR,LF,'INVALID TRACK REQUESTED $'
SECERR		DB	CR,LF,'INVALID SECTOR REQUESTED $'
GETTK		DB	CR,LF,'ENTER TRACK IN HEX:- $'
GETSEC		DB	CR,LF,'ENTER SECTOR IN HEX:- $'
GTDMA		DB	CR,LF,'ENTER DMA ADDRESS IN HEX:- $'
GETCNT		DB	CR,LF,'# OF SECTORS REQ IN HEX:- $'
L0		DB	1ah,CR,LF,'TRACK = $'
L1		DB	'      SECTOR = $'
L2		DB	'      DMA ADDRESS = $'
FINMSG		DB	'^C',CR,LF,LF,'$'
WAITSTR		DB	'Press any character to continue....$'
;
MSGH1:	DB	BELL,CR,LF,'Drive not Ready',0
MSGH2:	DB	BELL,CR,LF,'Restore Failed',0
;
RCA:	DS	2	     ;REAL CYLINDER ADDRESS
RHD:	DS	1	     ;  HEAD
RSA:	DS	1	     ;  SECTOR
;
CTA:	DS	2	     ;COMMAND TABLE ADDRESS
ERFLG:	DS	1	     ;ERROR FLAG
SKRTC:	DS	1		;SEEK RETRY COUNT
VSRTC:	DS	1		;SEEK VERIFY COUNT
SKDIR:	DS	1		;SEEK DIRECTION
RSKNT:	DS	2		;RESIDUAL SEEK COUNT
SPARE:	DS	2
;
;
@TRK:	DS	2		;REQUESTED TRACK
@SECT:	DS	1		;REQUESTED SECTOR
@DMA:	DS	2		;REQUESTED DMA ADDRESS
COUNT	DB	0
FLAG	DB	0
;
	DS	30
STACK:	EQU	$

